@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmode.c,v 1.6.4.7 2004/01/02 22:23:10 pwessel Exp $
+ *    $Id: blockmode.c,v 1.16 2004/04/25 09:10:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -29,7 +29,7 @@
    		mode coding.
    Modified:	3.3.5: 10-JUL-2000 PW: Added -L option
    Version:	3.4 01-MAR-2001 by PW, Replace -N with -F, and added -C
-   Version:	3.4.3
+   Version:	4 01-MAR-2003 by PW
 */
 
 #include "gmt.h"
@@ -79,16 +79,14 @@ main (int argc, char **argv)
 				case 'R':
 				case 'V':
 				case ':':
+				case 'b':
+				case 'f':
 				case '\0':
                                       error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
                                       break;
                               
 				/* Supplemental parameters */
                               
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
-					
 				case 'C':
 					mode_xy = FALSE;
 					use_center = TRUE;
@@ -96,8 +94,10 @@ main (int argc, char **argv)
 				case 'I':
 					GMT_getinc (&argv[i][2], &delta_x, &delta_y);
 					break;
-				case 'L':
-					GMT_geographic_in = TRUE;
+				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
+					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
 				case 'F':
@@ -132,7 +132,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "blockmode %s - Block averaging by mode estimates\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: blockmode [infile(s)] -I<xinc[m]>[/<yinc>[m]] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-L] [-Q] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
+		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-Q] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]]\n\t[-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -144,7 +144,6 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-C Output center of block and mode z-value  [Default is mode location (but see -Q)]\n");
 		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		fprintf (stderr, "\t-Q Quicker; get mode z and mean x,y.  [Default gets mode x, mode y, mode z.]\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W sets Weight options.  -WI reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z) Output.\n");
@@ -155,6 +154,8 @@ main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 3 (or 4 if -W is set).\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -171,28 +172,28 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error = TRUE;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
 	n_req = (weighted) ? 4 : 3;
-	if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = n_req;
-	if (GMT_io.binary[0] && n_req > GMT_io.ncol[0]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
+	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
 		error++;
 	}
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
-	GMT_setmode (1);
+	GMT_setmode (GMT_OUT);
 #endif
 
 	idx = 1.0 / delta_x;
@@ -219,7 +220,7 @@ main (int argc, char **argv)
 	
 	/* Read the input data  */
 	
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted;
+	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted;
 
 	if (n_files > 0)
 		nofile = FALSE;
@@ -234,7 +235,7 @@ main (int argc, char **argv)
 			fp = GMT_stdin;
 			done = TRUE;
 #ifdef SET_IO_MODE
-			GMT_setmode (0);
+			GMT_setmode (GMT_IN);
 #endif
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -244,11 +245,11 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) {
+		if (gmtdefs.io_header[GMT_IN]) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
 				GMT_fgets (buffer, BUFSIZ, fp);
 				buffer[strlen(buffer)-1] = 0;
-				if (first) (report && !(weighted)) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
+				if (first && gmtdefs.io_header[GMT_OUT]) (report && !(weighted)) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
 			}
 			first = FALSE;
 		}
