@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdraster.c,v 1.6.4.6 2002/11/20 01:14:26 pwessel Exp $
+ *	$Id: grdraster.c,v 1.15 2004/05/24 20:47:25 pwessel Exp $
  *
- *    Copyright (c) 1991-2002 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* grdraster.c -- read a rasterfile and extract a region as a grdfile.
@@ -22,7 +22,7 @@
  *				  grdraster.info which may hold the character
  *				  L (for Littleendian) or B (for Bigendian).
  *				  If present, and different from the endianness
- *				  of the current machine, byte-swapping will occurr.
+ *				  of the current machine, byte-swapping will occur.
  *		18 Oct 1999   PW: Replaced GMTSWAP with WORDS_BIGENDIAN as provided
  *				  by the new configure script.
  *		05 May 2000   PW: Added option -Z [-bo[s]] to write xyz to stdout instead
@@ -39,6 +39,7 @@ struct RASTER_INFO {
 	int	nanset;		/* True if raster uses nanflag to signal NaN  */
 	char	type;
 	BOOLEAN swap_me;	/* TRUE if data set need to be swapped */
+	BOOLEAN geo;		/* TRUE if we believe x/y is lon/lat, FALSE otherwise */
 };
 
 #if WORDS_BIGENDIAN == 0
@@ -93,7 +94,6 @@ main (int argc, char **argv)
 	}
 	project_info.region_supplied = FALSE;	/* Because () sets it to TRUE by using GMT_get_common_args to process file regions */
 
-
 	for (i = 1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -130,7 +130,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdraster %s - Extract a region from a raster and save in a grdfile\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdraster <file number> -R<west/east/south/north>[r] [-G<grdfilename>] [-I<dx>[m][/<dy>[m]]]\n");
-		fprintf (stderr, " [-bo[s][<n>]]\n");
+		fprintf (stderr, " [-bo[s]]\n");
 
 		fprintf (stderr, "\t<file number> (#) corresponds to one of these:\n\n");
 		fprintf (stderr, "#	Data Description	Unit	Coverage		Spacing	Registration\n");
@@ -201,7 +201,10 @@ main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+
+	GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = (myras.geo) ? GMT_IS_LON : GMT_IS_FLOAT;
+	GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = (myras.geo) ? GMT_IS_LAT : GMT_IS_FLOAT;
 
 	/* Everything looks OK so far.  If (incset) verify that it will work, else set it.  */
 	if (incset) {
@@ -242,7 +245,7 @@ main (int argc, char **argv)
 			fprintf (stderr, "%s: %s -> -R%d:%2.2d/%d:%2.2d/%d:%2.2d/%d:%2.2d\n", GMT_program, r_opt, w, wm, e, em, s, sm, n, nm);
 		}
 		else if (gmtdefs.verbose)
-			fprintf (stderr, "%s: %s -> -R%lg/%lg/%lg/%lg\n", GMT_program, r_opt, h.x_min, h.x_max, h.y_min, h.y_max);
+			fprintf (stderr, "%s: %s -> -R%g/%g/%g/%g\n", GMT_program, r_opt, h.x_min, h.x_max, h.y_min, h.y_max);
 	}
 	
 	/* Now Enforce that wesn will fit x_inc, y_inc.  Set nx, ny but reset later based on G or P  */
@@ -273,9 +276,9 @@ main (int argc, char **argv)
 		}
 		else {
 			if (project_info.region)
-				fprintf(stderr, "%s WARNING:  Region reset to -R%lg/%lg/%lg/%lg.\n", GMT_program, h.x_min, h.x_max, h.y_min, h.y_max);
+				fprintf(stderr, "%s WARNING:  Region reset to -R%g/%g/%g/%g.\n", GMT_program, h.x_min, h.x_max, h.y_min, h.y_max);
 			else
-				fprintf(stderr, "%s WARNING:  Region reset to -R%lg/%lg/%lg/%lgr.\n", GMT_program, h.x_min, h.y_min, h.x_max, h.y_max);
+				fprintf(stderr, "%s WARNING:  Region reset to -R%g/%g/%g/%gr.\n", GMT_program, h.x_min, h.y_min, h.x_max, h.y_max);
 		}
 		error = 0;
 	}
@@ -342,7 +345,7 @@ main (int argc, char **argv)
 	/* Now open file and do it:  */
 	
 	if ( (fp = fopen(myras.h.remark, "rb") ) == NULL) {
-		fprintf(stderr,"%s ERROR opening %s for read (File not found or permission problems).\n", GMT_program, myras.h.remark);
+		fprintf(stderr,"%s ERROR opening %s for read.\n", GMT_program, myras.h.remark);
 		exit (EXIT_FAILURE);
 	}
 	if (gmtdefs.verbose) fprintf(stderr, "%s:  Reading from raster %s\n", GMT_program, myras.h.remark);
@@ -425,6 +428,9 @@ main (int argc, char **argv)
 					GMT_free ((void *)grd);
 					exit (EXIT_FAILURE);
 				}
+#ifdef DEBUG
+				fprintf (stderr, "%s: Doing line %6.6d\r", GMT_program, j);
+#endif
 				switch (myras.type) {
 					case 'u':
 						convert_u_row(myras, floatrasrow, (unsigned char *)buffer);
@@ -472,7 +478,7 @@ main (int argc, char **argv)
 	fclose(fp);
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s:  Finished reading from %s\n", GMT_program, myras.h.remark);
-	if (gmtdefs.verbose) fprintf (stderr, "%s:  min max and # NaN found: %lg %lg %d\n", GMT_program, h.z_min, h.z_max, n_nan);
+	if (gmtdefs.verbose) fprintf (stderr, "%s:  min max and # NaN found: %g %g %d\n", GMT_program, h.z_min, h.z_max, n_nan);
 
 	if (n_nan == h.nx * h.ny) fprintf(stderr,"%s: WARNING - Your grd file is entirely full of NaNs.\n", GMT_program);
 
@@ -714,11 +720,22 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 		}
 		strncpy(buf, &rasinfo[nfound].h.command[i], (size_t)j-i);
 		buf[j-i]='\0';
+		if (strchr (buf, ':') || strchr (buf, 'W') || strchr (buf, 'E') || strchr (buf, 'S') || strchr (buf, 'N')) {
+			GMT_io.in_col_type[0] = GMT_IS_LON;
+			GMT_io.in_col_type[1] = GMT_IS_LAT;
+		}
+		else {
+			GMT_io.in_col_type[0] = GMT_IS_FLOAT;
+			GMT_io.in_col_type[1] = GMT_IS_FLOAT;
+		}
+			
 		if (GMT_get_common_args(buf, &rasinfo[nfound].h.x_min, &rasinfo[nfound].h.x_max, &rasinfo[nfound].h.y_min, &rasinfo[nfound].h.y_max) ) {
 			fprintf(stderr,"%s: error reading grdraster.info\n", GMT_program);
 			fprintf(stderr,"\t-R string conversion error.\n");
 			return(0);
 		}
+		rasinfo[nfound].geo = (fabs (rasinfo[nfound].h.x_min) > 360.0 || fabs (rasinfo[nfound].h.x_max) > 360.0 || fabs (rasinfo[nfound].h.y_min) > 90.0 || fabs (rasinfo[nfound].h.y_max) > 90.0) ? FALSE : TRUE;
+		GMT_processed_option[8] = FALSE;
 		/* Now find the -I string:  */
 		i = j+1;
 		while (i < length && (rasinfo[nfound].h.command[i] != '-') ) i++;
