@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtdefaults.c,v 1.3.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: gmtdefaults.c,v 1.9 2004/01/02 22:45:13 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -21,7 +21,7 @@
  *
  * Author:	Paul Wessel
  * Date:	12-JUL-2000
- * Version:	3.4.3
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -36,6 +36,8 @@ main (int argc, char **argv)
 	
 	for (i = strlen(argv[0]); i >= 0 && argv[0][i] != '/'; i--);
 	GMT_program = &argv[0][i+1];	/* Name without full path */
+	GMT_set_home ();
+	GMT_init_fonts (&N_FONTS);
 
 	GMT_stdout = stdout;	GMT_stdin = stdin;
 
@@ -73,7 +75,7 @@ main (int argc, char **argv)
 	}
 	
 	if (argc == 1 || GMT_quick) {
-		fprintf (stderr, "gmtdefaults %s - List GMT-SYSTEM default parameters\n\n", GMT_VERSION);
+		fprintf (stderr, "gmtdefaults %s - List GMT-SYSTEM default parameters to stdout\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtdefaults [-D[s|u] | -L]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
 		fprintf (stderr, "\t-D prints the default settings for the GMT system\n");
