@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.3.4.8 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: grdmask.c,v 1.17 2004/04/25 09:10:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -30,7 +30,7 @@
  *				Also added -L for geographic data
  * 		PW: 13-JUL-2000, 3.3.5
  *		PW: 20-AUG-2002, Added -A and do fix_up path as in psxy
- * Version:	3.4.3
+ * Version:	4
  */
  
 #include "gmt.h"
@@ -44,7 +44,7 @@ main (int argc, char **argv)
 	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, polar;
 	
 	BOOLEAN	error = FALSE, done, nofile = TRUE, pixel = FALSE, draw_arc = TRUE, greenwich;
-	BOOLEAN	do_coverage = FALSE, map_units = FALSE, resample = FALSE;
+	BOOLEAN	do_coverage = FALSE, map_units = FALSE, periodic = FALSE, resample = FALSE;
 	
 	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, offset, dx, dy, r, ca, sa, lon_sum, dlon;
 	double xinc2, yinc2, distance, radius = 0.0, shrink, km_pr_deg, idx, idy, x0, y0, *in, step = 1.0;
@@ -76,15 +76,14 @@ main (int argc, char **argv)
 				case 'R':
 				case 'V':
 				case ':':
+				case 'b':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
 				
 				/* Supplemental parameters */
 				
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'A':	/* Turn off draw_arc mode */
 					if (argv[i][2])
 						step = atof (&argv[i][2]);	/* Undocumented test feature */
@@ -108,8 +107,10 @@ main (int argc, char **argv)
 				case 'I':
 					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
 					break;
-				case 'L':
-					GMT_geographic_in = TRUE;
+				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
+					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
@@ -132,7 +133,8 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdmask %s - Create mask grdfile from polygons or point coverage\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> -I<xinc[m|c]>[/<yinc>[m|c]]\n");
-		fprintf (stderr, "\t-R<west/east/south/north> [-A] [-F] [-H[<nrec>]] [-L] [-N<out>/<edge>/<in>] [-M[<flag>]] [-S<radius>[k] [-V] [-:] [-bi[s][<n>]]\n\n");
+		fprintf (stderr, "\t-R<west/east/south/north> [-A] [-F] [-H[<nrec>]] [-N<out>/<edge>/<in>] [-M[<flag>]]\n");
+		fprintf (stderr, "\t[-S<radius>[k] [-V] [-:] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -145,7 +147,6 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs for geographic data\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets values to use if point is outside, on the path, or inside.\n");
 		fprintf (stderr, "\t   NaN is a valid entry.  Default values are 0/0/1.\n");
@@ -158,6 +159,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t    [Default is 2 input columns].\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -174,23 +176,23 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G:  Must specify output file\n", GMT_program);
 		error = TRUE;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 2) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 2) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	GMT_grd_RI_verify (&header, 1);
@@ -227,7 +229,7 @@ main (int argc, char **argv)
 	}
 	
 	if (map_units) {
-		km_pr_deg = 0.001 * 2.0 * M_PI * gmtdefs.ellipse[gmtdefs.ellipsoid].eq_radius / 360.0;
+		km_pr_deg = 0.001 * 2.0 * M_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
 		shrink = cos (0.5 * (header.y_min + header.y_max) * D2R);
 		di = (int)ceil (radius / (km_pr_deg * header.x_inc * shrink));
 		dj = (int)ceil (radius / (km_pr_deg * header.y_inc));
@@ -236,7 +238,8 @@ main (int argc, char **argv)
 		di = (int)ceil (radius * idx);
 		dj = (int)ceil (radius * idy);
 	}
-	resample = (draw_arc && GMT_geographic_in);
+	periodic = (GMT_io.in_col_type[0] == GMT_IS_LON);	/* Dealing with geographic coordinates */
+	resample = (draw_arc && periodic);
 	greenwich = (header.x_min <= 0.0 || header.x_max <= 0.0);
 	
 	/* Initialize all nodes to the 'outside' value */
@@ -250,7 +253,7 @@ main (int argc, char **argv)
 	n_args = (argc > 1) ? argc : 2;
 	
 	done = FALSE;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 	
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -258,7 +261,7 @@ main (int argc, char **argv)
 			fp = GMT_stdin;
 			done = TRUE;
 #ifdef SET_IO_MODE
-			GMT_setmode (0);
+			GMT_setmode (GMT_IN);
 #endif
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -268,7 +271,7 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 		
 		n_read = 0;
 		n_fields = GMT_input (fp,  &n_expected_fields, &in);
@@ -294,7 +297,7 @@ main (int argc, char **argv)
 				if (x[n_path] < xmin) xmin = x[n_path];
 				if (y[n_path] > ymax) ymax = y[n_path];
 				if (y[n_path] < ymin) ymin = y[n_path];
-				if (GMT_geographic_in && n_path > 0) {	/* Do longitude-difference check sum */
+				if (periodic && n_path > 0) {	/* Do longitude-difference check sum */
 					dlon = x[n_path] - x[n_path-1];
 					if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 					lon_sum += dlon;
@@ -313,17 +316,19 @@ main (int argc, char **argv)
 				for (k = 0; k < n_path; k++) {
 
 					if (y[k] < header.y_min || y[k] > header.y_max) continue;
-					if (GMT_geographic_in) {	/* Make sure we are not off by 360 somewhere */
+					if (periodic) {	/* Make sure we are not off by 360 somewhere */
 						x[k] -= 360.0;
 						while (x[k] < header.x_min) x[k] += 360.0;
 						if (x[k] > header.x_max) continue;
 					}
 					else if (x[k] < header.x_min || x[k] > header.x_max) continue;
 					
-					/* OK, this point is within bounds */
+					/* OK, this point is within bounds, but may be exactly on the border */
 					
 					i0 = (int)floor (((x[k] - header.x_min) * idx) + offset);
+					if (i0 == header.nx) i0--;	/* Was exactly on the xmax edge */
 					j0 = (int)floor (((header.y_max - y[k]) * idy) + offset);
+					if (j0 == header.ny) j0--;	/* Was exactly on the ymin edge */
 					data[j0 * header.nx + i0] = out_edge_in[2];	/* This is the nearest node */
 					if (radius == 0.0) continue;
 
@@ -343,12 +348,13 @@ main (int argc, char **argv)
 				}
 			}
 			else {	/* assign 'inside' to nodes if they are inside given polygon */
+
 				if (x[n_path-1] != x[0] || y[n_path-1] != y[0]) {	/* Explicitly close the polygon */
 					x[n_path] = x[0];
 					y[n_path] = y[0];
 					dlon = x[n_path] - x[n_path-1];
 					n_path++;
-					if (GMT_geographic_in) {
+					if (periodic) {
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 						lon_sum += dlon;
 					}
@@ -356,7 +362,7 @@ main (int argc, char **argv)
 				
 				if (resample) n_path = GMT_fix_up_path (&x, &y, n_path, greenwich, step);
 				
-				if (GMT_geographic_in && fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
+				if (periodic && fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
 					polar = (y[0] < 0.0) ? -1 : +1;	/* S or N pole */
 					for (j = 0; j < n_path; j++) {
 						r = 90.0 - fabs (y[j]);
@@ -375,7 +381,7 @@ main (int argc, char **argv)
 					for (i = 0; i < header.nx; i++) {
 						xx = header.x_min + i * header.x_inc;
 						if (pixel) xx += xinc2;
-						if (GMT_geographic_in) {	/* Make sure we are not off by 360 somewhere */
+						if (periodic) {	/* Make sure we are not off by 360 somewhere */
 							xx -= 360.0;
 							while (xx < xmin) xx += 360.0;
 						}
