@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdreformat.c,v 1.2.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: grdreformat.c,v 1.8 2004/01/02 22:45:13 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -19,8 +19,8 @@
  * grdreformat.c reads a grd file in one format and outputs it in another
  *
  * Author:	Paul Wessel
- * Date:	3-JAN-1991-2000
- * Version:	3.4.3
+ * Date:	3-JAN-1991-2004
+ * Version:	4
  */
 
 #include "gmt.h"
@@ -29,7 +29,7 @@ main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, global = FALSE;
 
-	int i, nfiles = 0, nm, nx, ny, one_or_zero;
+	int i, nfiles = 0, nm, nx, ny, one_or_zero, no_header = 0;
 
 	double w, e, s, n, scale = 1.0, offset = 0.0;
 
@@ -58,6 +58,9 @@ main (int argc, char **argv)
 					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
 					break;
 					
+				case 'N':
+					no_header = 64;
+					break;
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -74,7 +77,7 @@ main (int argc, char **argv)
 	
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr, "grdreformat %s - Converting between different grdfile formats\n\n", GMT_VERSION);
-		fprintf( stderr, "usage: grdreformat ingrdfile[=id[/scale/offset]] outgrdfile[=id[/scale/offset]] [-Rw/e/s/n] [-V]\n");
+		fprintf( stderr, "usage: grdreformat ingrdfile[=id[/scale/offset]] outgrdfile[=id[/scale/offset]] [-N] [-Rw/e/s/n] [-V]\n");
 
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -82,6 +85,8 @@ main (int argc, char **argv)
 		fprintf (stderr, "	outgrdfile is the new converted grd file\n");
 		fprintf( stderr, "	   scale and offset, if given, will multiply data by scale and add offset.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-N  Do NOT write the header (native grids only - ignored otherwise).\n");
+		fprintf (stderr, "\t\t  Useful when creating files to be used by grdraster.\n");
 		GMT_explain_option ('r');
 		GMT_explain_option ('V');
 		
@@ -103,7 +108,7 @@ main (int argc, char **argv)
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 	
 	GMT_grd_i_format = GMT_grd_get_i_format (file[0], fname[0], &scale, &offset);
 	GMT_grd_o_format = GMT_grd_get_o_format (file[1], fname[1], &scale, &offset);
@@ -117,6 +122,7 @@ main (int argc, char **argv)
 		if (file[0][0] == '=') strcpy (fname[0], "<stdin>");
 		if (file[1][0] == '=') strcpy (fname[1], "<stdout>");
 		fprintf (stderr, "%s: Translating file %s (format = %d) to file %s (format = %d)\n", GMT_program, fname[0], GMT_grd_i_format, fname[1], GMT_grd_o_format);
+		if (no_header && !GRD_IS_CDF(GMT_grd_o_format)) fprintf (stderr, "%s: No grd header will be written\n", GMT_program);
 	}
 	
 	if (GMT_read_grd_info (file[0], &grd)) {
@@ -154,7 +160,7 @@ main (int argc, char **argv)
 		}
 	}
 
-	if (GMT_write_grd (file[1], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
+	if (GMT_write_grd (file[1], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, no_header)) {
 		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, fname[1]);
 		exit (EXIT_FAILURE);
 	}
