@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: makecpt.c,v 1.2.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: makecpt.c,v 1.11 2004/07/19 02:36:13 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -24,17 +24,18 @@
  * Default color table is still rainbow.
  *
  * Author:	Walter H.f. Smith & P. Wessel
- * Date:	04-JUL-2000
- * Version:	3.4.3
+ * Date:	22-SEP-2000
+ * Version:	4
  */
 
 #include "gmt.h"
 
 main(int argc, char **argv)
 {
-	int	i, nz;
+	int	i, nz, log_mode = 0;
 
-	BOOLEAN	error = FALSE, continuous = FALSE, reverse = FALSE, got_t = FALSE, got_tfile = FALSE, ok = FALSE;
+	BOOLEAN	error = FALSE, continuous = FALSE, reverse = FALSE, got_t = FALSE;
+	BOOLEAN got_tfile = FALSE, no_BFN = FALSE, ok = FALSE;
 
 	double	z_start, z_stop, z_inc, *z;
 
@@ -46,7 +47,7 @@ main(int argc, char **argv)
 
 	z_start = z_stop = z_inc = 0.0;
 
-	/* Get list of available color tables in GMT /share */
+	/* Get list of available color tables in $GMTHOME/share/cpt */
 
 	sprintf (CPT_lis, "%s%cshare%cGMT_CPT.lis", GMTHOME, DIR_DELIM, DIR_DELIM);
 
@@ -68,6 +69,9 @@ main(int argc, char **argv)
 				case 'I':
 					reverse = TRUE;
 					break;
+				case 'N':
+					no_BFN = TRUE;
+					break;
 				case 'T':
 					if (!access (&argv[i][2], R_OK)) {
 						got_tfile = TRUE;
@@ -78,6 +82,12 @@ main(int argc, char **argv)
 						sscanf (&argv[i][2], "%lf/%lf/%lf", &z_start, &z_stop, &z_inc);
 					}
 					break;
+				case 'Q':
+					if (argv[i][2] == 'o')	/* Input data is z, but take log10(z) before interpolation colors */
+						log_mode = 2;
+					else			/* Input is log10(z) */
+						log_mode = 1;
+					break;
 				case 'Z':
 					continuous = TRUE;
 					break;
@@ -90,22 +100,27 @@ main(int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_quick) {
-		fprintf(stderr,"makecpt %s - Make GMT color palette tables\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  makecpt [-C<table>] [-I] [-T<z0/z1/dz> | -T<file>] [-V] [-Z]\n\n");
+		fprintf (stderr, "makecpt %s - Make GMT color palette tables\n\n", GMT_VERSION);
+		fprintf (stderr, "usage:  makecpt [-C<table>] [-I] [-N] [-Q[i|o]] [-T<z0/z1/dz> | -T<file>] [-V] [-Z]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf(stderr,"\t-C Specify a colortable [Default is rainbow]:\n");
-		fprintf(stderr,"\t   [Default min/max values for -T are given in brackets]\n");
-		fprintf(stderr,"\t   ---------------------------------\n");
+		fprintf (stderr, "\t-C Specify a colortable [Default is rainbow]:\n");
+		fprintf (stderr, "\t   [Default min/max values for -T are given in brackets]\n");
+		fprintf (stderr, "\t   ---------------------------------\n");
 		while (fgets (buffer, BUFSIZ, fpc)) if (!(buffer[0] == '#' || buffer[0] == 0)) fprintf (stderr, "\t   %s", buffer);
 		fclose (fpc);
-		fprintf(stderr,"\t   ---------------------------------\n");
-		fprintf(stderr,"\t-I reverses the sense of the color table\n");
-		fprintf(stderr,"\t-T Give start, stop, and increment for colorscale in z-units, or filename with custom z-values\n");
-		fprintf(stderr,"\t   If not given, the range in the master cptfile is used\n");
+		fprintf (stderr, "\t   ---------------------------------\n");
+		fprintf (stderr, "\t-I reverses the sense of the color table\n");
+		fprintf (stderr, "\t-N Do Not write back-, fore-, and nan colors [Default will].\n");
+		fprintf (stderr, "\t-Q assign a logarithmic colortable [Default is linear]\n");
+		fprintf (stderr, "\t   -Qi: z-values are actually log10(z). Assign colors and write z. [Default]\n");
+		fprintf (stderr, "\t   -Qo: z-values are z, but take log10(z), assign colors and write z.\n");
+		fprintf (stderr, "\t        If -T<z0/z1/dz> is given, dz is 1, 2, or 3 (as in logarithmic anotations)\n");
+		fprintf (stderr, "\t-T Give start, stop, and increment for colorscale in z-units, or filename with custom z-values\n");
+		fprintf (stderr, "\t   If not given, the range in the master cptfile is used\n");
 		GMT_explain_option ('V');
-		fprintf(stderr,"\t-Z will create a continuous color palette.\n");
-		fprintf(stderr,"\t   [Default is discontinuous, i.e., constant color intervals]\n");
+		fprintf (stderr, "\t-Z will create a continuous color palette.\n");
+		fprintf (stderr, "\t   [Default is discontinuous, i.e., constant color intervals]\n");
 		exit (EXIT_FAILURE);
 	}
 
@@ -132,9 +147,9 @@ main(int argc, char **argv)
 
 	if (!ok)	{	/* No table in current dir, try /share */
 		if (table)
-			sprintf (CPT_file, "%s%cshare%cGMT_%s.cpt", GMTHOME, DIR_DELIM, DIR_DELIM, table);
+			sprintf (CPT_file, "%s%cshare%ccpt%cGMT_%s.cpt", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM, table);
 		else	/* Default to rainbow colors */
-			sprintf (CPT_file, "%s%cshare%cGMT_rainbow.cpt", GMTHOME, DIR_DELIM, DIR_DELIM);
+			sprintf (CPT_file, "%s%cshare%ccpt%cGMT_rainbow.cpt", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM);
 
 		ok = !access (CPT_file, R_OK);
 		if (!ok) {
@@ -154,7 +169,7 @@ main(int argc, char **argv)
 
 	/* OK, we can now do the resampling */
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 	GMT_read_cpt (CPT_file);
 
@@ -180,6 +195,17 @@ main(int argc, char **argv)
 		}
 		z = (double *) GMT_memory ((void *)z, (size_t)nz, sizeof(double), GMT_program);
 	}
+	else if (got_t && log_mode == 2) {	/* Establish a log10 grid */
+		if (!(z_inc == 1.0 || z_inc == 2.0 || z_inc == 3.0)) {
+			fprintf (stderr, "%s: Error: For -Qo logarithmic spacing, dz must be 1, 2, or 3\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+		if (z_start <= 0.0) {
+			fprintf (stderr, "%s: Error: For -Qo logarithmic spacing, z_start must be > 0\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+		nz = GMT_log_array (z_start, z_stop, z_inc, &z);
+	}
 	else if (got_t) {	/* Establish linear grid */
 		nz = irint ((z_stop - z_start) / z_inc) + 1;
 		z = (double *) GMT_memory (VNULL, (size_t)nz, sizeof(double), GMT_program);
@@ -193,13 +219,15 @@ main(int argc, char **argv)
 		z[i] = GMT_lut[i-1].z_high;
 	}
 	
+	if (log_mode == 2) for (i = 0; i < nz; i++) z[i] = d_log10 (z[i]);	/* Make log10(z) values for interpolation step */
+	
 	/* Write to GMT_stdout.  */
 
 	fprintf (GMT_stdout, "#\tcpt file created by: %s", GMT_program);
 	for (i = 1; i < argc; i++) fprintf (GMT_stdout, " %s", argv[i]);
 	fprintf (GMT_stdout, "\n");
 
-	GMT_sample_cpt (z, nz, continuous, reverse);
+	GMT_sample_cpt (z, nz, continuous, reverse, log_mode, no_BFN);
 
 	GMT_free ((void *)z);
 
