@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: pscoupe.c,v 1.3.4.1 2002/02/27 17:41:10 pwessel Exp $
+ *    $Id: pscoupe.c,v 1.13 2004/09/10 23:16:15 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -15,7 +15,7 @@ PostScript code is written to stdout.
  Author:       Genevieve Patau
  Date:         9 September 1992
  Last change : 02 April 2001
- Version:      3.4
+ Version:      4
  Roots:        based on psxy.c version 3.0
 
  */
@@ -83,11 +83,13 @@ main (int argc, char **argv)
     struct AXIS T, N, P, Tr, Nr, Pr;
 
     int n_plane = 0, n_plane_old;
-    int default_fontsize = 9, default_justify = 2;
-    int fontsize = default_fontsize, justify = default_justify, form = 0;
+    int default_justify = 2;
+    int justify = default_justify, form = 0;
 
+    double default_fontsize = 9.0;
+    double fontsize = default_fontsize;
     double default_offset = gmtdefs.measure_unit == 0 ? 0.1 : 0.04;
-    double offset = default_offset, angle = 0.;
+    double offset = default_offset, angle = 0.0;
     double size;
     double tmp1, tmp2, tmp3, tmp4, tmp5;
     double n_dep, distance;
@@ -215,11 +217,11 @@ main (int argc, char **argv)
                     scale = GMT_convert_units (txt_a, GMT_INCH);
 
                     if(strstr(argv[i], "/\0") != NULL) {
-                        sscanf(strstr(argv[i], "/\0"), "/%d/%lf",
+                        sscanf(strstr(argv[i], "/\0"), "/%lf/%lf",
                            &fontsize, &offset);
-                        if (fontsize == 0) fontsize = default_fontsize;
-                        if (fontsize < 0) no_label = TRUE;
-                        if (offset == 0.) offset = default_offset;
+                        if (fontsize == 0.0) fontsize = default_fontsize;
+                        if (fontsize < 0.0) no_label = TRUE;
+                        if (offset == 0.0) offset = default_offset;
                         if (argv[i][strlen(argv[i]) - 1] == 'u') justify = 10;
                     }
 
@@ -275,11 +277,11 @@ main (int argc, char **argv)
                     }
 
                     if(strstr(argv[i], "/\0") != NULL) {
-                        sscanf(strstr(argv[i], "/\0"), "/%d/%lf",
+                        sscanf(strstr(argv[i], "/\0"), "/%lf/%lf",
                            &fontsize, &offset);
-                        if (fontsize == 0) fontsize = default_fontsize;
-                        if (fontsize < 0) no_label = TRUE;
-                        if (offset == 0.) offset = default_offset;
+                        if (fontsize == 0.0) fontsize = default_fontsize;
+                        if (fontsize < 0.0) no_label = TRUE;
+                        if (offset == 0.0) offset = default_offset;
                         if (argv[i][strlen(argv[i]) - 1] == 'u') justify = 10;
                     }
 
@@ -472,7 +474,7 @@ main (int argc, char **argv)
             n_files++;
     }
     
-    /* Check that the options selected are mutually consistant */
+    /* Check that the options selected are mutually consistent */
     
     no_size_needed = (read_cmt || read_planes || read_aki || read_axis || read_tensor);
     error += GMT_check_rgb (pen.rgb)
@@ -554,8 +556,8 @@ main (int argc, char **argv)
         fprintf (stderr, "         (y) Best double couple defined main axis (Harvard CMT)\n");
         fprintf (stderr, "             X,Y,depth,T_value,T_azimuth,T_plunge,N_value,N_azimuth,N_plunge,\n");
         fprintf (stderr, "             P_value,P_azimuth,P_plunge,exp,event_title\n");
-        fprintf (stderr, "         Optionnally add /fontsize[/offset][u]\n");
-        fprintf (stderr, "      Default values are /%d/%lf\n", default_fontsize, default_offset);
+        fprintf (stderr, "         Optionally add /fontsize[/offset][u]\n");
+        fprintf (stderr, "      Default values are /%lg/%lf\n", default_fontsize, default_offset);
         fprintf (stderr, "      fontsize < 0 : no label written;\n");
         fprintf (stderr, "      offset is from the limit of the beach ball.\n");
         fprintf (stderr, "      By default label is above the beach ball. Add u to plot it under.\n");
@@ -592,7 +594,7 @@ main (int argc, char **argv)
         n_files = 1;
     n_args = (argc > 1) ? argc : 2;
     
-    GMT_put_history (argc, argv);   /* Update .gmtcommands */
+    GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
     if(frame_coupe) {
         west = 0.;
@@ -609,20 +611,20 @@ main (int argc, char **argv)
     ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, 
         gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale, 
         gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi, 
-        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, 
+        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, 
         GMT_epsinfo (argv[0]));
     GMT_echo_command (argc, argv);
 
     if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
     
     GMT_setpen (&pen);
-    ps_setfont (gmtdefs.anot_font);
+    ps_setfont (gmtdefs.annot_font[0]);
     
     if (skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
     
     old_GMT_world_map = GMT_world_map;
     
-    ix = (gmtdefs.xy_toggle);    iy = 1 - ix;
+    ix = (gmtdefs.xy_toggle[0]);    iy = 1 - ix;
 
     done = FALSE;
 
@@ -640,10 +642,10 @@ main (int argc, char **argv)
 
         if (!nofile && gmtdefs.verbose) {
             fprintf (stderr, "pscoupe: Working on file %s\n", argv[fno]);
-            sprintf (line, "File: %s", argv[fno]);
+            sprintf (line, "File: %s\0", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header) 
+        if (gmtdefs.io_header[GMT_IN]) 
             for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
         
 
@@ -1031,11 +1033,9 @@ Definition of scalar moment.
     ps_setpaint (gmtdefs.basemap_frame_rgb);
 
     if (pen.texture) ps_setdash (CNULL, 0);
-    if (frame_info.plot) {
-        ps_setpaint (gmtdefs.basemap_frame_rgb);
-        GMT_map_basemap ();
-        ps_setpaint (gmtdefs.background_rgb);
-    }
+
+    GMT_map_basemap ();
+
     ps_plotend (gmtdefs.last_page);
     
     GMT_end (argc, argv);
