@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: psmeca.c,v 1.4.4.1 2002/02/27 17:41:10 pwessel Exp $
+ *    $Id: psmeca.c,v 1.15 2004/09/10 23:16:15 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -14,7 +14,7 @@ PostScript code is written to stdout.
 
  Author:     Genevieve Patau
  Date:         7 July 1998
- Version:    3.3
+ Version:    4
  Roots:      based on psxy.c    
 
  Last change  18 February 2000
@@ -86,14 +86,16 @@ main (int argc, char **argv)
     struct AXIS T, N, P;
 
     int n_plane = 0, n_plane_old;
-    int default_fontsize = 9, default_justify = 2;
-    int fontsize = default_fontsize, justify = default_justify, form = 0;
+    int default_justify = 2;
+    int justify = default_justify, form = 0;
     int new = 1;
 
+    double default_fontsize = 9.0;
+    double fontsize = default_fontsize;
     double default_offset = gmtdefs.measure_unit == 0 ? 0.1 : 0.04;
     double default_pointsize = 0.005;
-    double pointsize = default_pointsize, offset = default_offset, angle = 0.;
-    double fault, depth, depmin = 0., depmax = 900.;
+    double pointsize = default_pointsize, offset = default_offset, angle = 0.0;
+    double fault, depth, depmin = 0.0, depmax = 900.0;
     double size, a_size;
     double P_x, P_y, T_x, T_y;
     char P_sym_type, T_sym_type;
@@ -207,12 +209,12 @@ main (int argc, char **argv)
                     n=0; while (txt_a[n] && txt_a[n] != '/') n++; txt_a[n]=0;
                     scale = GMT_convert_units (txt_a, GMT_INCH);
 
-                    if(strstr(argv[i], "/\0") != NULL) {
-                        sscanf(strstr(argv[i], "/\0"), "/%d/%lf",
+                    if(strstr(argv[i], "/") != NULL) {
+                        sscanf(strstr(argv[i], "/"), "/%lf/%lf",
                            &fontsize, &offset);
-                        if (fontsize == 0) fontsize = default_fontsize;
-                        if (fontsize < 0) no_label = TRUE;
-                        if (offset == 0.) offset = default_offset;
+                        if (fontsize == 0.0) fontsize = default_fontsize;
+                        if (fontsize < 0.0) no_label = TRUE;
+                        if (offset == 0.0) offset = default_offset;
                         if (argv[i][strlen(argv[i]) - 1] == 'u') justify = 10;
                     }
 
@@ -297,8 +299,8 @@ main (int argc, char **argv)
                         n=0; while (txt_a[n] && txt_a[n] != '/') n++; txt_a[n]=0;
                         a_size = GMT_convert_units (txt_a, GMT_INCH);
 
-                        if(strstr(argv[i], "/\0") != NULL) {
-                            strcpy(txt_a,strstr(argv[i], "/\0"));
+                        if(strstr(argv[i], "/") != NULL) {
+                            strcpy(txt_a,strstr(argv[i], "/"));
                             switch (strlen(txt_a)) {
                                 case 1:
                                     P_sym_type = 'c';
@@ -421,7 +423,7 @@ main (int argc, char **argv)
             n_files++;
     }
     
-    /* Check that the options selected are mutually consistant */
+    /* Check that the options selected are mutually consistent */
     
     no_size_needed = (read_cmt || read_planes || read_aki || read_tensor || read_axis);
     error += GMT_check_rgb (pen.rgb)
@@ -504,8 +506,8 @@ main (int argc, char **argv)
         fprintf (stderr, "             X, Y, depth, T_value, T_azim, T_plunge, N_value, N_azim, N_plunge\n");
         fprintf (stderr, "             P_value, P_azim, P_plunge, exp, newX, newY, event_title\n");
         fprintf (stderr, "         Use -o option for old (psvelomeca) format (not depth in third column)\n");
-        fprintf (stderr, "         Optionnally add /fontsize[/offset][u]\n");
-        fprintf (stderr, "      Default values are /%d/%lf\n", default_fontsize, default_offset);
+        fprintf (stderr, "         Optionally add /fontsize[/offset][u]\n");
+        fprintf (stderr, "      Default values are /%g/%f\n", default_fontsize, default_offset);
         fprintf (stderr, "      fontsize < 0 : no label written;\n");
         fprintf (stderr, "      offset is from the limit of the beach ball.\n");
         fprintf (stderr, "      By default label is above the beach ball. Add u to plot it under.\n");
@@ -517,7 +519,7 @@ main (int argc, char **argv)
         fprintf (stderr, "        -z overlays zero trace moment tensor.\n");
         GMT_explain_option ('U');
         GMT_explain_option ('V');
-        fprintf (stderr, "        -W sets default pen attributes [width = %lgp, color = (%d/%d/%d), texture = solid line].\n", 
+        fprintf (stderr, "        -W sets default pen attributes [width = %gp, color = (%d/%d/%d), texture = solid line].\n", 
             pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
         fprintf (stderr, "        -Z Use cpt-file to assign colors based on depth-value in 3rd column\n");
         fprintf (stderr, "        -a plots axis. Default symbols are circles.\n");
@@ -550,25 +552,25 @@ main (int argc, char **argv)
         n_files = 1;
     n_args = (argc > 1) ? argc : 2;
     
-    GMT_put_history (argc, argv);   /* Update .gmtcommands */
+    GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
     
     GMT_map_setup (west, east, south, north);
 
     ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, 
         gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale, 
         gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi, 
-        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, 
+	GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name,
         GMT_epsinfo (argv[0]));
     GMT_echo_command (argc, argv);
 
     if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
     
     GMT_setpen (&pen);
-    ps_setfont (gmtdefs.anot_font);
+    ps_setfont (gmtdefs.annot_font[0]);
     
     if (skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
     
-    ix = (gmtdefs.xy_toggle);    iy = 1 - ix;
+    ix = (gmtdefs.xy_toggle[0]);    iy = 1 - ix;
 
     done = FALSE;
 
@@ -589,7 +591,7 @@ main (int argc, char **argv)
             sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header) 
+        if (gmtdefs.io_header[GMT_IN]) 
             for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
         
 
@@ -886,14 +888,14 @@ Definition of scalar moment.
                 switch (justify) {
                     case 2 :
                         if(draw_box) {
-                            ps_rect(plot_x - size * 0.5, plot_y + size * 0.5 + offset + (double) (fontsize) / 72., plot_x + size * 0.5, plot_y + size * 0.5 + offset, bfill.rgb, FALSE);
+                            ps_rect(plot_x - size * 0.5, plot_y + size * 0.5 + offset + (fontsize / 72.0), plot_x + size * 0.5, plot_y + size * 0.5 + offset, bfill.rgb, FALSE);
                         }
                         ps_text(plot_x, plot_y + size * 0.5 + offset, fontsize, event_title, angle, justify, form);
                         ps_plot(plot_x, plot_y + size * 0.5 + offset, 3);
                         break;
                     case 10 :
                         if(draw_box) {
-                            ps_rect(plot_x - size * 0.5, plot_y - size * 0.5 - offset - (double) (fontsize) / 72., plot_x + size * 0.5, plot_y - size * 0.5 - offset, bfill.rgb, FALSE);
+                            ps_rect(plot_x - size * 0.5, plot_y - size * 0.5 - offset - (fontsize / 72.0), plot_x + size * 0.5, plot_y - size * 0.5 - offset, bfill.rgb, FALSE);
                         }
                         ps_text(plot_x, plot_y - size * 0.5 - offset, fontsize, event_title, angle, justify, form);
                         ps_plot(plot_x, plot_y - size * 0.5 - offset, 3);
@@ -975,11 +977,9 @@ Definition of scalar moment.
     ps_setpaint (gmtdefs.basemap_frame_rgb);
 
     if (pen.texture) ps_setdash (CNULL, 0);
-    if (frame_info.plot) {
-        ps_setpaint (gmtdefs.basemap_frame_rgb);
-        GMT_map_basemap ();
-        ps_setpaint (gmtdefs.background_rgb);
-    }
+    
+    GMT_map_basemap ();
+
     ps_plotend (gmtdefs.last_page);
     
     GMT_end (argc, argv);
