@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: pspolar.c,v 1.3.4.1 2002/02/27 17:41:10 pwessel Exp $ 
+ *    $Id: pspolar.c,v 1.11 2004/09/10 23:16:15 pwessel Exp $ 
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -13,7 +13,7 @@
  *
  * Author:         Genevieve Patau
  * Date:           19-OCT-1995 (psprojstations)
- * Version:        3.4
+ * Version:        4
  * Roots:          heavily based on psxy.c
  * Last modified : 02-APR-2001
  *
@@ -38,7 +38,7 @@ main (int argc,char **argv)
 {
     int     i, symbol = 0, n, n_files = 0, fno;
     int     n_args;
-    int     form_s = 0, justify_s = 5, fontsize_s = 12;
+    int     form_s = 0, justify_s = 5;
     
     BOOLEAN error = FALSE, nofile = TRUE;
     BOOLEAN done, greenwich, label_s = FALSE;
@@ -56,7 +56,7 @@ main (int argc,char **argv)
     double lon, lat, plot_x0, plot_y0;
     double new_lon, new_lat, new_plot_x0, new_plot_y0;
     double radius, ech = 0., azimut, ih, plongement;
-    double c_pointsize = 0.015;
+    double c_pointsize = 0.015, fontsize_s = 12.0;
     double angle_s = 0.0;
     double azS, sizeS;
     double v_width, h_length, h_width, shape;
@@ -248,7 +248,7 @@ main (int argc,char **argv)
                 case 'T':       /* Information about label printing */
                         label_s = TRUE;
                         if (strlen(argv[i]) > 2) {
-                             sscanf (&argv[i][2], "%lf/%d/%d/%d/", &angle_s,
+                             sscanf (&argv[i][2], "%lf/%d/%d/%lf/", &angle_s,
                                      &form_s, &justify_s, &fontsize_s);
                         }
                         break;
@@ -273,7 +273,7 @@ main (int argc,char **argv)
             n_files++;
     }
 
-    /* Check that the options selected are consistant */
+    /* Check that the options selected are consistent */
     
     if (!project_info.region_supplied) {
         error++;
@@ -322,12 +322,12 @@ main (int argc,char **argv)
         fprintf (stderr, "           Default is no fill\n");
         fprintf (stderr, "        -f Outline beach ball\n");
         fprintf (stderr, "           Add <pen attributes> if not current pen.\n");
-        fprintf (stderr, "        -G Specify color symbol for station in compresive part. Fill can be either\n");
+        fprintf (stderr, "        -G Specify color symbol for station in compressive part. Fill can be either\n");
         fprintf (stderr, "           Fill can be either <r/g/b> (each 0-255) for color\n");
         fprintf (stderr, "           or <gray> (0-255) for gray-shade [0].\n");
         fprintf (stderr, "           Add L[<pen>] to outline\n");
         fprintf (stderr, "           Default is black.\n");
-        fprintf (stderr, "        -g Outline of station symbol in compresive part.\n");
+        fprintf (stderr, "        -g Outline of station symbol in compressive part.\n");
         fprintf (stderr, "           Add <pen attributes> if not current pen.\n");
         fprintf (stderr, "        -h Use special format derived from HYPO71 output.\n");
         GMT_explain_option ('H');
@@ -371,7 +371,7 @@ main (int argc,char **argv)
         n_files = 1;
     n_args = (argc > 1) ? argc : 2;
     
-    GMT_put_history (argc, argv);   /* Update .gmtcommands */
+    GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
     greenwich = (west < 0.0 || east <= 0.0);
     
@@ -380,13 +380,13 @@ main (int argc,char **argv)
     ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation,
         gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale,
         gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi,
-        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb,
+        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name,
         GMT_epsinfo (argv[0]));
     GMT_echo_command (argc, argv);
 
     if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
     
-    if (label_s) ps_setfont (gmtdefs.anot_font);
+    if (label_s) ps_setfont (gmtdefs.annot_font[0]);
 
     if (symbol > 0 && skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
     
@@ -408,10 +408,10 @@ main (int argc,char **argv)
 
         if (!nofile && gmtdefs.verbose) {
             fprintf (stderr, "%s: Working on file %s\n", argv[0], argv[fno]);
-            sprintf (line, "File: %s", argv[fno]);
+            sprintf (line, "File: %s\0", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
+        if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
         
         GMT_world_map = TRUE;
         
@@ -640,11 +640,9 @@ main (int argc,char **argv)
     GMT_world_map = old_GMT_world_map;
     
     if (pen.texture) ps_setdash (CNULL, 0);
-    if (frame_info.plot) {
-        ps_setpaint (gmtdefs.basemap_frame_rgb);
-        GMT_map_basemap ();
-        ps_setpaint (gmtdefs.background_rgb);
-    }
+
+    GMT_map_basemap ();
+
     ps_plotend (gmtdefs.last_page);
     
     GMT_end (argc, argv);
