@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: psvelo.c,v 1.3.4.1 2002/02/27 17:41:10 pwessel Exp $
+ *    $Id: psvelo.c,v 1.12 2004/09/10 23:16:15 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -15,7 +15,7 @@ PostScript code is written to stdout.
 
  Author:     Kurt Feigl
  Date:	      7 July 1998
- Version:     3.3.4
+ Version:     4
  Roots:       based on psxy.c	
  Adapted to version 3.3 by Genevieve Patau (25 June 1999)
  Last modified : 18 February 2000
@@ -75,7 +75,7 @@ main (int argc, char **argv)
 	double v_width = 0.01, h_length = 0.12, h_width = 0.03, vector_shape = 0.4;
         double wedge_amp = 1.e7;
 	double t11 = 1.0, t12 = 0.0, t21 = 0.0, t22 = 1.0;
-        double hl,hw,vw;
+        double hl,hw,vw, fontsize = 0.0;
 
         char station_name[20];
         char txt_a[32], txt_b[32], txt_c[32];
@@ -87,7 +87,7 @@ main (int argc, char **argv)
 	struct GMT_PEN pen;
 	struct GMT_FILL fill, efill; /* efill is for uncertainty wedge */
 
-        int justify, fontsize;
+        int justify;
 
 	argc = GMT_begin (argc, argv);
 	
@@ -160,7 +160,7 @@ main (int argc, char **argv)
                                             strcpy(txt_a, &argv[i][3]);
                                             n=0; while (txt_a[n] && txt_a[n] != '/') n++; txt_a[n]=0;
                                             scale = GMT_convert_units (txt_a, GMT_INCH);
-                                            sscanf(strchr(&argv[i][3],'/')+1, "%lf/%d", &confidence, &fontsize);
+                                            sscanf(strchr(&argv[i][3],'/')+1, "%lf/%lf", &confidence, &fontsize);
 					    /* confidence scaling */
 					    conrad = sqrt( -2.0 * log(1.0 - confidence)); 
 					  }
@@ -213,7 +213,7 @@ main (int argc, char **argv)
 			n_files++;
 	}
 	
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	no_size_needed = (read_ellipse || read_rotated_ellipse || read_anisotropy || read_cross || read_wedge );
 	error += GMT_check_rgb (pen.rgb)
@@ -274,7 +274,7 @@ main (int argc, char **argv)
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
 	
-        GMT_put_history (argc, argv);   /* Update .gmtcommands */
+        GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
 	greenwich = (project_info.w < 0.0 || project_info.e <= 0.0);
 	
@@ -282,20 +282,21 @@ main (int argc, char **argv)
 
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb,
+		     gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 	
         GMT_setpen (&pen);
-	ps_setfont (gmtdefs.anot_font);
+	ps_setfont (gmtdefs.annot_font[0]);
         if (shade_uncert) outline = TRUE;
 	
 	if (skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
 	
 	old_GMT_world_map = GMT_world_map;
 	
-	ix = (gmtdefs.xy_toggle);	iy = 1 - ix;
+	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
 
 	done = FALSE;
 
@@ -313,10 +314,10 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) {
 			fprintf (stderr, "psvelo: Working on file %s\n", argv[fno]);
-			sprintf (line, "File: %s", argv[fno]);
+			sprintf (line, "File: %s\0", argv[fno]);
 			ps_comment (line);
 		}
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
+		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
 		
 		if (gmtdefs.verbose && (read_ellipse || read_rotated_ellipse))	 
 		  fprintf (stderr, "psvelo: 2-D confidence interval and scaling factor %f %f\n",confidence, conrad);
@@ -419,7 +420,7 @@ main (int argc, char **argv)
 					  ps_vector(plot_x, plot_y, plot_vx, plot_vy, vw, hl, hw, vector_shape, 
 						    fill.rgb, outline);
 					  justify = plot_vx - plot_x > 0. ? 7 : 5;
-                                          if(fontsize > 0 && strlen(station_name) > 0) {
+                                          if(fontsize > 0.0 && strlen(station_name) > 0) {
 						ps_text(plot_x + (6 - justify) / 25.4 , plot_y, fontsize,
 					     	station_name, ANGLE, justify,FORM); /* 1 inch = 2.54 cm */
                                           }
@@ -428,7 +429,7 @@ main (int argc, char **argv)
                                             ps_plot(plot_x, plot_y, 3);
                                             ps_circle(plot_x, plot_y, POINTSIZE, fill.rgb, 1);
                                             justify = 10;
-                                            if(fontsize > 0 && strlen(station_name) > 0) {
+                                            if(fontsize > 0.0 && strlen(station_name) > 0) {
                                                  ps_text(plot_x, plot_y - 1. / 25.4, fontsize, station_name, ANGLE, justify, FORM);
                                             }
                                             /*  1 inch = 2.54 cm */
@@ -477,11 +478,9 @@ main (int argc, char **argv)
 	if (skip_if_outside) GMT_map_clip_off ();
 	
 	if (pen.texture) ps_setdash (CNULL, 0);
-	if (frame_info.plot) {
-		ps_setpaint (gmtdefs.basemap_frame_rgb);
-		GMT_map_basemap ();
-		ps_setpaint (gmtdefs.background_rgb);
-	}
+	
+	GMT_map_basemap ();
+
 	ps_plotend (gmtdefs.last_page);
 	
 	GMT_end (argc, argv);
