@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_mgg.h,v 1.1.1.1.4.1 2002/01/17 17:42:52 pwessel Exp $
+ *	$Id: gmt_mgg.h,v 1.2 2004/06/09 20:08:16 pwessel Exp $
  *
- *    Copyright (c) 1991-2002 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 
@@ -11,9 +11,11 @@
 #define MDEG2DEG	0.000001	/* Convert millidegrees to degrees */
 #define NGDC_OLDEST_YY	39	/* Oldest NGDC cruise is from 1939 */
 
+#define GMTMGG_TIME_MAXMONTH	61	/* 5 years is a long time for one cruise */
+
 struct GMTMGG_TIME {
-  int daymon[61];	/* Cumulative number of days up to last month */
-  int first_year;	/* The year the cruise started */
+  int daymon[GMTMGG_TIME_MAXMONTH];	/* Cumulative number of days up to last month */
+  int first_year;			/* The year the cruise started */
 };
 
 struct GMTMGG_REC {	/* Format of *.gmt file records */
