@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: nearneighbor.c,v 1.5.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: nearneighbor.c,v 1.19 2004/04/25 09:10:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -28,7 +28,7 @@
  *
  * Author:	Paul Wessel
  * Date:	14-JUL-2000
- * Version:	3.4.3
+ * Version:	4
  */
  
 #include "gmt.h"
@@ -54,7 +54,7 @@ main (int argc, char **argv)
 	BOOLEAN go, error = FALSE, done = FALSE, nofile = TRUE;
 	BOOLEAN set_empty = FALSE, weighted = FALSE, wrap_180, replicate_x, replicate_y;
 
-	double radius = 0.0, weight, weight_sum, *x0, *y0, dx, dy, delta, distance, factor;
+	double radius = 0.0, weight, weight_sum, grd_sum, *x0, *y0, dx, dy, delta, distance, factor;
 	double *in, *shrink, km_pr_deg, x_left, x_right, y_top, y_bottom, offset, xinc2, yinc2, idx, idy;
 	double half_y_width, y_width, half_x_width, x_width, three_over_radius;
 
@@ -89,15 +89,14 @@ main (int argc, char **argv)
 				case 'R':
 				case 'V':
 				case ':':
+				case 'b':	/* Input triplets [quadruplets] are binary, not ascii */
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
 				
 				/* Supplemental parameters */
 				
-				case 'b':	/* Input triplets [quadruplets] are binary, not ascii */
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'I':
 					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
 					break;
@@ -119,10 +118,16 @@ main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						error += GMT_boundcond_parse (&edgeinfo, &argv[i][2]);
-						if (edgeinfo.gn) GMT_geographic_in = TRUE;
+						if (edgeinfo.gn) {
+							GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+							GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+						}
+					}
+					else {
+						GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+						GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+						fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					}
-					else
-						GMT_geographic_in = TRUE;
 					break;
 				case 'N':
 					n_sectors = atoi (&argv[i][2]);
@@ -149,7 +154,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "nearneighbor %s - A \"Nearest neighbor\" gridding algorithm\n\n", GMT_VERSION);
 		fprintf(stderr, "usage: nearneighbor [xyzfile(s)] -G<out_grdfile> -I<dx>[m|c][/<dy>[m|c]]\n");
 		fprintf(stderr, "\t-N<sectors> -R<west/east/south/north> -S<radius>[m|c|k|K] [-E<empty>] [-F]\n");
-		fprintf(stderr, "\t[-H ] [-L[<flags>]] [-V ] [-W] [-:] [-bi[s][<n>]]\n\n");
+		fprintf(stderr, "\t[-H ] [-L<flags>] [-V ] [-W] [-:] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
 		fprintf(stderr, "\t-G name of output grid.\n");
 		fprintf(stderr, "\t-I sets the grid spacing for the grid.  Append m for minutes, c for seconds.\n");
@@ -166,13 +171,13 @@ main (int argc, char **argv)
 		fprintf(stderr, "\t   g for geographic boundary conditions, or one or both of\n");
 		fprintf(stderr, "\t   x for periodic boundary conditions on x\n");
 		fprintf(stderr, "\t   y for periodic boundary conditions on y\n");
-		fprintf (stderr, "\t   If no flag is given, x is assumed to be longitude and periodic in 360.\n");
 		GMT_explain_option ('V');
 		fprintf(stderr, "\t-W input file has observation weights in 4th column.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf(stderr, "\t   Default is 3 (or 4 if -W is set) columns\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		
 		exit (EXIT_FAILURE);
@@ -198,28 +203,28 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error++;
 	}	
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
 	n_req = (weighted) ? 4 : 3;
-	if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = n_req;
-	if (GMT_io.binary[0] && n_req > GMT_io.ncol[0]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
+	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	GMT_grd_RI_verify (&header, 1);
 
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted;
+	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted;
 	
         if (n_files > 0)
         	nofile = FALSE;
@@ -261,7 +266,7 @@ main (int argc, char **argv)
 	for (i = 0; i < header.nx; i++) x0[i] = header.x_min + i * header.x_inc + xinc2;
 	for (j = 0; j < header.ny; j++) y0[j] = header.y_max - j * header.y_inc - yinc2;
 	if (distance_flag) {	/* Input data is geographical */
-		km_pr_deg = 0.001 * 2.0 * M_PI * gmtdefs.ellipse[gmtdefs.ellipsoid].eq_radius / 360.0;
+		km_pr_deg = 0.001 * 2.0 * M_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
 		max_di = (int) (ceil (header.nx / 2.0) + 0.1);
 		actual_max_di = 0;
 		for (j = 0; j < header.ny; j++) {
@@ -298,7 +303,7 @@ main (int argc, char **argv)
 			fp = GMT_stdin;
 			done = TRUE;
 #ifdef SET_IO_MODE
-			GMT_setmode (0);
+			GMT_setmode (GMT_IN);
 #endif
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -308,7 +313,7 @@ main (int argc, char **argv)
 		
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 	
@@ -325,10 +330,10 @@ main (int argc, char **argv)
 			point[n].z = (float)in[2];
 			if (weighted) point[n].w = (float)in[3];
 		
-			/* Find indeces of the node closest to this data point */
+			/* Find indices of the node closest to this data point */
 
-			i0 = (int)floor (((in[0] - header.x_min) * idx) + offset);
-			j0 = (int)floor (((header.y_max - in[1]) * idy) + offset);
+			i0 = GMT_x_to_i(in[0], header.x_min, idx, offset, header.nx);
+			j0 = GMT_y_to_j(in[1], header.y_min, idy, offset, header.ny);
 		
 			/* Loop over all nodes within radius of this node */
 
@@ -381,7 +386,7 @@ main (int argc, char **argv)
 					sector = ((int)((d_atan2 (dy, dx) + M_PI) * factor)) % n_sectors;
 					assign_node (&grid_node[k], n_sectors, sector, distance, n);
 
-					/* With periodic, gridline-registrered grids there are duplicate rows and/or columns
+					/* With periodic, gridline-registered grids there are duplicate rows and/or columns
 					   so we may have to assign the point to more than one node.  The next section deals
 					   with this situation.
 					*/
@@ -401,6 +406,7 @@ main (int argc, char **argv)
 				}
 			}
 			n++;
+			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10d\r", GMT_program, n);
 			if (n == n_alloc) {
 				n_alloc += GMT_CHUNK;
 				point = (struct POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct POINT), GMT_program);
@@ -408,7 +414,8 @@ main (int argc, char **argv)
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10d\n", GMT_program, n);
+				
 	point = (struct POINT *) GMT_memory ((void *)point, (size_t)n, sizeof (struct POINT), GMT_program);
 	grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
 
@@ -438,17 +445,19 @@ main (int argc, char **argv)
 			/* OK, here we have enough data and need to calculate the weighted value */
 
 			n_set++;
-			weight_sum = grd[ij] = 0.0;	/* Initialize sums */
+			weight_sum = grd_sum = 0.0;	/* Initialize sums */
 			for (k = 0; k < n_sectors; k++) {
 				delta = three_over_radius * grid_node[ij]->distance[k];
 				weight = 1.0 / (1.0 + delta * delta);	/* This is distance weight */
 				if (weighted) weight *= point[grid_node[ij]->datum[k]].w;	/* This is observation weight */
-				grd[ij] += (float)(weight * point[grid_node[ij]->datum[k]].z);
+				grd_sum += weight * point[grid_node[ij]->datum[k]].z;
 				weight_sum += weight;
 			}
-			grd[ij] /= (float)weight_sum;
+			grd[ij] = (float)(grd_sum / weight_sum);
 		}
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\r", GMT_program, j);
 	}
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\n", GMT_program, j);
 	
 	if (GMT_write_grd (outfile, &header, grd, 0.0, 0.0, 0.0, 0.0, pad, FALSE)) {
 		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, outfile);
