@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: project.c,v 1.4.4.10 2004/01/07 20:45:09 pwessel Exp $
+ *	$Id: project.c,v 1.24 2004/04/25 09:10:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -32,7 +32,7 @@
 		03-NOV-1998	PW: Can read any number of data columns; z in -Fz refers to
 				all these columns in the output.
    Version:	3.4		PW: Fixed problem with small circle distances
-   Version:	3.4.3
+   Version:	4		
 */
 
 #include "gmt.h"
@@ -62,7 +62,7 @@ main (int argc, char **argv)
 	int	nc, ne, np, nl, nw, n_files = 0, fno, n_args, n_fields, n_expected_fields, n_total_used = 0, n_z = 0, n_items;
 	int     output_choice[7];
 
-	double	xx, yy, x_a, y_a, x_b, y_b, x_p, y_p, d_to_km, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
+	double	xx, yy, x_a, y_a, x_b, y_b, x_p, y_p, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
 	double	theta, d_inc = 0.0, d_along, *in, *out = (double *)NULL;
 	
 	double	azimuth = 0.0, l_min = 0.0, l_max = 0.0, w_min = 0.0, w_max = 0.0;
@@ -86,9 +86,15 @@ main (int argc, char **argv)
 	x_a = x_b = x_p = y_a = y_b = y_p = 0.0;
 	n_definitions = 0;
 	n_outputs = 0;
-	GMT_geographic_in = GMT_geographic_out = TRUE;
 	j = 1;
 	for (i = 0; i < 7; i++) output_choice[i] = 0;
+	
+	/* New in GMT 4: Must process -N before the rest to ensure col_types are set properly (there is no -H here)  */
+	for (i = 1; i < argc; i++) if (!strcmp (argv[i], "-N")) flat_earth = TRUE;
+	if (!flat_earth) {
+		GMT_io.in_col_type[0] = GMT_IS_LON;
+		GMT_io.in_col_type[1] = GMT_IS_LAT;
+	}
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -100,15 +106,14 @@ main (int argc, char **argv)
 				case 'H':
 				case 'V':
 				case ':':
+				case 'b':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
 				
 				/* Supplemental parameters */
 				
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'F':
 					for (j = 2, k = 0; argv[i][j]; j++, k++) {
 						switch (argv[i][j]) {
@@ -149,8 +154,9 @@ main (int argc, char **argv)
 					break;
 				case 'C':
 					nc = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					x_a = GMT_ddmmss_to_degree (txt_a);
-					y_a = GMT_ddmmss_to_degree (txt_b);
+					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_a), txt_a);
+					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_a), txt_b);
+					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
 					origin_set = TRUE;
 					break;
 				case 'D':
@@ -168,8 +174,9 @@ main (int argc, char **argv)
 					break;
 				case 'E':
 					ne = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					x_b = GMT_ddmmss_to_degree (txt_a);
-					y_b = GMT_ddmmss_to_degree (txt_b);
+					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_b), txt_a);
+					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_b), txt_b);
+					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
 					two_points = TRUE;
 					n_definitions++;
 					break;
@@ -190,20 +197,19 @@ main (int argc, char **argv)
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
 					break;
+				case 'N': /* Handled above but still in argv */
+					break;
 				case 'Q':
 					convert_units = TRUE;
 					break;
-				case 'N':
-					flat_earth = TRUE;
-					GMT_geographic_in = GMT_geographic_out = FALSE;
-					break;
 				case 'S':
 					sort_output = TRUE;
 					break;
 				case 'T':
 					np = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					x_p = GMT_ddmmss_to_degree (txt_a);
-					y_p = GMT_ddmmss_to_degree (txt_b);
+					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_p), txt_a);
+					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_p), txt_b);
+					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
 					pole_set = TRUE;
 					n_definitions++;
 					break;
@@ -224,8 +230,9 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordiantes\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	project [files] -C<ox>/<oy> [-A<azimuth>] [-D<d_or_g>] [-E<bx>/<by>]\n");
-		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [-H[<nrec>]] [-L[w][<l_min>/<l_max>]] [-M[<flag>]] [-N] [-Q] [-S]\n");
-		fprintf(stderr,"\t[-T<px>/<py>] [-V] [-W<w_min>/<w_max>] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
+		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [-H[<nrec>]] [-L[w][<l_min>/<l_max>]]\n");
+		fprintf(stderr,"\t[-M[<flag>]] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n");
+		fprintf(stderr,"\t[-:] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 
@@ -279,6 +286,8 @@ main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf(stderr,"\t  Default is 2 input columns (x, y)\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -335,29 +344,29 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%d)\n", GMT_program, n_outputs);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 2) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 2) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
-	GMT_setmode (1);
+	GMT_setmode (GMT_OUT);
 #endif
 
-	pure_ascii = !(GMT_io.binary[0] || GMT_io.binary[1]);
+	pure_ascii = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
 
 	if (n_outputs == 0 && !(generate) ) {	/* Generate default -F setting (all) */
 		n_outputs = 7;
@@ -369,8 +378,6 @@ main (int argc, char **argv)
 
 	p_data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct DATA), GMT_program);
 	
-	d_to_km = 0.001 * 2.0 * M_PI * gmtdefs.ellipse[N_ELLIPSOIDS-1].eq_radius / 360.0;
-
 	if (generate && two_points && (l_min == l_max) ) stay_within = TRUE;	/* Default generate from A to B  */
 
 	/* Set up rotation matrix e for flat earth, or pole and center for spherical; get l_min, l_max if stay_within  */
@@ -383,7 +390,7 @@ main (int argc, char **argv)
 			xx = x_b - x_a;
 			yy = y_b - y_a;
 			l_max = d_sqrt(xx*xx + yy*yy);
-			if (convert_units) l_max *= d_to_km;
+			if (convert_units) l_max *= DEG2KM;
 		}
 	}
 	else {
@@ -398,18 +405,18 @@ main (int argc, char **argv)
 			l_min = 0.0;
 			l_max = GMT_dot3v(a,b);
 			l_max = d_acos(l_max) * R2D;
-			if (convert_units) l_max *= d_to_km;
+			if (convert_units) l_max *= DEG2KM;
 		}
 	}
 
 	/* Now things are initialized.  We will work in degrees for awhile, so we convert things:  */
 
 	if (convert_units) {
-		d_inc /= d_to_km;
-		l_min /= d_to_km;
-		l_max /= d_to_km;
-		w_min /= d_to_km;
-		w_max /= d_to_km;
+		d_inc /= DEG2KM;
+		l_min /= DEG2KM;
+		l_max /= DEG2KM;
+		w_min /= DEG2KM;
+		w_max /= DEG2KM;
 	}
 
 	/*  Now we are ready to work  */
@@ -468,15 +475,15 @@ main (int argc, char **argv)
 
 		if (convert_units) {
 			for (i = 0; i < n_used; i++) {
-				p_data[i].a[2] *= d_to_km;
-				p_data[i].a[3] *= d_to_km;
+				p_data[i].a[2] *= DEG2KM;
+				p_data[i].a[3] *= DEG2KM;
 			}
 		}
 
 		/* Now output generated track */
 
-		if (!GMT_io.binary[1]) {
-			if (gmtdefs.io_header) fprintf (GMT_stdout, "lon\tlat\tdist\n");
+		if (!GMT_io.binary[GMT_OUT]) {
+			if (gmtdefs.io_header[GMT_OUT]) fprintf (GMT_stdout, "lon\tlat\tdist\n");
 	
 			for (i = 0; i < n_used; i++) {
 				for (j = 0; j < n_outputs; j++) out[j] = p_data[i].a[output_choice[j]];
@@ -487,7 +494,7 @@ main (int argc, char **argv)
 	
 	else {	/* Must read input file */
 
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
 
 		if (n_files > 0)
 			nofile = FALSE;
@@ -503,7 +510,7 @@ main (int argc, char **argv)
 				fp = GMT_stdin;
 				done = TRUE;
 #ifdef SET_IO_MODE
-				GMT_setmode (0);
+				GMT_setmode (GMT_IN);
 #endif
 			}
 			else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -513,7 +520,7 @@ main (int argc, char **argv)
 
 			if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-			if (gmtdefs.io_header) {
+			if (gmtdefs.io_header[GMT_IN]) {
 				GMT_fgets (record_str, BUFSIZ, fp);
 				sscanf (record_str, "%s %s %s", heading[0], heading[1], heading[6]);
 				if (! (heading[6]) ) strcpy (heading[6],"Z");
@@ -625,15 +632,15 @@ main (int argc, char **argv)
 
 				if (convert_units) {
 					for (i = 0; i < n_used; i++) {
-						p_data[i].a[2] *= d_to_km;
-						p_data[i].a[3] *= d_to_km;
+						p_data[i].a[2] *= DEG2KM;
+						p_data[i].a[3] *= DEG2KM;
 					}
 				}
 
 				/* Now output  */
 
-				if (!GMT_io.binary[1]) {	/* First do header */
-					if (first && gmtdefs.io_header) {
+				if (!GMT_io.binary[GMT_OUT]) {	/* First do header */
+					if (first && gmtdefs.io_header[GMT_OUT]) {
 						for (j = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1)
 								fprintf (GMT_stdout, "%s", heading[6]);
@@ -859,7 +866,7 @@ void sphere_project_setup(double alat, double alon, double *a, double blat, doub
 	   After p is found, then c = normalized(p X a).
 	*/
 
-	double	e[9];	/* Euler roatation matrix, if needed  */
+	double	e[9];	/* Euler rotation matrix, if needed  */
 	double neg_azim;
 
 	/* First find p vector  */
@@ -894,7 +901,7 @@ void	flat_project_setup(double alat, double alon, double blat, double blon, doub
 	/* Sets up stuff for rotation of cartesian 2-vectors, analogous
 	   to the spherical three vector stuff above.  Also change azim
 	   to the cartesian theta, counterclockwise from the x axis.  */		
-
+	
 	if (two_pts) {
 		*azim = d_atan2((blat - alat), (blon - alon));
 	}
