@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pscoast.c,v 1.3.4.9 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pscoast.c,v 1.27 2004/05/23 05:52:07 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -47,7 +47,7 @@
  *
  * Author:	Paul Wessel
  * Date:	24-JUN-2000
- * Version:	3.4.3
+ * Version:	4
  *
  */
 
@@ -70,15 +70,15 @@ main (int argc, char **argv)
 	BOOLEAN	error = FALSE, set_lake_fill = FALSE, draw_river = FALSE, shift = FALSE, need_coast_base, recursive;
 	BOOLEAN	greenwich = FALSE, draw_coast = FALSE, fill_ocean = FALSE, fill_land = FALSE, possibly_donut_hell = FALSE;
 	BOOLEAN clobber_background, draw_border = FALSE, paint_polygons = FALSE, donut, fill_in_use = FALSE;
-	BOOLEAN draw_scale = FALSE, gave_xy = FALSE, fancy = FALSE, donut_hell = FALSE, world_map_save, did_clip = FALSE;
+	BOOLEAN donut_hell = FALSE, world_map_save, did_clip = FALSE;
 	BOOLEAN clip_ocean = FALSE, clip_land = FALSE, end_of_clip = FALSE, clipping = FALSE, dumping = FALSE;
 	
 	double	west = 0.0, east = 0.0, south = 0.0, north = 0.0, bin_x[5], bin_y[5];
 	double west_border, east_border, anti_lon, anti_lat, edge = 720.0, left, right;
-	double *xtmp, *ytmp, min_area = 0.0, x0, y0, scale_lat, length, step = 0.01;
+	double *xtmp, *ytmp, min_area = 0.0, step = 0.01;
 	double anti_x, anti_y, x_0, y_0, x_c, y_c, dist, out[2];
 	
-	char res = 'l', key[5], *string, measure = '\0', comment[128];
+	char res = 'l', key[5], *string, comment[128];
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
 
 	struct GMT_PEN coast, pen, bpen[N_BLEVELS], rpen[N_RLEVELS];
@@ -86,6 +86,8 @@ main (int argc, char **argv)
 	struct GMT_SHORE c;
 	struct GMT_BR b, r;
 	struct POL *p;
+	struct MAP_SCALE ms;
+	struct MAP_ROSE mr;
 		
 	GMT_init_fill (&fill[0], 255, 255, 255);	/* Default Ocean color */
 	GMT_init_fill (&fill[1], 0, 0, 0);		/* Default Land color */
@@ -99,6 +101,7 @@ main (int argc, char **argv)
 	
 	memset ((void *)rlevels, 0, (size_t)(N_RLEVELS * sizeof (int)));
 	memset ((void *)blevels, 0, (size_t)(N_BLEVELS * sizeof (int)));
+	ms.plot = mr.plot = FALSE;
 	
 	argc = GMT_begin (argc, argv);
 	
@@ -123,16 +126,13 @@ main (int argc, char **argv)
 				case 'Y':
 				case 'y':
 				case 'c':
+				case 'b':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
 				
 				/* Supplemental parameters */
 
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
-
 				case 'A':
 					n = sscanf (&argv[i][2], "%lf/%d/%d", &min_area, &min_level, &max_level);
 					if (n == 1) min_level = 0, max_level = MAX_LEVEL;
@@ -165,8 +165,7 @@ main (int argc, char **argv)
 					}
 					break;
 				case 'L':
-					error += GMT_getscale (&argv[i][2], &x0, &y0, &scale_lat, &length, &measure, &fancy, &gave_xy);
-					draw_scale = TRUE;
+					error += GMT_getscale (&argv[i][2], &ms);
 					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
@@ -175,12 +174,12 @@ main (int argc, char **argv)
 
 				case 'N':
 					if (!argv[i][2]) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR:  -N option takes two arguments\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR:  -N option takes at least one argument\n", GMT_program);
 						error++;
 						continue;
 					}
 					draw_border = TRUE;
-					memset ((void *)key, 0, (size_t)5);
+					strncpy(key, "", 5);
 					for (k = 2; argv[i][k] && argv[i][k] != '/'; k++);
 					if (argv[i][k] && argv[i][k] == '/') {	/* Specified pen args */
 						strncpy (key, &argv[i][2], (size_t)k-2);
@@ -217,12 +216,12 @@ main (int argc, char **argv)
 					
 				case 'I':
 					if (!argv[i][2]) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR:  -I option takes two arguments\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR:  -I option takes at least one argument\n", GMT_program);
 						error++;
 						continue;
 					}
 					draw_river = TRUE;
-					memset ((void *)key, 0, (size_t)5);
+					strncpy(key, "", 5);
 					for (k = 2; argv[i][k] && argv[i][k] != '/'; k++);
 					if (argv[i][k] && argv[i][k] == '/') {	/* Specified pen args */
 						strncpy (key, &argv[i][2], (size_t)k-2);
@@ -280,6 +279,9 @@ main (int argc, char **argv)
 						fill_ocean = TRUE;
 					}
 					break;
+				case 'T':
+					error += GMT_getrose (&argv[i][2], &mr);
+					break;
 				case 'W':
 					if (argv[i][2] && GMT_getpen (&argv[i][2], &coast)) {
 						GMT_pen_syntax ('W');
@@ -309,10 +311,12 @@ main (int argc, char **argv)
 	
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr,"pscoast %s - Plot continents, shorelines, rivers, and borders on maps\n\n", GMT_VERSION);
-		fprintf (stderr,"usage: pscoast -J<params> -R<west>/<east>/<south>/<north> [-A<min_area>[/<min_level>/<max_level>]] [-B<tickinfo>]\n");
-		fprintf (stderr, "\t[-C[<fill>]] [-D<resolution>] [-Eaz/el] [-G[<fill>]] [-I<feature>[/<pen>]]\n");
-		fprintf (stderr, "\t[-K] [-L[f][x]<lon0>/<lat0>/<slat>/<length>[m|n|k]] [-M[<flag>]] [-N<feature>[/<pen>]] [-O] [-P] [-Q]\n");
-		fprintf (stderr, "\t[-S<fill>] [-U[dx/dy/][label]] [-V] [-W[<pen>]] [-X<x_shift>] [-Y<y_shift>] [-bo[s][<n>]] [-c<ncopies>]\n");
+		fprintf (stderr,"usage: pscoast -J<params> -R<west>/<east>/<south>/<north>\n\t[-A<min_area>[/<min_level>/<max_level>]]\n");
+		fprintf (stderr, "\t[-B<tickinfo>] [-C[<fill>]] [-D<resolution>] [-Eaz/el] [-G[<fill>]] [-I<feature>[/<pen>]] [-K]\n");
+		fprintf (stderr, "\t[-L[f][x]<lon0>/<lat0>[/<slon>]/<slat>/<length>[m|n|k][:label:<just>][+p<pen>][+f<fill>]]\n");
+		fprintf (stderr, "\t[-M[<flag>]] [-N<feature>[/<pen>]] [-O] [-P] [-Q] [-S<fill>]\n");
+		fprintf (stderr, "\t[-T[f|m][x]<lon0>/<lat0>/<size>[/<info>][:w,e,s,n:][+<gints>[/<mints>]]] [-U[dx/dy/][label]]\n");
+		fprintf (stderr, "\t[-V] [-W[<pen>]] [-X<x_shift>] [-Y<y_shift>] [-bo[s][<n>]] [-c<ncopies>]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -353,9 +357,13 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t      i = All intermittent rivers (5-7)\n");
 		fprintf (stderr, "\t      c = All canals (8-10)\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "\t-L draws a simple map scaLe centered on <lon0>/<lat0>.  Use -Lx to specify cartesian coordinates instead.\n");
-		fprintf (stderr, "\t   Scale is calculated at latitude <slat>. <length> is in km, or [nautical] miles if [n] m is appended\n");
-		fprintf (stderr, "\t   -Lf draws a \"fancy\" scale [Default is plain]\n");
+		fprintf (stderr, "\t-L draws a simple map scaLe centered on <lon0>/<lat0>.  Use -Lx to specify Cartesian\n");
+		fprintf (stderr, "\t   coordinates instead.  Scale is calculated at latitude <slat>; optionally give longitude\n");
+		fprintf (stderr, "\t   <slon> [Default is central longitude].  <length> is in km, or [nautical] miles if [n] m\n");
+		fprintf (stderr, "\t   is appended.  -Lf draws a \"fancy\" scale [Default is plain]. By default, the label is set\n");
+		fprintf (stderr, "\t   to the distance unit and placed on top [t].  Use the :label:<just> mechanism to specify another\n");
+		fprintf (stderr, "\t   label (or - to keep the default) and placement (t,b,l,r, and u - to use the label as a unit.\n");
+		fprintf (stderr, "\t   Append +p<pen> and/or +f<fill> to draw/paint a rectangle beneath the scale [no rectangle]\n");
 		fprintf (stderr, "\t-M Dump a multisegment ascii (or binary, see -bo) file to standard output.  No plotting occurs\n");
 		fprintf (stderr, "\t   Specify any combination of -W, -I, -N.  Optionally, append 1-char\n");
 		fprintf (stderr, "\t   segment header <flag> [%c]\n", GMT_io.EOF_flag);
@@ -372,12 +380,23 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0].\n");
 		fprintf (stderr, "\t   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
 		fprintf (stderr, "\t   3) c to issue clip paths for water areas.\n");
+		fprintf (stderr, "\t-T draws a north-poinTing rose centered on <lon0>/<lat0>.  Use -Tx to specify Cartesian\n");
+		fprintf (stderr, "\t   coordinates instead.  -Tf draws a \"fancy\" rose [Default is plain].  Give rose diameter\n");
+		fprintf (stderr, "\t   <size> and optionally the west, east, south, north labels desired [W,E,S,N].\n");
+		fprintf (stderr, "\t   For fancy rose, specify as <info> the kind you want: 1 draws E-W, N-S directions [Default],\n");
+		fprintf (stderr, "\t   2 adds NW-SE and NE-SW, while 3 adds WNW-ESE, NNW-SSE, NNE-SSW, and ENE-WSW.\n");
+		fprintf (stderr, "\t   For Magnetic compass rose, specify -Tm.  Use the optional <info> = <dec>/<dlabel> (where <dec> is\n");
+		fprintf (stderr, "\t   the magnetic declination and <dlabel> is a label for the magnetic compass needle) to plot\n");
+		fprintf (stderr, "\t   directions to both magnetic and geographic north [Default is just geographic].\n");
+		fprintf (stderr, "\t   If the North label = \'*\' then a north star is plotted instead of the label.\n");
+		fprintf (stderr, "\t   Append +<gints>/<mints> to override default annotation/tick interval(s) [10/5/1/30/5/1].\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		fprintf (stderr,"\t-W draw shorelines.  Append pen [Default is solid line of unit thickness].\n");
 		GMT_explain_option ('X');
-		GMT_explain_option ('c');
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('c');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -386,7 +405,7 @@ main (int argc, char **argv)
 		if (min_level < 2) min_level = 2;
 	}
 		
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	clipping = (clip_land || clip_ocean);
 	if (!(end_of_clip || project_info.region_supplied)) {
@@ -458,7 +477,7 @@ main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 	need_coast_base = (fill_land || fill_ocean || set_lake_fill || draw_coast || clip_land || clip_ocean);
 	clobber_background = (fill_land && fill_ocean);
@@ -506,9 +525,14 @@ main (int argc, char **argv)
 		if (!GMT_io.binary[1]) fprintf (GMT_stdout, "# Data from the %s resolution GMT shoreline, borders, and rivers database\n", shore_resolution[base]);
 	}
 	else {
+		if (end_of_clip)
+			gmtdefs.page_orientation |= GMT_CLIP_OFF;	/* Signal that this program terminates clipping that initiated prior to this process */
+		else if (clipping)
+			gmtdefs.page_orientation |= GMT_CLIP_ON;	/* Signal that this program initiates clipping that wil outlive this process */
+			
 		ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo(argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo(argv[0]));
 		GMT_echo_command (argc, argv);
 		if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 		if (!end_of_clip && base > 0) ps_command ("2 setlinejoin");	/* Avoid excessive mitering for cruder lines */
@@ -516,7 +540,7 @@ main (int argc, char **argv)
         	if (end_of_clip) {  /* Just undo previous clip-path */
               		ps_clipoff ();
 
-			if (frame_info.plot) GMT_map_basemap (); /* Basemap needed */
+			GMT_map_basemap (); /* Basemap needed */
 
 			ps_plotend (gmtdefs.last_page);
 
@@ -557,8 +581,8 @@ main (int argc, char **argv)
 		GMT_geo_to_xy (project_info.central_meridian, project_info.pole, &x_0, &y_0);
 		if (fill_land && gmtdefs.verbose) {
 			fprintf (stderr, "%s: Warning: Fill continent option (-G) may not work for this projection.\n", GMT_program);
-			fprintf (stderr, "If the antipole (%lg/%lg) is in the ocean then chances are good\n", anti_lon, anti_lat);
-			fprintf (stderr, "Else: avoid projection center coordinates that are exact multiples of %lg degrees\n", c.bsize);
+			fprintf (stderr, "If the antipole (%g/%g) is in the ocean then chances are good\n", anti_lon, anti_lat);
+			fprintf (stderr, "Else: avoid projection center coordinates that are exact multiples of %g degrees\n", c.bsize);
 		}
 	}
 
@@ -878,7 +902,8 @@ main (int argc, char **argv)
 			GMT_map_basemap ();
 		}
 	
-		if (draw_scale) GMT_draw_map_scale (x0, y0, scale_lat, length, measure, gave_xy, fancy);
+		if (ms.plot) GMT_draw_map_scale (&ms);
+		if (mr.plot) GMT_draw_map_rose (&mr);
 
 		if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 		ps_plotend (gmtdefs.last_page);
