@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pscontour.c,v 1.4.4.11 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pscontour.c,v 1.49 2004/06/19 03:06:15 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -17,12 +17,12 @@
  *--------------------------------------------------------------------*/
 /* pscontour will read a file of points in the plane, performs the
  * Delaunay triangulation, and contours these triangles.  As an option
- * the user may provide a file with indeces of which vertices constitute
+ * the user may provide a file with indices of which vertices constitute
  * the triangles.
  *
  * Author:	Paul Wessel
- * Date:	1-JAN-1995
- * Version:	3.4.3
+ * Date:	13-SEP-2001
+ * Version:	4
  */
 
 #include "gmt.h"
@@ -58,44 +58,36 @@ struct PT {
 	double x, y;
 	struct PT *next;
 };
-
-struct LABEL {
-	double x, y;
-	double angle;
-	char label[64];
-	struct LABEL *next_label, *prev_label;
-} *anchor, *old_label;
 	
-int get_triangle_crossings(double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
-void draw_contour(double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap), plot_labels(int size, int box, int rgb[]);
+int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
 void paint_it (double x[], double y[], int n, double z);
 
 main(int argc, char **argv)
 {
-	int n, np, nx, i, j, ij, k, k2, k3, node1, node2, c, rgb[3];
+	int n, np, nx, i, ij, k, k2, k3, node1, node2, c, rgb[3];
 	int n_alloc, box = 1, section = 0, *ind, *vert, *cind, n_fields, n_contours;
-	int add, close, b_rgb[3], label_font_size = 9, bad, n_expected_fields, low, high, last_entry, last_exit;
+	int add, close, bad, n_expected_fields, low, high, last_entry, last_exit;
 	
-	BOOLEAN error = FALSE, image_them = FALSE, draw_contours = FALSE, dump = FALSE, clip = TRUE;
-	BOOLEAN fix_angle = FALSE, more, t_set = FALSE, draw_mesh = FALSE, use_cpt_colors = FALSE, use_cpt_anot = TRUE;
+	BOOLEAN error = FALSE, image_them = FALSE, draw_contours = FALSE, dump = FALSE, clip = TRUE, skip_if_outside = FALSE;
+	BOOLEAN more, t_set = FALSE, draw_mesh = FALSE, use_cpt_colors = FALSE, use_cpt_annot = TRUE, skip = FALSE;
 	
 	double xx[3], yy[3], zz[3], xout[5], yout[5], *in, west, east, south, north;
-	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, anot_dist, label_angle = 0.0, current_contour = -DBL_MAX;
+	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, label_angle = 0.0, current_contour = -DBL_MAX, xyz[2][3];
 	
 	char line[BUFSIZ], *cpt_file = CNULL, *t_file = CNULL, dfile[BUFSIZ];
+	char cont_label[128], format[128];
 	
 	FILE *fp = NULL, *fp_d = NULL;
 	
 	struct GMT_PEN pen, lpen;
+	struct GMT_CONTOUR G;
 
 	argc = GMT_begin (argc, argv);
 	
 	GMT_init_pen (&pen, GMT_PENWIDTH);
 	GMT_init_pen (&lpen, GMT_PENWIDTH);
-	anot_dist = 4.0; /* Distance in inches between anotations */
-	if (gmtdefs.measure_unit == GMT_CM) anot_dist = 10.0 / 2.54;	/* change to 10 cm */
 	dfile[0] = 0;
-	for (i = 0; i < 3; i++) b_rgb[i] = gmtdefs.page_rgb[i];	/* Default box color is page color */
+	GMT_contlabel_init (&G);
 	
 	/* Check and interpret the command line arguments */
 	
@@ -118,6 +110,7 @@ main(int argc, char **argv)
 				case 'x':
 				case 'Y':
 				case 'y':
+				case 'b':
 				case 'c':
 				case ':':
 				case '\0':
@@ -127,33 +120,21 @@ main(int argc, char **argv)
 				/* Supplemental parameters */
 			
 				case 'A':
-					if (argv[i][2] == '-') use_cpt_anot = FALSE;
-					for (j = 2, bad = 0; argv[i][j] && argv[i][j] != 'f'; j++);
-					if (argv[i][j])	{ /* Found font size option */
-						label_font_size = atoi (&argv[i][j+1]);
-						if (label_font_size <= 0) bad++;
-					}
-						
-					for (j = 2; argv[i][j] && argv[i][j] != 'a'; j++);
-					if (argv[i][j])	{ /* Found fixed angle option */
-						label_angle = atof (&argv[i][j+1]);
-						fix_angle = TRUE;
-						if (label_angle < -90.0 || label_angle > 180.0) bad++;
-					}
-						
-					for (j = 2; argv[i][j] && argv[i][j] != '/'; j++);
-					if (argv[i][j] && GMT_getrgb (&argv[i][j+1], b_rgb)) bad++;
-					if (strchr (argv[i], 'o')) box = 2;
+					/* Format can be one of two:
+					 * 3.4.x: -A[-][f<fontsize>][a<angle>][/<r/g/b>][t|o]
+					 * 4.x:   -A[-][+a<angle>][+c<dx>[/<dy>]][+f<size>][+g[<fill>]][+j<just>][+o][+p[<pen>]][+v]
+					 */
+					if (argv[i][2] == '-') use_cpt_annot = FALSE;
+					bad = GMT_contlabel_specs (&argv[i][2], &G);
+					G.annot = use_cpt_annot;
 					if (bad) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -A option.  Correct syntax:\n", GMT_program);
-						fprintf (stderr, "\t-A[-][f<fontsize>][a<fixedangle>][/<r/g/b>][o]\n");
+						fprintf (stderr, "\t-A[-][+a<angle>][+c<dx>[/<dy>]][+f<font>][+g[<fill>]][+j<just>][+o][+p[<pen>]][+s<size>][+u<unit>][+v]\n");
 						error += bad;
 					}
 					break;
+				break;
 
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'C':
 					cpt_file = &argv[i][2];
 					break;
@@ -165,7 +146,7 @@ main(int argc, char **argv)
 					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
 					break;
 				case 'G':
-					anot_dist = GMT_convert_units (&argv[i][2], GMT_INCH);
+					error += GMT_contlabel_info ('G', &argv[i][2], &G);
 					break;
 				case 'I':
 					image_them = TRUE;
@@ -185,6 +166,9 @@ main(int argc, char **argv)
 				case 'N':
 					clip = FALSE;
 					break;
+				case 'S':		/* Skip points outside border */
+					skip_if_outside = TRUE;
+					break;
 				case 'T':
 					t_file = &argv[i][2];
 					t_set = TRUE;
@@ -215,9 +199,10 @@ main(int argc, char **argv)
 	if (GMT_quick || argc == 1) {	/* Display usage */
 		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: pscontour <xyzfile> -C<cpt_file> -J<params> -R<west>/<east>/<south>/<north>\n");
-		fprintf (stderr, "\t[-A[-][f<fontsize>][/r/g/b][a<angle>][o]] [-B<tickinfo>] [-D<dumpfile>] [-E<az>/<el>] [-G<gap>]\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-I] [-K] [-L<pen>] [-M[<flag>]] [-N] [-O] [-P] [-T<indexfile>]\n");
-		fprintf (stderr, "\t[-U] [-V] [-W[+]<pen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
+		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [-B<tickinfo>] [-D<dumpfile>] [-E<az>/<el>]\n");
+		fprintf (stderr, "\t[-G[d|f|l|n|x]<info>] [-H[<nrec>]] [-I] [-K] [-L<pen>] [-M[<flag>]] [-N] [-O]\n");
+ 		fprintf (stderr, "\t[-P] [S-] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [-X<x_shift>] [-Y<y_shift>]\n");
+		fprintf (stderr, "\t[-c<ncopies>] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -225,16 +210,15 @@ main(int argc, char **argv)
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A Annotation format information.\n");
-		fprintf (stderr, "\t   Append - to disable all contour anotations\n");
-		fprintf (stderr, "\t   Append f followed by desired font size in points [Default is 9].\n");
-		fprintf (stderr, "\t   Append /r/g/b to change color of text box [Default is %d/%d/%d]\n", b_rgb[0], b_rgb[1], b_rgb[2]);
-		fprintf (stderr, "\t   Append o to draw outline of text box [Default is no outline]\n");
-		fprintf (stderr, "\t   Append a<angle> to force anotations at this fixed angle [Default follows contour]\n");
+		fprintf (stderr, "\t-A Annotation label information.\n");
+		fprintf (stderr, "\t   Give A- to disable all contour annotations implied in -C\n");
+		fprintf (stderr, "\t   <labelinfo> controls the specifics of the labels.  Append what you need:\n");
+		GMT_label_syntax (5, 0);
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "\t-G Gap between anotations in %s [Default = %lg]\n", GMT_unit_names[gmtdefs.measure_unit], anot_dist);
+		fprintf (stderr, "\t-G Controls placement of labels along contours.  Choose among five algorithms:\n");
+		GMT_cont_syntax (3, 0);
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I Color triangles using the cpt file\n");
 		GMT_explain_option ('K');
@@ -244,8 +228,9 @@ main(int argc, char **argv)
 		fprintf (stderr,"\t-N do NOT clip contours/image at the border [Default clips]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr,"\t-T file with triplets of point indeces for each triangle\n");
-		fprintf (stderr,"\t   [Default performs the Delauney triangulation on xyz-data]\n");
+		fprintf (stderr,"\t-S Skip xyz points outside region [Default keeps all]\n");
+		fprintf (stderr,"\t-T file with triplets of point indices for each triangle\n");
+		fprintf (stderr,"\t   [Default performs the Delaunay triangulation on xyz-data]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W selects contouring and sets contour pen attributes\n");
@@ -257,11 +242,12 @@ main(int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr,"\t   Default is 3 input columns.\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
@@ -283,26 +269,26 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 3;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 3) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 3;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 3) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 3 columns\n", GMT_program);
 		error++;
 	}
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
-	if (GMT_io.binary[1] && GMT_io.multi_segments) {
+	if (GMT_io.binary[GMT_OUT] && GMT_io.multi_segments) {
 		fprintf (stderr, "%s: GMT Warning.  -M for output ignored with -D, -bo\n", GMT_program);
 		error++;
 	}
@@ -328,7 +314,7 @@ main(int argc, char **argv)
 	
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi,
-		GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 		
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
@@ -339,19 +325,20 @@ main(int argc, char **argv)
 	if (fp == NULL) {
 		fp = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
-	
+
 	n_alloc = GMT_CHUNK;
 	x = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 	
+	xyz[0][2] = DBL_MAX;	xyz[1][2] = -DBL_MAX;
 	n = 0;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 3;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
 	n_fields = GMT_input (fp, &n_expected_fields, &in);
 		
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -361,11 +348,26 @@ main(int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 
-		if (!GMT_is_dnan (in[2])) {	/* Unless z = NaN */
+		if (skip_if_outside) {	/* Must check if points are inside plot region */
+			GMT_map_outside (in[0], in[1]);
+			skip = ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1);
+		}
+		
+		if (!(skip || GMT_is_dnan (in[2]))) {	/* Unless outside or z = NaN */
 		
 			x[n] = in[0];
 			y[n] = in[1];
 			z[n] = in[2];
+			if (z[n] < xyz[1][2]) {	/* New minimum */
+				xyz[0][0] = x[n];
+				xyz[0][1] = y[n];
+				xyz[0][2] = z[n];
+			}
+			if (z[n] > xyz[1][2]) {	/* New maximum */
+				xyz[1][0] = x[n];
+				xyz[1][1] = y[n];
+				xyz[1][2] = z[n];
+			}
 			n++;
 		
 			if (n == n_alloc) {
@@ -384,6 +386,8 @@ main(int argc, char **argv)
 	y = (double *) GMT_memory ((void *)y, (size_t)n, sizeof (double), GMT_program);
 	z = (double *) GMT_memory ((void *)z, (size_t)n, sizeof (double), GMT_program);
 	
+	if (GMT_contlabel_prep (&G, xyz, 1)) exit (EXIT_FAILURE);	/* Prep for crossing lines, if any */
+
 	/* Map transform */
 	
 	for (i = 0; i < n; i++) GMT_geo_to_xy (x[i], y[i], &x[i], &y[i]);
@@ -394,20 +398,20 @@ main(int argc, char **argv)
 
 		ij = np = 0;
 		
-		if (GMT_io.binary[0])	/* Binary input */
+		if (GMT_io.binary[GMT_IN])	/* Binary input */
 			more = (fread ((void *)ind, sizeof (int), (size_t)3, fp_d) == 3);
 		else		/* ascii input */
 			more = (fgets (line, BUFSIZ, fp_d) != CNULL);
 			
 		while (more) {
-			if (!GMT_io.binary[0] && sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]) != 3) continue;
+			if (!GMT_io.binary[GMT_IN] && sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]) != 3) continue;
 			ij += 3;
 			np++;
 			if (ij == n_alloc) {
 				n_alloc += (3 * GMT_CHUNK);
 				ind = (int *) GMT_memory ((void *)ind, (size_t)n_alloc, sizeof (int), GMT_program);
 			}
-			if (GMT_io.binary[0])	/* Binary input */
+			if (GMT_io.binary[GMT_IN])	/* Binary input */
 				more = (fread ((void *)&ind[ij], sizeof (int), (size_t)3, fp_d) == 3);
 			else		/* ascii input */
 				more = (fgets (line, BUFSIZ, fp_d) != CNULL);
@@ -415,7 +419,7 @@ main(int argc, char **argv)
 		ind = (int *) GMT_memory ((void *)ind, (size_t)ij, sizeof (int), GMT_program);
 		fclose (fp_d);
 	}
-	else	/* Do Delauney triangulation */
+	else	/* Do Delaunay triangulation */
 			
 		np = GMT_delaunay (x, y, n, &ind);
 	
@@ -440,18 +444,18 @@ main(int argc, char **argv)
 	
 	for (i = 0; i < GMT_n_colors; i++) {
 		cont[i].val = GMT_lut[i].z_low;
-		cont[i].type = (GMT_lut[i].anot && use_cpt_anot) ? 'A' : 'C';
-		cont[i].angle = (fix_angle) ? label_angle : GMT_d_NaN;
+		cont[i].type = (GMT_lut[i].annot && use_cpt_annot) ? 'A' : 'C';
+		cont[i].angle = (G.angle_type == 2) ? label_angle : GMT_d_NaN;
 	}
 	cont[GMT_n_colors].val = GMT_lut[GMT_n_colors-1].z_high;
-	cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].anot & 2) && use_cpt_anot) ? 'A' : 'C';
-	cont[GMT_n_colors].angle = (fix_angle) ? label_angle : GMT_d_NaN;
+	cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].annot & 2) && use_cpt_annot) ? 'A' : 'C';
+	cont[GMT_n_colors].angle = (G.angle_type == 2) ? label_angle : GMT_d_NaN;
 	for (i = 0; i < n_contours; i++) {
 		cont[i].n_alloc = GMT_SMALL_CHUNK;
 		cont[i].L = (struct PSLINE *) GMT_memory (VNULL, (size_t)GMT_SMALL_CHUNK, sizeof (struct PSLINE), GMT_program);
 	}
 	
-	GMT_setpen (&pen);
+	G.line_pen = pen;
 
 	for (i = ij = 0; i < np; i++, ij += 3) {	/* For all triangles */
 			
@@ -476,7 +480,7 @@ main(int argc, char **argv)
 			}
 			else {	/* Must paint all those slices separately */
 			
-				/* Find vertecies with the lowest and highest values */
+				/* Find vertices with the lowest and highest values */
 				
 				for (k = 1, low = high = 0; k < 3; k++) {
 					if (zz[k] < zz[low])   low = k;
@@ -492,7 +496,7 @@ main(int argc, char **argv)
 					xout[2] = xc[1];	yout[2] = yc[1];
 					n = 3;
 				}
-				else {	/* Need twe other two vertices to form a 4-sided polygon */
+				else {	/* Need the other two vertices to form a 4-sided polygon */
 					node2 = get_other_node (node1, low);	/* The other node needed */
 					xout[1] = xx[node2];	yout[1] = yy[node2];
 					if (low == vert[0] || node2 == vert[1]) {	/* Add segment in opposite order */
@@ -612,8 +616,6 @@ main(int argc, char **argv)
 		struct CHAIN *head_c, *last_c, *this_c;
 		struct PT *p, *q;
 		
-		anchor = old_label = (struct LABEL *) GMT_memory (VNULL, (size_t)1, sizeof (struct LABEL), GMT_program);
-
 		for (c = 0; c < n_contours; c++) {
 		
 			if (cont[c].nl == 0) {
@@ -710,24 +712,30 @@ main(int argc, char **argv)
 					if (current_contour != cont[c].val) {
 						GMT_get_rgb24 (cont[c].val, rgb);
 						ps_setpaint (rgb);
+						memcpy ((void *)&G.line_pen.rgb, (void *)rgb, (size_t)(3 * sizeof (int)));
+						if (!G.got_font_rgb && G.curved_text) memcpy ((void *)&G.font_rgb, (void *)rgb, (size_t)(3 * sizeof (int)));
 						current_contour = cont[c].val;
 					}
 				}
 
-				draw_contour (xp, yp, n, cont[c].val, cont[c].type, cont[c].angle, close, anot_dist);
+				if (cont[c].type == 'A' || cont[c].type == 'a') {	/* Annotated contours */
+					GMT_get_format (cont[c].val, G.unit, CNULL, format);
+					sprintf (cont_label, format, cont[c].val);
+				}
 				if (dump) GMT_dump_contour (xp, yp, n, cont[c].val, section++, close, dfile);
+				GMT_hold_contour (&xp, &yp, n, cont_label, cont[c].type, cont[c].angle, close, &G);
 
 				GMT_free ((void *)xp);
 				GMT_free ((void *)yp);
 			}
 					
 		}
-		plot_labels (label_font_size, box, b_rgb);
+		GMT_contlabel_plot (&G);
 	}
 							
         if (clip) GMT_map_clip_off ();
 
-	if (frame_info.plot) GMT_map_basemap ();
+	GMT_map_basemap ();
 	
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
@@ -805,7 +813,7 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 				vout[j++] = i;	/* Keep track of the side number */
 			}
 		}
-		if (j%2) j--;	/* Contour went through a single vertice only, skip this */
+		if (j%2) j--;	/* Contour went through a single vertex only, skip this */
 	}
 	
 	*xc = xout;
@@ -817,91 +825,6 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	return (j/2);
 }
 
-void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap)
-{
-	int i;
-	double dist, angle, dx, dy, width;
-	char label[128], format[64];
-	struct LABEL *new_label;
-	
-	if (nn < 2) return;
-		
-	sprintf (format, "%lg contour", cval);
-	ps_comment (format);
-	
-	ps_line (xx, yy, nn, 3, closed, TRUE);
-	
-	if (ctype == 'A' || ctype == 'a') {	/* Annotated contours */
-		GMT_get_format (cval, CNULL, format);
-		sprintf (label, format, cval);
-		dist = (closed) ? gap - 1.0: 0.0;	/* Label closed contours longer than 1 inch */
-		for (i = 1; i < nn; i++) {
-
-			dx = xx[i] - xx[i-1];
-			if (fabs (dx) > (width = GMT_half_map_width (yy[i-1]))) {
-				width *= 2.0;
-				dx = copysign (width - fabs (dx), -dx);
-				if (xx[i] < width)
-					xx[i-1] -= width;
-				else
-					xx[i-1] += width;
-			}
-			dy = yy[i] - yy[i-1];
-			dist += hypot (dx, dy);
-			if (dist > gap) {	/* Time for label */
-				new_label = (struct LABEL *) GMT_memory (VNULL, (size_t)1, sizeof (struct LABEL), GMT_program);
-				new_label->x = 0.5 * (xx[i-1] + xx[i]);
-				new_label->y = 0.5 * (yy[i-1] + yy[i]);
-				strcpy (new_label->label, label);
-				if (GMT_is_dnan (cangle)) {	/* Must calculate label angle */
-					angle = d_atan2 (dy, dx) * R2D;
-					if (angle < 0.0) angle += 360.0;
-					if (angle > 90.0 && angle < 270) angle -= 180.0;
-				}
-				else
-					angle = cangle;
-				new_label->angle = angle;
-				new_label->prev_label = old_label;
-				dist = 0.0;
-				old_label->next_label = new_label;
-				old_label = new_label;
-			}
-		}
-	}
-}
-
-void plot_labels (int size, int box, int rgb[])
-{
-	/* box = 1: white box only, box = 2: white box + draw outline */
-	double dx, dy;
-	struct LABEL *this, *old;
-	
-	dx = 0.5 * gmtdefs.anot_offset;
-	dy = 0.05 * gmtdefs.anot_offset;
-	box--;
-	ps_comment ("Contour annotations:");
-	
-	ps_setpaint (gmtdefs.basemap_frame_rgb);
-	
-	for (old = anchor; old->next_label; old = old->next_label) {	/* First draw boxes if not 3-D*/
-		this = old->next_label;
-		GMT_textbox3d (this->x, this->y, project_info.z_level, size, gmtdefs.anot_font, this->label, this->angle, 6, box, dx, dy, rgb);
-	}
-	for (old = anchor; old->next_label; old = old->next_label) { /* Then labels */
-		this = old->next_label;
-		GMT_text3d (this->x, this->y, project_info.z_level, size, gmtdefs.anot_font, this->label, this->angle, 6, 0);
-	}
-		
-	ps_setpaint (gmtdefs.background_rgb);
-	
-	this = anchor;
-	while (this) {	/* Free memory */
-		old = this;
-		this = old->next_label;
-		GMT_free ((void *)old);
-	}
-}
-
 void paint_it (double x[], double y[], int n, double z) {
 	int index, k, rgb[3];
 	struct GMT_FILL *f;
@@ -915,7 +838,7 @@ void paint_it (double x[], double y[], int n, double z) {
 	
 	if (project_info.three_D)
 		for (k = 0; k < n; k++) GMT_xy_do_z_to_xy (x[k], y[k], project_info.z_level, &x[k], &y[k]);
-	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn.fill[index+3]))) {
+	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {
 		GMT_fill (x, y, n, f, FALSE);	/* Contours drawn separately later if desired */
 	}
 	else
