@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pshistogram.c,v 1.4.4.7 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pshistogram.c,v 1.20 2004/08/17 01:48:06 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -27,7 +27,7 @@
  * Updated to v3.1 13-Jun-1998 Paul Wessel
  * Updated to v3.3 23-Apr-1999 Paul Wessel
  *		     Now uses two passes to get accurate mean and stdev
- * Version:	3.4.3
+ * Version:	4
  */
 
 #include "gmt.h"
@@ -65,6 +65,7 @@ int read_data(void);
 int fill_boxes(void);
 int plot_boxes(BOOLEAN stairs, BOOLEAN flip_to_y);
 int get_loc_scl(float *x, int n, float *stats);
+int GMT_mode_selection = 0, GMT_n_multiples = 0;
 
 main (int argc, char **argv)
 {
@@ -97,16 +98,15 @@ main (int argc, char **argv)
 				case 'x':
 				case 'Y':
 				case 'y':
+				case 'b':
 				case 'c':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
 
 				/* Supplemental parameters */
 			
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case '2':
 					second = TRUE;
 					break;
@@ -148,13 +148,13 @@ main (int argc, char **argv)
 
 					strcpy (string, &argv[i][2]);
 					text = strtok (string, "/");
-					west = GMT_ddmmss_to_degree (text);
+					GMT_scanf (text, GMT_IS_GEO, &west);
 					text = strtok (CNULL, "/");
-					east = GMT_ddmmss_to_degree (text);
+					GMT_scanf (text, GMT_IS_GEO, &east);
 					text = strtok (CNULL, "/");
-					south = GMT_ddmmss_to_degree (text);
+					GMT_scanf (text, GMT_IS_GEO, &south);
 					text = strtok (CNULL, "/");
-					north = GMT_ddmmss_to_degree (text);
+					GMT_scanf (text, GMT_IS_GEO, &north);
 					break;
 				case 'W':
 					box_width = atof (&argv[i][2]);
@@ -180,7 +180,7 @@ main (int argc, char **argv)
 		fprintf (stderr,"pshistogram %s - Calculate and plot histograms\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pshistogram [file] -Jx|X<scale> -W<width> [-2] [-B<tickinfo>] [-C] [-Eaz/el] [-G<fill>]\n");
 		fprintf (stderr, "\t[-H[<nrec>]] [-I[o]] [-K] [-L<pen>] [-O] [-P] [-Q] [-Rw/e/s/n] [-S] [-U[text]] [-V]\n");
-		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z[type]] [-c<ncopies>] [-bi[s][<n>]]\n\n");
+		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z[type]] [-c<ncopies>] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		fprintf (stderr, "\t-Jx|X for linear projection.  Scale in %s/units (or width in %s)\n", GMT_unit_names[gmtdefs.measure_unit], GMT_unit_names[gmtdefs.measure_unit]);
@@ -222,6 +222,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 
 		exit (EXIT_FAILURE);
@@ -249,23 +250,23 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Must specify either fill color (-G), outline pen attributes (-L), or both.\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 0) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 0) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Must specify number of columns in binary input data (-bi)\n", GMT_program);
 		error++;
 	}
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (inquire) gmtdefs.verbose = TRUE;
@@ -274,7 +275,7 @@ main (int argc, char **argv)
 	if (fp_in == NULL) {
 		fp_in = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
 
@@ -291,16 +292,18 @@ main (int argc, char **argv)
 	}
 	
 	if (west == east) {	/* Set automatic x range [ and tickmarks] */
-		if (frame_info.anot_int[0] == 0.0) {
+		if (frame_info.axis[0].item[0].interval == 0.0) {
 			tmp = pow (10.0, floor (d_log10 (x_max-x_min)));
 			if (((x_max-x_min) / tmp) < 3.0) tmp *= 0.5;
 		}
 		else
-			tmp = frame_info.anot_int[0];
+			tmp = frame_info.axis[0].item[0].interval;
 		west = floor (x_min / tmp) * tmp;
 		east = ceil (x_max / tmp) * tmp;
-		if (frame_info.anot_int[0] == 0.0) {
-			frame_info.anot_int[0] = frame_info.frame_int[0] = tmp;
+		if (frame_info.axis[0].item[0].interval == 0.0) {
+			frame_info.axis[0].item[0].interval = frame_info.axis[0].item[4].interval = tmp;
+			frame_info.axis[0].item[0].parent = 0;
+			frame_info.axis[0].item[0].active = TRUE;
 			frame_info.plot = TRUE;
 		}
 	}
@@ -347,15 +350,17 @@ main (int argc, char **argv)
 	if (automatic) {	/* Set up s/n based on 'clever' rounding up of the minmax values */
 		project_info.region = TRUE;
 		south = 0.0;
-		if (frame_info.anot_int[1] == 0.0) {
+		if (frame_info.axis[1].item[0].interval == 0.0) {
 			tmp = pow (10.0, floor (d_log10 (yy1)));
 			if ((yy1 / tmp) < 3.0) tmp *= 0.5;
 		}
 		else
-			tmp = frame_info.anot_int[1];
+			tmp = frame_info.axis[1].item[0].interval;
 		north = ceil (yy1 / tmp) * tmp;
-		if (frame_info.anot_int[1] == 0.0) {	/* Tickmarks not set */
-			frame_info.anot_int[1] = frame_info.frame_int[1] = tmp;
+		if (frame_info.axis[1].item[0].interval == 0.0) {	/* Tickmarks not set */
+			frame_info.axis[1].item[0].interval = frame_info.axis[1].item[4].interval = tmp;
+			frame_info.axis[1].item[0].parent = 1;
+			frame_info.axis[1].item[0].active = TRUE;
 			frame_info.plot = TRUE;
 		}
 		if (project_info.pars[0] == 0.0) {	/* Must give default xscale */
@@ -370,18 +375,18 @@ main (int argc, char **argv)
 	
 	if (automatic && gmtdefs.verbose) {
 		sprintf (format, "%%s: Use w/e/s/n = %s/%s/%s/%s and x-tick/y-tick = %s/%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (stderr, format, GMT_program, west, east, south, north, frame_info.anot_int[0], frame_info.anot_int[1]);
+		fprintf (stderr, format, GMT_program, west, east, south, north, frame_info.axis[0].item[0].interval, frame_info.axis[1].item[0].interval);
 	}
 
 	if (flip_to_y) {
 		char buffer[128];
-		d_swap (frame_info.anot_int[0], frame_info.anot_int[1]);
-		d_swap (frame_info.frame_int[0], frame_info.frame_int[1]);
-		d_swap (frame_info.grid_int[0], frame_info.grid_int[1]);
-		text = frame_info.label[0];
-		strcpy (buffer, frame_info.label[0]);
-		strcpy(frame_info.label[0], frame_info.label[1]);
-		strcpy (frame_info.label[1], buffer);
+		d_swap (frame_info.axis[0].item[0].interval, frame_info.axis[1].item[0].interval);
+		d_swap (frame_info.axis[0].item[4].interval, frame_info.axis[1].item[4].interval);
+		d_swap (frame_info.axis[0].item[5].interval, frame_info.axis[1].item[5].interval);
+		text = frame_info.axis[0].label;
+		strcpy (buffer, frame_info.axis[0].label);
+		strcpy(frame_info.axis[0].label, frame_info.axis[1].label);
+		strcpy (frame_info.axis[1].label, buffer);
 		GMT_map_setup (south, north, west, east);
 	}
 	else
@@ -389,7 +394,7 @@ main (int argc, char **argv)
 
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 	
@@ -403,7 +408,7 @@ main (int argc, char **argv)
 	
 	GMT_map_clip_off();
 	
-	if (frame_info.plot) GMT_map_basemap ();
+	GMT_map_basemap ();
 
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 	ps_plotend (gmtdefs.last_page);
@@ -422,9 +427,9 @@ int	read_data (void) {
 	
 	n = 0;
 	x_min = DBL_MAX;	x_max = -DBL_MAX;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 1 + second;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 1 + second;
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
 	
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 	
@@ -602,7 +607,7 @@ int	plot_boxes (BOOLEAN stairs, BOOLEAN flip_to_y)
 		}
 	}
 
-	if (stairs) ps_plot (px[1], py[1], 2);
+	if (stairs) ps_plot (px[1], py[1], -2);
 	
 	return(0);
 }
@@ -624,8 +629,9 @@ int	get_loc_scl (float *x, int n, float *stats)
 	stats[1] = (float)median;
 
 	/* Get mode */
-	GMT_mode_f (x, n, j, 0, &mode);
+	GMT_mode_f (x, n, j, 0, GMT_mode_selection, &GMT_n_multiples, &mode);
 	stats[2] = (float)mode;
+	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, GMT_n_multiples);
 	
 	/* Get MAD for L1 */
 	GMT_getmad_f (x, n, median, &mad);
