@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pslib_inc.h,v 1.3.4.7 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pslib_inc.h,v 1.25 2004/09/03 20:26:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -20,8 +20,8 @@
  * include file pslib.h
  *
  * Author:	Paul Wessel
- * Date:	10-OCT-2000
- * Version:	3.4.3
+ * Date:	02-JUN-2004
+ * Version:	4.1
  */
 
 #ifndef _PSLIB_INC_H
@@ -55,10 +55,11 @@
 
 #include <ctype.h>
 #include <float.h>
+#include <limits.h>
 #include <math.h>
 #include <stddef.h>
 #ifdef __MACHTEN__
-/* Kludge to fix a Macthen POSIX bug */
+/* Kludge to fix a Machten POSIX bug */
 #include <sys/types.h>
 #endif
 #include <stdio.h>
@@ -76,7 +77,7 @@
  *			PSL CONSTANTS MACRO DEFINITIONS
  *--------------------------------------------------------------------*/
 
-#define PSL_Version "3.4"
+#define PSL_Version "4.1"
 #ifndef R2D
 #define R2D (180.0/M_PI)
 #endif
@@ -100,7 +101,6 @@
 #ifndef CNULL
 #define CNULL (char *)NULL
 #endif
-#define PS_FIRST_JAPANESE_FONT 35
 /*--------------------------------------------------------------------
  *			PSL FUNCTION MACRO DEFINITIONS
  *--------------------------------------------------------------------*/
@@ -117,44 +117,22 @@
 #define YIQ(rgb) irint (0.299 * (rgb[0]) + 0.587 * (rgb[1]) + 0.114 * (rgb[2]))	/* How B/W TV's convert RGB to Gray */
 #define iscolor(rgb) (rgb[0] != rgb[1] || rgb[1] != rgb[2])
 
-#define N_FONTS	39	/* Current no of fonts;  To add more, modify the file
-			 * PS_font_names.h and PS_font_heights.h and set N_FONTS */
+int N_PS_FONTS;		/* Total no of fonts;  To add more, modify the file CUSTOM_font_info.d */
+int N_GMT_FONTS;	/* Total no of standard GMT fonts. */
 
 typedef int BOOLEAN;		/* BOOLEAN used for logical variables */
 
-struct EPS {    /* Holds info for eps files */
-        int x0, x1, y0, y1;     /* Bounding box values in points */
-	int portrait;		/* TRUE if start of plot was portrait */
-        char *font[N_FONTS];    /* Pointers to font names used */
-        int fontno[N_FONTS];    /* Array with font ids */
-        char *name;             /* User name */
-        char *title;            /* Plot title */
-};
-
-struct rasterfile {
-        int     ras_magic;              /* magic number */
-        int     ras_width;              /* width (pixels) of image */
-        int     ras_height;             /* height (pixels) of image */
-        int     ras_depth;              /* depth (1, 8, or 24 bits) of pixel */
-        int     ras_length;             /* length (bytes) of image */
-        int     ras_type;               /* type of file; see RT_* below */
-        int     ras_maptype;            /* type of colormap; see RMT_* below */
-        int     ras_maplength;          /* length (bytes) of following map */
-        /* color map follows for ras_maplength bytes, followed by image */
-};
-
-#define	RAS_MAGIC	0x59a66a95	/* Magic number for Sun rasterfile */
-#define RT_OLD		0	/* Old-style, unencoded Sun rasterfile */
-#define RT_STANDARD	1	/* Standard, unencoded Sun rasterfile */
-#define RT_BYTE_ENCODED	2	/* Run-length-encoded Sun rasterfile */
-#define RT_FORMAT_RGB	3	/* [X]RGB instead of [X]BGR Sun rasterfile */
-#define RMT_NONE	0       /* ras_maplength is expected to be 0 */
-#define RMT_EQUAL_RGB	1       /* red[ras_maplength/3], green[], blue[] follow */
-
 /*--------------------------------------------------------------------
  *			PSL PARAMETERS DEFINITIONS
  *--------------------------------------------------------------------*/
 
+struct PS_FONT {
+	char *name;		/* Name of this font */
+	double height;		/* Height of A for unit fontsize */ 
+	BOOLEAN encoded;	/* TRUE if we never should reencode this font (e.g. symbols) */
+				/* This is also changed to TRUE after we do reencode a font */
+};
+	
 /* Global structure used internally by pslib */
 
 struct {
@@ -166,8 +144,10 @@ struct {
 	BOOLEAN landscape;	/* TRUE = Landscape, FALSE = Portrait			*/
 	int ix, iy;		/* Absolute coordinates of last point		*/
 	int npath;		/* Length of current unstroked path		*/
+	int split;		/* 1 if last ps_line had to split path		*/
 	char bw_format[8];	/* Format used for grayshade value		*/
-	char rgb_format[24];	/* Same, for color triplets			*/
+	char rgb_format[64];	/* Same, for RGB color triplets			*/
+	char cmyk_format[64];	/* Same, for CMYK color quadruples		*/
 	double xscl, yscl;	/* Global scale values [1.0]			*/
 	double xoff, yoff;	/* Origin offset [1/1]				*/
 	double scale;		/* Must be set through plotinit();		*/
@@ -178,23 +158,29 @@ struct {
 	int max_path_length;	/* Maximum path length encountered during execution	*/
 	int clip_path_length;	/* Current length of clip path			*/
 	int hex_image;		/* TRUE writes images in hex, FALSE uses binary	*/
+	int cmyk_mode;		/* TRUE writes all color calls in cmyk, FALSE uses rgb	*/
 	int absolute;		/* TRUE will reset origin, FALSE means relative position	*/
 	int eps_format;		/* TRUE makes EPS file, FALSE means PS file	*/
-	BOOLEAN font_used[N_FONTS];	/* TRUE if the font has been reencoded */
-	BOOLEAN euro;		/* TRUE when we must reencode fonts on demand */
+	char *encoding;		/* The encoding name. e.g. ISO-8859-1 */
+	struct PS_FONT *font;	/* Pointer to array of font structures 		*/
 } ps;
 
 char *PSHOME = (char *)NULL;	/* Pointer to path of directory with lib subdirectory */
 
-	/* Font information */
-	
-
-double ps_font_height[N_FONTS] = {	/* Based on the size of A devided by fontsize */
-#include "PS_font_heights.h"
-};
-
-char *ps_font_name[N_FONTS] = {
-#include "PS_font_names.h"
+char *ps_scandcodes[13][4] = {	/* Short-hand converrsion for Norwegian characters in both Standard [0], Standard+ [1], ISOLatin1 [2], and ISOLatin1+ [2] encoding */
+	{ "AA", "\\375", "\\305", "\\305"},	/* Aring */
+	{ "AE", "\\341", "\\306", "\\306"},	/* AE */
+	{ "OE", "\\351", "\\330", "\\330"},	/* Oslash */
+	{ "aa", "\\376", "\\345", "\\345"},	/* aring */
+	{ "ae", "\\372", "\\346", "\\346"},	/* ae */
+	{ "oe", "\\371", "\\370", "\\370"},	/* oslash */
+	{ "C", "\\201", "\\307", "\\307"},	/* Ccedilla */
+	{ "N", "\\204", "\\321", "\\321"},	/* Ntilde */
+	{ "U", "\\335", "\\334", "\\334"},	/* Udieresis */
+	{ "c", "\\215", "\\347", "\\347"},	/* ccedilla */
+	{ "n", "\\227", "\\36", "\\361"},	/* ntilde */
+	{ "\\373", "\\373", "\\337", "\\337"},	/* germandbls */
+	{ "u", "\\370", "\\374", "\\374"}	/* udieresis */
 };
 
 	/* Pattern information */
@@ -213,4 +199,7 @@ struct USERIMAGE {
 
 int no_rgb[3] = {-1, -1, -1};
 
+int ps_outline_offset[4] = {0, 2, 5, 3};
+char ps_paint_code[4] = {'A', 'C', 'K', ' '};
+
 #endif	/* _PSLIB_INC_H */
