@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.4.4.7 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: psrose.c,v 1.30 2004/08/17 01:48:06 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -28,7 +28,7 @@
  *
  * Author:	Paul Wessel
  * Date:	20-JUN-2000
- * Version:	3.4.3
+ * Version:	4
  */
  
 #include "gmt.h"
@@ -46,7 +46,7 @@ void arc3d(double x, double y, double r, double a1, double a2, int status);
 
 main (int argc, char **argv)
 {
-	int i, n_bins, bin, n_anot, n = 0, a_rgb[3];
+	int i, n_bins, bin, n_annot, n = 0, a_rgb[3];
 	int status, n_alpha, n_modes, n_fields, n_expected_fields;
 	
 	BOOLEAN error = FALSE, inquire = FALSE, outline = FALSE, find_mean = FALSE, normalize = FALSE;
@@ -60,7 +60,7 @@ main (int argc, char **argv)
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, asize, lsize, zscale = 1.0;
 	
 	char text[BUFSIZ], lw[10], le[10], ln[10], *file = CNULL, format[BUFSIZ];
-	char *choice[2] = {"OFF", "ON"}, txt_a[32], txt_b[32], txt_c[32];
+	char *choice[2] = {"OFF", "ON"}, txt_a[32], txt_b[32], txt_c[32], txt_d[64];
 	
 	FILE *fp = NULL, *fpm = NULL;
 	
@@ -79,8 +79,8 @@ main (int argc, char **argv)
 	a_rgb[0] = a_rgb[1] = a_rgb[2] = 0;
 	GMT_init_fill (&fill, -1, -1, -1);
 	GMT_init_pen (&pen, GMT_PENWIDTH);
-	asize = gmtdefs.anot_font_size * GMT_u2u[GMT_PT][GMT_INCH];
-	lsize = gmtdefs.label_font_size * GMT_u2u[GMT_PT][GMT_INCH];
+	asize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
+	lsize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 	strcpy (lw, "WEST");	strcpy (le, "EAST");	strcpy (ln, "NORTH");
 	mode_direction = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
 	mode_length = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
@@ -105,6 +105,7 @@ main (int argc, char **argv)
 				case 'x':
 				case 'Y':
 				case 'y':
+				case 'b':
 				case 'c':
 				case ':':
 				case '\0':
@@ -114,9 +115,6 @@ main (int argc, char **argv)
 					
 				/* Supplemental options */
 				
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'A':		/* Get Sector angle in degrees */
 					sector = atof (&argv[i][2]);
 					if (argv[i][strlen (argv[i])-1] == 'r') rose = TRUE;
@@ -139,14 +137,10 @@ main (int argc, char **argv)
 					break;
 				case 'M':		/* Get arrow parameters */
 					if (argv[i][2]) {
-						n = sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%d/%d/%d", txt_a, txt_b, txt_c, &a_rgb[0], &a_rgb[1], &a_rgb[2]);
-						if (n != 6) {
+						n = sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%s", txt_a, txt_b, txt_c, txt_d);
+						if (n != 4 || GMT_getrgb (txt_d, a_rgb)) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -M option.  Correct syntax:\n", GMT_program);
-							fprintf (stderr, "\t-M<tailwidth/headlength/headwidth/r/g/b>\n");
-							error = TRUE;
-						}
-						else if (GMT_check_rgb (a_rgb)) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -M option.  RGB must all be in 0-255 range\n", GMT_program);
+							fprintf (stderr, "\t-M<tailwidth/headlength/headwidth/<color>>\n");
 							error = TRUE;
 						}
 						else {
@@ -219,7 +213,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-R specifies the region.  (r0 = 0, r1 = max_radius.  For azimuth:\n");
 		fprintf (stderr, "\t   Specify theta0/theta1 = -90/90 (half-circle) or 0/360 only)\n");
 		fprintf (stderr, "\t   If r0 = r1 = 0, psrose will compute reasonable a r1 value.\n");
-		fprintf (stderr, "\t-S specifies the radius of the unit circle in %s [%lg]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], scale);
+		fprintf (stderr, "\t-S specifies the radius of the unit circle in %s [%g]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], scale);
 		fprintf (stderr, "\t-T indicates that the vectors are oriented (two-headed), not directed [Default]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
@@ -227,7 +221,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z multiply the radii by scale before plotting\n");
 		GMT_explain_option ('c');
-		fprintf (stderr, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle]);
+		fprintf (stderr, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle[0]]);
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns.\n");
@@ -235,7 +229,7 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
@@ -257,23 +251,23 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -R option:  theta0/theta1 must be either -90/90 or 0/360\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 2) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 2) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (file && (fp = GMT_fopen (file, GMT_io.r_mode)) == NULL) {
@@ -284,7 +278,7 @@ main (int argc, char **argv)
 	if (fp == NULL) {
 		fp = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
 	
@@ -313,12 +307,12 @@ main (int argc, char **argv)
 	azimuth = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	length = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
+	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
 		
 	/* Read data and do some stats */
 	
 	n = 0;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 	
 	n_fields = GMT_input (fp,  &n_expected_fields, &in);
 
@@ -403,20 +397,20 @@ main (int argc, char **argv)
 	}
 	
 	if (automatic) {
-		if (fabs (frame_info.anot_int[0]) < GMT_CONV_LIMIT) {
+		if (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) {
 			tmp = pow (10.0, floor (d_log10 (max)));
 			if ((max / tmp) < 3.0) tmp *= 0.5;
 		}
 		else
-			tmp = frame_info.anot_int[0];
+			tmp = frame_info.axis[0].item[GMT_ANNOT_UPPER].interval;
 		max_radius = ceil (max / tmp) * tmp;
-		if (fabs (frame_info.anot_int[0]) < GMT_CONV_LIMIT || fabs (frame_info.grid_int[0]) < GMT_CONV_LIMIT) {	/* Tickmarks not set */
-			frame_info.anot_int[0] = frame_info.grid_int[0] = tmp;
+		if (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT || frame_info.axis[0].item[GMT_GRID_UPPER].interval < GMT_CONV_LIMIT) {	/* Tickmarks not set */
+			frame_info.axis[0].item[GMT_ANNOT_UPPER].interval = frame_info.axis[0].item[GMT_GRID_UPPER].interval = tmp;
 			frame_info.plot = TRUE;
 		}
 	}
 	
-	if (frame_info.plot && fabs (frame_info.anot_int[1]) < GMT_CONV_LIMIT) frame_info.anot_int[1] = frame_info.grid_int[1] = 30.0;
+	if (frame_info.plot && frame_info.axis[1].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) frame_info.axis[1].item[GMT_ANNOT_UPPER].interval = frame_info.axis[1].item[GMT_GRID_UPPER].interval = 30.0;
 	
 	/* Ready to plot */
 	
@@ -427,7 +421,7 @@ main (int argc, char **argv)
 	
 	ps_plotinit (NULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 	
@@ -444,7 +438,7 @@ main (int argc, char **argv)
 			x = radius * cos (angle);
 			y = radius * sin (angle);
 			plot3d (0.0, 0.0, 3);
-			plot3d (x, y, 2);
+			plot3d (x, y, -2);
 		}
 	}
 	
@@ -473,7 +467,7 @@ main (int argc, char **argv)
 			yy[i] = 0.0;
 			i++;
 		}
-		if (project_info.three_D) GMT_2D_to_3D (xx, yy, i);
+		if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, i);
 		ps_polygon (xx, yy, i, fill.rgb, outline);
 	}
 	
@@ -530,74 +524,82 @@ main (int argc, char **argv)
 			sincos (angle, &s, &c);
 			xr = scale * mode_length[i] * c;
 			yr = scale * mode_length[i] * s;
-			GMT_vector3d (0.0, 0.0, xr, yr, project_info.z_level, tailwidth, headlength, headwidth, gmtdefs.vector_shape, a_rgb, TRUE);
+			GMT_vector3D (0.0, 0.0, xr, yr, project_info.z_level, tailwidth, headlength, headwidth, gmtdefs.vector_shape, a_rgb, TRUE);
 		}
 		
 	}
 	
 	if (frame_info.plot) {	/* Draw grid lines etc */
-		GMT_setpen (&gmtdefs.grid_pen);
+		GMT_setpen (&gmtdefs.grid_pen[0]);
 		off = max_radius * scale;
-		n_alpha = (frame_info.grid_int[1] > 0.0) ? irint (total_arc / frame_info.grid_int[1]) : -1;
+		n_alpha = (frame_info.axis[1].item[GMT_GRID_UPPER].interval > 0.0) ? irint (total_arc / frame_info.axis[1].item[GMT_GRID_UPPER].interval) : -1;
 		for (i = 0; i <= n_alpha; i++) {
-			angle = D2R * i * frame_info.grid_int[1];
+			angle = D2R * i * frame_info.axis[1].item[GMT_GRID_UPPER].interval;
 			sincos (angle, &s, &c);
 			x = max_radius * scale * c;
 			y = max_radius * scale * s;
 			plot3d (0.0, 0.0, 3);
-			plot3d (x, y, 2);
+			plot3d (x, y, -2);
 		}
 		
-		n_bins = (frame_info.grid_int[0] > 0.0) ? irint (max_radius / frame_info.grid_int[0]) : -1;
+		n_bins = (frame_info.axis[0].item[GMT_GRID_UPPER].interval > 0.0) ? irint (max_radius / frame_info.axis[0].item[GMT_GRID_UPPER].interval) : -1;
 		for (i = 1; i <= n_bins; i++)
-			arc3d (0.0, 0.0, i * frame_info.grid_int[0] * scale, 0.0, total_arc, 3);
+			arc3d (0.0, 0.0, i * frame_info.axis[0].item[GMT_GRID_UPPER].interval * scale, 0.0, total_arc, 3);
 		ps_setpaint (gmtdefs.basemap_frame_rgb);
-		y = lsize + 6.0 * gmtdefs.anot_offset;
-		GMT_text3d (0.0, off + y, project_info.z_level, gmtdefs.header_font_size, gmtdefs.header_font, frame_info.header, 0.0, 2, 0);
+		y = lsize + 6.0 * gmtdefs.annot_offset[0];
+		GMT_text3D (0.0, off + y, project_info.z_level, gmtdefs.header_font_size, gmtdefs.header_font, frame_info.header, 0.0, 2, 0);
 		
-		GMT_get_format (frame_info.anot_int[0], frame_info.unit[0], format);
+		GMT_get_format (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval, frame_info.axis[0].unit, frame_info.axis[0].prefix, format);
 		
 		if (half_only) {
-			int which;
-			which = (gmtdefs.degree_format >= 100);	/* 0 is small [Default], 1 is large */
-			sprintf (lw, "90%sW", GMT_degree_symbol[which]);
-			sprintf (le, "90%sE", GMT_degree_symbol[which]);
-			sprintf (ln, "0%s", GMT_degree_symbol[which]);
-			y = -(3.0 * gmtdefs.anot_offset + GMT_font_height[gmtdefs.anot_font] * asize);
-			if (frame_info.label[0][0]) GMT_text3d (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.label[0], 0.0, 10, 0);
-			y = -(5.0 * gmtdefs.anot_offset + GMT_font_height[gmtdefs.anot_font] * lsize + GMT_font_height[gmtdefs.label_font] * lsize);
-			if (frame_info.label[1][0]) GMT_text3d (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.label[1], 0.0, 10, 0);
-			GMT_text3d (0.0, -gmtdefs.anot_offset, project_info.z_level, gmtdefs.anot_font_size, gmtdefs.anot_font, "0", 0.0, 10, 0);
-			n_anot = (frame_info.anot_int[0] > 0.0) ? irint (max_radius / frame_info.anot_int[0]) : -1;
-			for (i = 1; i <= n_anot; i++) {
-				x = i * frame_info.anot_int[0];
+			int which, no_degree;
+			which = (gmtdefs.degree_format >= 100) ? gmt_ring : gmt_degree;
+			no_degree = (gmtdefs.degree_format >= 1000);	/* No, we don't want the degree symbol at all */
+			if (no_degree) {
+				sprintf (lw, "90W");
+				sprintf (le, "90E");
+				sprintf (ln, "0");
+			}
+			else {
+				sprintf (lw, "90%cW", gmtdefs.encoding.code[which]);
+				sprintf (le, "90%cE", gmtdefs.encoding.code[which]);
+				sprintf (ln, "0%c", gmtdefs.encoding.code[which]);
+			}
+			y = -(3.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * asize);
+			if (frame_info.axis[0].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[0].label, 0.0, 10, 0);
+			y = -(5.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * lsize + GMT_font[gmtdefs.label_font].height * lsize);
+			if (frame_info.axis[1].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[1].label, 0.0, 10, 0);
+			GMT_text3D (0.0, -gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], "0", 0.0, 10, 0);
+			n_annot = (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval > 0.0) ? irint (max_radius / frame_info.axis[0].item[GMT_ANNOT_UPPER].interval) : -1;
+			for (i = 1; i <= n_annot; i++) {
+				x = i * frame_info.axis[0].item[GMT_ANNOT_UPPER].interval;
 				sprintf (text, format, x);
 				x *= scale;
-				GMT_text3d (x, -gmtdefs.anot_offset, project_info.z_level, gmtdefs.anot_font_size, gmtdefs.anot_font, text, 0.0, 10, 0);
-				GMT_text3d (-x, -gmtdefs.anot_offset, project_info.z_level, gmtdefs.anot_font_size, gmtdefs.anot_font, text, 0.0, 10, 0);
+				GMT_text3D (x, -gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
+				GMT_text3D (-x, -gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 			}
 		}
 		else {
-			GMT_text3d (0.0, -off - 2.0 * gmtdefs.anot_offset, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, "SOUTH", 0.0, 10, 0);
+			GMT_text3D (0.0, -off - 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, "SOUTH", 0.0, 10, 0);
 			plot3d (off, -off, 3);
-			plot3d ((max_radius - frame_info.grid_int[0]) * scale, -off, 2);
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -off, 2);
 			plot3d (off, -off, 3);
 			plot3d (off, gmtdefs.tick_length - off, 2);
-			plot3d ((max_radius - frame_info.grid_int[0]) * scale, -off, 3);
-			plot3d ((max_radius - frame_info.grid_int[0]) * scale, gmtdefs.tick_length - off, 2);
-			y = -(off + 5.0 * gmtdefs.anot_offset + GMT_font_height[gmtdefs.anot_font] * lsize + GMT_font_height[gmtdefs.label_font] * lsize);
-			if (frame_info.label[1][0]) GMT_text3d (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.label[1], 0.0, 10, 0);
-			if (frame_info.label[0][0]) {
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -off, 3);
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, gmtdefs.tick_length - off, -2);
+			y = -(off + 5.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * lsize + GMT_font[gmtdefs.label_font].height * lsize);
+			if (frame_info.axis[1].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[1].label, 0.0, 10, 0);
+			if (frame_info.axis[0].label[0]) {
 				strcat (format, " %s");
-				sprintf (text, format, frame_info.grid_int[0], frame_info.label[0]);
+				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval, frame_info.axis[0].label);
 			}
 			else
-				sprintf (text, format, frame_info.grid_int[0]);
-			GMT_text3d ((max_radius - 0.5 * frame_info.grid_int[0]) * scale, -(off + gmtdefs.anot_offset), project_info.z_level, gmtdefs.anot_font_size, gmtdefs.anot_font, text, 0.0, 10, 0);
+				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
+			GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 		}
-		GMT_text3d (off + 2.0 * gmtdefs.anot_offset, 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, le, 0.0, 5, 0);
-		GMT_text3d (-off - 2.0 * gmtdefs.anot_offset, 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, lw, 0.0, 7, 0);
-		GMT_text3d (0.0, off + 2.0 * gmtdefs.anot_offset, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, ln, 0.0, 2, 0);
+		GMT_text3D (off + 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, le, 0.0, 5, 0);
+		GMT_text3D (-off - 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, lw, 0.0, 7, 0);
+		GMT_text3D (0.0, off + 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, ln, 0.0, 2, 0);
 		ps_setpaint (gmtdefs.background_rgb);
 	}
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin + y_origin, 0.0);
@@ -637,7 +639,7 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 			yy[i] = y + r * s;
 		}
 		n++;	
-		GMT_2D_to_3D (xx, yy, n);
+		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
 		ps_line (xx, yy, n, status, FALSE, TRUE);
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
@@ -674,7 +676,7 @@ void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOO
 		}
 		xx[i] = x;	yy[i] = y;
 		n += 2;	
-		GMT_2D_to_3D (xx, yy, n);
+		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
 		ps_polygon (xx, yy, n, rgb, outline);
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
