@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: psscale.c,v 1.4.4.8 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: psscale.c,v 1.35 2004/09/16 18:12:13 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -20,7 +20,7 @@
  * horizontally.  psscale will interpolate colors if the lower
  * and upper rgb values for an interval are different.  The
  * resolution for this interpolation can be set on the command line.
- * If the scale is to be used with illumninated 2-D and 3-D plots
+ * If the scale is to be used with illuminated 2-D and 3-D plots
  * then options for intensities are available.
  *
  * Author:	Paul Wessel
@@ -30,27 +30,28 @@
  *		3.3.2 15-SEP-1999. PW: Fixed bad loop for -Z
  *		3.3.4 21-JAN-2000. PW: Now paint rectangles for discrete cpt files
  *		      08-FEB-2000. PW: Also understand patterns for discrete cpt files
- * Version:	3.4.3
+ *		      10-NOV-2003. PW: Enhanced -E to set back or foreground only
+ * Version:	4
  *
  */
 
 #include "gmt.h"
 
 int bit_dpi = 300;
-BOOLEAN B_set = FALSE;
+BOOLEAN B_set = FALSE, flip_anot = FALSE;
 
-void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN horizontal, BOOLEAN intens, double max_intens, BOOLEAN skip_lines, BOOLEAN extend, double e_length, BOOLEAN monochrome);
+void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN horizontal, BOOLEAN intens, double max_intens, BOOLEAN skip_lines, int extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome);
 void fix_format (char *unit, char *format);
 
 main(int argc, char **argv)
 {
-	int i, n;
+	int i, j, n, extend = 0;
 	
-	BOOLEAN horizontal = FALSE, equi = FALSE, intens = FALSE, error = FALSE;
-	BOOLEAN skip_lines = FALSE, extend = FALSE, must_shift_back = FALSE, monochrome = FALSE;
+	BOOLEAN horizontal = FALSE, equi = FALSE, intens = FALSE, error = FALSE, interval_annot = FALSE;
+	BOOLEAN skip_lines = FALSE, must_shift_back = FALSE, monochrome = FALSE;
 	
 	double xpos, ypos, length, width, max_intens = 1.0, dz;
-	double *z_width, e_length = 0.0, x_origin, y_origin;
+	double *z_width, e_length = 0.0, x_origin, y_origin, gap = 0.0;
 	
 	char flag, *cpt_file = NULL, *z_file = NULL, line[BUFSIZ];
 	char txt_a[32], txt_b[32], txt_c[32], txt_d[32], text[128];
@@ -87,6 +88,9 @@ main(int argc, char **argv)
 				
 				/* Supplemental parameters */
 			
+				case 'A':
+					flip_anot = TRUE;
+					break;
 				case 'C':
 					cpt_file = &argv[i][2];
 					break;
@@ -104,8 +108,18 @@ main(int argc, char **argv)
 					width  = GMT_convert_units (txt_d, GMT_INCH);
 					break;
 				case 'E':
-					if (argv[i][2]) e_length = GMT_convert_units (&argv[i][2], GMT_INCH);
-					extend = TRUE;
+					j = 2;
+					if (argv[i][2] == 'b') {
+						extend = 1;
+						j = 3;
+					}
+					else if (argv[i][2] == 'f') {
+						extend = 2;
+						j = 3;
+					}
+					else
+						extend = 3;
+					if (argv[i][j]) e_length = GMT_convert_units (&argv[i][j], GMT_INCH);
 					break;
 				case 'I':
 					intens = TRUE;
@@ -119,6 +133,9 @@ main(int argc, char **argv)
 					break;
 				case 'L':
 					equi = TRUE;
+					j = 2;
+					if (argv[i][2] == 'i') interval_annot = TRUE, j = 3;
+					if (argv[i][j]) gap = GMT_convert_units (&argv[i][j], GMT_INCH);
 					break;
 				case 'S':
 					skip_lines = TRUE;
@@ -139,25 +156,30 @@ main(int argc, char **argv)
 	
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "%s %s - To create grayscale or colorscale for maps\n\n", GMT_program, GMT_VERSION);
-		fprintf(stderr, "usage: psscale -D<xpos/ypos/length/width[h]> [-C<cpt_file>] [-E[<length>]] [-B<tickinfo>] [-I[<intens>]\n");
-		fprintf(stderr, "\t[-K] [-L] [-M] [-N<dpi>] [-O] [-P] [-S] [-U] [-V] [-X<x_shift>] [-Y<y_shift>] [-Z<zfile>] [-c<ncopies>]\n\n");
+		fprintf(stderr, "usage: psscale -D<xpos/ypos/length/width[h]> [-A] [-C<cpt_file>] [-E[b|f][<length>]] [-B<tickinfo>] [-I[<intens>]\n");
+		fprintf(stderr, "\t[-K] [-L[i][<gap>]] [-M] [-N<dpi>] [-O] [-P] [-S] [-U] [-V] [-X<x_shift>] [-Y<y_shift>] [-Z<zfile>] [-c<ncopies>]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
 		fprintf (stderr, "\t-D set mid-point position and length/width for scale.\n");
 		fprintf (stderr, "\t   Give negative length to reverse the scalebar\n");
 		fprintf (stderr, "\t   Append h for horizontal scale\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-B Set scale anotation interval and label. Use y-label to set unit label\n");
-		fprintf (stderr, "\t   If no anotation interval is set it is taken from the cpt file\n");
+		fprintf (stderr, "\t-A Place the desired annotations on the other side of the colorscale instead\n");
+		fprintf (stderr, "\t-B Set scale annotation interval and label. Use y-label to set unit label\n");
+		fprintf (stderr, "\t   If no annotation interval is set it is taken from the cpt file\n");
 		fprintf (stderr, "\t-C Color palette file. If not set, stdin is read.\n");
-		fprintf (stderr, "\t   By default all color changes are anotated (but see -B).  To use a subset,\n");
+		fprintf (stderr, "\t   By default all color changes are annotated (but see -B).  To use a subset,\n");
 		fprintf (stderr, "\t   add an extra column to the cpt-file with a L, U, or B\n");
-		fprintf (stderr, "\t   to anotate Lower, Upper, or Both color segment boundaries\n");
-		fprintf (stderr, "\t-E add sidebar triangles for back- and forground colors.\n");
+		fprintf (stderr, "\t   to annotate Lower, Upper, or Both color segment boundaries\n");
+		fprintf (stderr, "\t-E add sidebar triangles for back- and foreground colors.\n");
+		fprintf (stderr, "\t   Specify b(ackground) or f(oreground) to get one only [Default is both]\n");
 		fprintf (stderr, "\t   Optionally, append triangle height [Default scales with barwidth]\n");
 		fprintf (stderr, "\t-I add illumination for +- <max_intens> [1.0]\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L For equal-sized color rectangles. -B interval, if set, is ignored\n");
+		fprintf (stderr, "\t   Append i to annotate the interval range instead of lower/upper.\n");
+		fprintf (stderr, "\t   If <gap> is appended, we separate each rectangle by <gap> units and center each\n");
+		fprintf (stderr, "\t   lower (z0) annotation on the rectangle.  Ignored if not a discrete cpt table.\n");
 		fprintf (stderr, "\t-M force monochrome colorbar using YIQ transformation\n");
 		fprintf (stderr, "\t-N effective dots-per-inch for color scale [300]\n");
 		GMT_explain_option ('O');
@@ -174,7 +196,7 @@ main(int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
@@ -196,8 +218,8 @@ main(int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	if (equi && B_set) B_set = FALSE;	/* -L overrides intervals set with -B */
-
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 	if (gmtdefs.verbose) {
 		if (cpt_file)
@@ -207,7 +229,7 @@ main(int argc, char **argv)
 	}
 
 	GMT_read_cpt (cpt_file);
-	if (gmtdefs.verbose) fprintf (stderr, "  CPT range from %lg to %lg\n",
+	if (gmtdefs.verbose) fprintf (stderr, "  CPT range from %g to %g\n",
 		GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high);
 
 	if (extend && e_length == 0.0) e_length = width * 0.5;
@@ -251,7 +273,7 @@ main(int argc, char **argv)
 	/* Because psscale uses -D to position things we need to make some
 	 * changes so that BoundingBox and others are set ~correctly */
 
-	sprintf (text, "X%lgi/%lgi", length, width);
+	sprintf (text, "X%gi/%gi", length, width);
 	GMT_map_getproject (text);	/* Fake linear projection */
 	GMT_map_setup (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width);
 
@@ -275,14 +297,14 @@ main(int argc, char **argv)
 		d_swap (z_project.xmin, z_project.ymin);
 		d_swap (z_project.xmax, z_project.ymax);
 	}
-	if (frame_info.anot_int[0] == 0.0) frame_info.plot = TRUE;
+	if (frame_info.axis[0].item[0].interval == 0.0) frame_info.plot = TRUE;
 	gmtdefs.page_orientation |= 8;	/* Ensure absolute coordinates */
 
 /*-----------------end of kludge-------------------------------------------*/
 
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 		
 	GMT_echo_command (argc, argv);
 	
@@ -296,7 +318,7 @@ main(int argc, char **argv)
 		ps_transrotate (gmtdefs.x_origin, gmtdefs.y_origin, 0.0);
 	}
 	
-	GMT_draw_colorbar (length, width, z_width, horizontal, intens, max_intens, skip_lines, extend, e_length, monochrome);
+	GMT_draw_colorbar (length, width, z_width, horizontal, intens, max_intens, skip_lines, extend, e_length, gap, interval_annot, monochrome);
 
 	if (must_shift_back) ps_transrotate (-x_origin, -y_origin, 0.0);
 
@@ -307,50 +329,50 @@ main(int argc, char **argv)
 	GMT_end (argc, argv);
 }
 
-void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN horizontal, BOOLEAN intens, double max_intens, BOOLEAN skip_lines, BOOLEAN extend, double e_length, BOOLEAN monochrome)
+void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN horizontal, BOOLEAN intens, double max_intens, BOOLEAN skip_lines, int extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome)
 {
-	int i, ii, j, ndec = -1, dec, rgb[3], rrggbb[3];
-	int nx, ny, nm, barmem, k;
-	BOOLEAN reverse, all = TRUE, use_image;
-	char format[64], text[64], test[64];
+	int i, ii, j, ndec = -1, dec, depth, rgb[3], rrggbb[3];
+	int nx, ny, nm, barmem, k, justify;
+	BOOLEAN reverse, all = TRUE, use_image, center = FALSE;
+	char format[64], text[64], test[64], unit[32];
 	unsigned char *bar;
-	double off, anot_off, label_off, len, len2, size, x0, x1, dx, xx;
+	double off, annot_off, label_off, len, len2, size, x0, x1, dx, xx, dir, y_base;
 	double z, xleft, xright, inc_i, inc_j;
 	double get_z (double x, double *width, int n);
 	double xp[4], yp[4];
 	struct GMT_FILL *f;
 	
-	ps_setfont (gmtdefs.anot_font);
+	ps_setfont (gmtdefs.annot_font[0]);
 
-	gmtdefs.anot_offset = fabs (gmtdefs.anot_offset);	/* No 'inside' anotations allowed in colorbar */
+	gmtdefs.annot_offset[0] = fabs (gmtdefs.annot_offset[0]);	/* No 'inside' annotations allowed in colorbar */
 
 	/* Find max decimals needed */
 	
 
 	if (B_set) {
-		ndec = GMT_get_format (frame_info.anot_int[0], frame_info.unit[0], format);
+		ndec = GMT_get_format (frame_info.axis[0].item[0].interval, frame_info.axis[0].unit, frame_info.axis[0].prefix, format);
 	}
 	else {
 		for (i = 0; i < GMT_n_colors; i++) {
-			if (GMT_lut[i].anot & 1) {
-				if ((dec = GMT_get_format (GMT_lut[i].z_low, CNULL, text)) > ndec) {
+			if (GMT_lut[i].annot & 1) {
+				if ((dec = GMT_get_format (GMT_lut[i].z_low, CNULL, CNULL, text)) > ndec) {
 					strcpy (format, text);
 					ndec = dec;
 				}
 			}
-			if (GMT_lut[i].anot & 2) {
-				if ((dec = GMT_get_format (GMT_lut[i].z_high, CNULL, text)) > ndec) {
+			if (GMT_lut[i].annot & 2) {
+				if ((dec = GMT_get_format (GMT_lut[i].z_high, CNULL, CNULL, text)) > ndec) {
 					strcpy (format, text);
 					ndec = dec;
 				}
 			}
-			if (GMT_lut[i].anot) all = FALSE;
+			if (GMT_lut[i].annot) all = FALSE;
 		}
 	}
 	
 	if (ndec == 0) {	/* Not -B and no decimals are needed */
 		strcpy (format, gmtdefs.d_format);
-		fix_format (frame_info.unit[0], format);	/* Add units if needed */
+		fix_format (frame_info.axis[0].unit, format);	/* Add units if needed */
 	}
 
 	len = gmtdefs.tick_length;	/* +ve means draw on the outside of bar */
@@ -360,9 +382,15 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 	reverse = (length < 0.0);
 	length = fabs (length);
 	xright = length;
-
 	use_image = (GMT_continuous || intens);
 	
+	if ((gap > 0.0 || interval_annot) && !use_image) {	/* Want to center annotations for discrete colortable, using lower z0 value */
+		center = skip_lines = TRUE;
+		gap *= 0.5;
+		if (interval_annot) sprintf (text, "%s - %s", format, format);
+		strcpy (format, text);
+	}
+
 	if (use_image) {	/* Make bitimage for colorbar using bit_dpi */
 		nx = irint (length * bit_dpi);
 		ny = irint (width * bit_dpi);
@@ -398,14 +426,27 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 	
 	GMT_setpen (&gmtdefs.frame_pen);
 
+	unit[0] = 0;
+	if (!strcmp (frame_info.axis[0].label, frame_info.axis[1].label)) frame_info.axis[1].label[0] = 0;	/* Defeat the auto-repeat of axis info */
+	strcpy (unit, frame_info.axis[1].label);	/* Save possible unit here since we need to copy x-axis info to the y-axis for vertical axis */
+			
+	if (flip_anot) {
+		justify = (horizontal) ? 2 : 5;
+		y_base = width;
+		dir = -1.0;
+	}
+	else {
+		justify = (horizontal) ? 10 : 7;
+		y_base = 0.0;
+		dir = 1.0;
+	}
+
 	/* Current point (0,0) is now at lower left location of bar */
 
+	depth = (monochrome || GMT_gray) ? 8 : 24;
 	if (horizontal) {
 		if (use_image) {	/* Must plot as image */
-			if (monochrome || GMT_gray)
-				ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
-			else 
-				GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
+			GMT_color_image (0.0, 0.0, length, width, bar, nx, ny, depth);
 		}
 		else {			/* Plot as rectangles */
 			x0 = x1 = 0.0;
@@ -413,50 +454,66 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 			for (i = 0; i < GMT_n_colors; i++) {
 				x1 += z_width[i];
 				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
-					xp[0] = xp[3] = x0;	xp[1] = xp[2] = x1;
-					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
+					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, center, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				}
+				else {
+					memcpy ((void *)rgb, (void *)GMT_lut[i].rgb_low, (size_t)(3 * sizeof (int)));
+					if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+					ps_rect (x0+gap, 0.0, x1-gap, width, rgb, center);
 				}
-				else
-					ps_rect (x0, 0.0, x1, width, GMT_lut[i].rgb_low, FALSE);
 				x0 = x1;
 			}
 		}
 		
-		if (extend) {	/* Add color triangles for fore- and background */
-			xp[0] = xp[1] = xleft;	xp[2] = xp[0] - e_length;
+		if (extend & 1) {	/* Add color triangles for background color  */
+			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
-			if ((f = GMT_bfn.fill[1]))
+			if ((f = GMT_bfn[GMT_BGD].fill))
 				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
-			else
-				ps_polygon (xp, yp, 3, GMT_bfn.background_rgb, FALSE);
+			else {
+				memcpy ((void *)rgb, (void *)GMT_bfn[GMT_BGD].rgb, (size_t)(3 * sizeof (int)));
+				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+				ps_polygon (xp, yp, 3, rgb, FALSE);
+			}
 			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
-			xp[0] = xp[1] = xright;	xp[2] = xp[0] + e_length;
+		}
+		if (extend & 2) {	/* Add color triangles for foreground color  */
+			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
-			if ((f = GMT_bfn.fill[0]))
+			if ((f = GMT_bfn[GMT_FGD].fill))
 				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
-			else
-				ps_polygon (xp, yp, 3, GMT_bfn.foreground_rgb, FALSE);
+			else {
+				memcpy ((void *)rgb, (void *)GMT_bfn[GMT_FGD].rgb, (size_t)(3 * sizeof (int)));
+				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+				ps_polygon (xp, yp, 3, rgb, FALSE);
+			}
 			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
 		}
 		
-		ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
-		ps_plot (xleft, width, 3);	ps_plotr (length, 0.0, 2);
-		ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
-		ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, 2);
-
+		if (!center) {
+			ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
+			ps_plot (xleft, width, 3);	ps_plotr (length, 0.0, 2);
+			ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
+			ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, -2);
+		}
+		
 		if (B_set) {	/* Used -B */
-			GMT_x_axis (xleft, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], frame_info.label[0], frame_info.unit[0], LINEAR, 0, TRUE, frame_info.side[0]-1);
-			dx = fabs (frame_info.grid_int[0]); 
+			if (flip_anot)
+				GMT_xy_axis (xleft, width, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[0], FALSE, frame_info.side[0]-1);
+			else
+				GMT_xy_axis (xleft, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[0], TRUE, frame_info.side[0]-1);
+			dx =  GMT_get_map_interval (0, GMT_GRID_UPPER); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 		}
 		else {
-			anot_off = (((len > 0.0) ? len : 0.0) + gmtdefs.anot_offset);
-			label_off = -(anot_off + 1.5 * gmtdefs.anot_offset + (gmtdefs.anot_font_size * GMT_u2u[GMT_PT][GMT_INCH]) * GMT_font_height[gmtdefs.anot_font]);
+			annot_off = (((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0]);
+			label_off = -(annot_off + 1.5 * gmtdefs.annot_offset[0] + (gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH]) * GMT_font[gmtdefs.annot_font[0]].height);
 		
 			/* First draw gridlines, unless skip_lines is TRUE */
 			
 			if (!skip_lines) {
-				GMT_setpen (&gmtdefs.grid_pen);
+				GMT_setpen (&gmtdefs.grid_pen[0]);
 				x1 = xleft;
 				for (i = 0; i < GMT_n_colors; i++) {
 					xx = (reverse) ? xright - x1 : x1;
@@ -466,52 +523,54 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 				}
 				xx = (reverse) ? xleft : xright;
 				ps_plot (xx, 0.0, 3);
-				ps_plotr (0.0, width, 2);
+				ps_plotr (0.0, width, -2);
 			}
 			
-			/* Then anotate and draw tickmarks */
+			/* Then annotate and draw tickmarks */
 			
 			GMT_setpen (&gmtdefs.tick_pen);
 			x1 = xleft;
+			if (center) x1 += 0.5 * z_width[0];
 			for (i = 0; i < GMT_n_colors; i++) {
 				xx = (reverse) ? xright - x1 : x1;
-				ps_plot (xx, 0.0, 3);
-				if (all || (GMT_lut[i].anot & 1)) {	/* Anotate this */
-					ps_plotr (0.0, -len, 2);
-					sprintf (text, format, GMT_lut[i].z_low);
-					ps_text (xx, -anot_off, gmtdefs.anot_font_size, text, 0.0, 10, 0);
+				if (!center) ps_plot (xx, y_base, 3);
+				if (all || (GMT_lut[i].annot & 1)) {	/* Annotate this */
+					if (!center) ps_plotr (0.0, -(len * dir), 2);
+					if (center && interval_annot)
+						sprintf (text, format, GMT_lut[i].z_low, GMT_lut[i].z_high);
+					else
+						sprintf (text, format, GMT_lut[i].z_low);
+					ps_text (xx, y_base - (dir * annot_off), gmtdefs.annot_font_size[0], text, 0.0, justify, 0);
 				}
 				else
-					ps_plotr (0.0, -len2, 2);
+					if (!center) ps_plotr (0.0, -(len2 * dir), -2);
 				x1 += z_width[i];
 			}
 			xx = (reverse) ? xleft : xright;
-			ps_plot (xx, 0.0, 3);
-			if (all || (GMT_lut[GMT_n_colors-1].anot & 2)) {
-				ps_plotr (0.0, -len, 2);
-				sprintf (text, format, GMT_lut[GMT_n_colors-1].z_high);
-				ps_text (xx, -anot_off, gmtdefs.anot_font_size, text, 0.0, 10, 0);
+			if (!center) {
+				ps_plot (xx, 0.0, 3);
+				if (all || (GMT_lut[GMT_n_colors-1].annot & 2)) {
+					ps_plotr (0.0, -(len * dir), -2);
+					sprintf (text, format, GMT_lut[GMT_n_colors-1].z_high);
+					ps_text (xx, y_base - (dir * annot_off), gmtdefs.annot_font_size[0], text, 0.0, justify, 0);
+				}
+				else
+					ps_plotr (0.0, -(len2 * dir), -2);
 			}
-			else
-				ps_plotr (0.0, -len2, 2);
-
-			if (frame_info.label[0][0]) {
+			if (frame_info.axis[0].label[0]) {
 				ps_setfont (gmtdefs.label_font);
-				ps_text (xleft+0.5*length, label_off, gmtdefs.label_font_size, frame_info.label[0], 0.0, 10, 0);
+				ps_text (xleft+0.5*length, y_base + (dir * label_off), gmtdefs.label_font_size, frame_info.axis[0].label, 0.0, justify, 0);
 			}
 		}
-		if (frame_info.label[1][0]) {	/* Add unit label */
-			ps_setfont (gmtdefs.anot_font);
-			ps_text (xright+e_length+gmtdefs.anot_offset, 0.5*width, gmtdefs.anot_font_size, frame_info.label[1], 0.0, 5, 0);
+		if (unit[0]) {	/* Add unit label */
+			ps_setfont (gmtdefs.annot_font[0]);
+			ps_text (xright+e_length+gmtdefs.annot_offset[0], 0.5*width, gmtdefs.annot_font_size[0], unit, 0.0, 5, 0);
 		}
 	}
 	else {	/* Vertical scale */
 		ps_transrotate (width, 0.0, 90.0);
 		if (use_image) {	/* Must plot with image */
-			if (monochrome || GMT_gray)
-				ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
-			else 
-				GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
+			GMT_color_image (0.0, 0.0, length, width, bar, nx, ny, depth);
 		}
 		else {			/* Plot as rectangles */
 			x0 = x1 = 0.0;
@@ -519,50 +578,77 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 			for (i = 0; i < GMT_n_colors; i++) {
 				x1 += z_width[i];
 				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
-					xp[0] = xp[3] = x0;	xp[1] = xp[2] = x1;
-					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
+					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, center, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				}
+				else {
+					memcpy ((void *)rgb, (void *)GMT_lut[i].rgb_low, (size_t)(3 * sizeof (int)));
+					if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+					ps_rect (x0 + gap, 0.0, x1 - gap, width, rgb, center);
 				}
-				else
-					ps_rect (x0, 0.0, x1, width, GMT_lut[i].rgb_low, FALSE);
 				x0 = x1;
 			}
 		}
-		sprintf (text, "%d", (int)fabs (floor (GMT_lut[0].z_low)));
-		sprintf (test, "%d", (int)fabs (ceil (GMT_lut[GMT_n_colors-1].z_high)));
-		off = ((MAX ((int)strlen (text), (int)strlen (test)) + ndec) * 0.49 + ((ndec > 0) ? 0.3 : 0.0) + ((GMT_lut[0].z_low < 0.0) ? 0.3 : 0.0))
-			* gmtdefs.anot_font_size * GMT_u2u[GMT_PT][GMT_INCH];
-		anot_off = -(((len > 0.0) ? len : 0.0) + gmtdefs.anot_offset + off);
-		label_off = anot_off - 0.7 * gmtdefs.anot_font_size * GMT_u2u[GMT_PT][GMT_INCH] - 1.5 * gmtdefs.anot_offset;
-		if (extend) {	/* Add color triangles for fore- and background */
-			xp[0] = xp[1] = xleft;	xp[2] = xp[0] - e_length;
+		if (center && interval_annot) {
+			sprintf (text, "%d - %d", (int)fabs (floor (GMT_lut[0].z_low)), (int)fabs (ceil (GMT_lut[0].z_high)));
+			sprintf (test, "%d - %d", (int)fabs (floor (GMT_lut[GMT_n_colors-1].z_low)), (int)fabs (ceil (GMT_lut[GMT_n_colors-1].z_high)));
+		}
+		else {
+			sprintf (text, "%d", (int)fabs (floor (GMT_lut[0].z_low)));
+			sprintf (test, "%d", (int)fabs (ceil (GMT_lut[GMT_n_colors-1].z_high)));
+		}
+		off = ((MAX ((int)strlen (text), (int)strlen (test)) + (interval_annot + 1) * ndec) * 0.49 + ((ndec > 0) ? 0.3 : 0.0) + ((GMT_lut[0].z_low < 0.0) ? 0.3 : 0.0))
+			* gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
+		annot_off = -(((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0] + off);
+		label_off = annot_off - 0.7 * gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH] - 1.5 * gmtdefs.annot_offset[0];
+		if (extend & 1) {	/* Add color triangles for background */
+			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
-			if ((f = GMT_bfn.fill[1]))
+			if ((f = GMT_bfn[GMT_BGD].fill))
 				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
-			else
-				ps_polygon (xp, yp, 3, GMT_bfn.background_rgb, FALSE);
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
-			xp[0] = xp[1] = xright;	xp[2] = xp[0] + e_length;
-			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
-			if ((f = GMT_bfn.fill[0]))
-				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
-			else
-				ps_polygon (xp, yp, 3, GMT_bfn.foreground_rgb, FALSE);
+			else {
+				memcpy ((void *)rgb, (void *)GMT_bfn[GMT_BGD].rgb, (size_t)(3 * sizeof (int)));
+				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+				ps_polygon (xp, yp, 3, rgb, FALSE);
+			}
 			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
 		}
-		ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
-		ps_plot (xleft, width, 3);	ps_plotr (length, 0.0, 2);
-		ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
-		ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, 2);
-		if (B_set) {	/* Used -B */
-			dx = fabs (frame_info.grid_int[0]); 
+		if (extend & 2) {	/* Add color triangles for foreground */
+			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
+			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
+			if ((f = GMT_bfn[GMT_FGD].fill))
+				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+			else {
+				memcpy ((void *)rgb, (void *)GMT_bfn[GMT_FGD].rgb, (size_t)(3 * sizeof (int)));
+				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
+				ps_polygon (xp, yp, 3, rgb, FALSE);
+			}
+			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+		}
+		if (!center) {
+			ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
+			ps_plot (xleft, width, 3);	ps_plotr (length, 0.0, 2);
+			ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
+			ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, -2);
+		}
+		if (B_set) {	/* Used -B. Must kludge by copying x-axis and scaling to y since we must use GMT_xy_axis to draw a rotated x-axis... */
+			dx = GMT_get_map_interval (0, GMT_GRID_UPPER); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 			ps_transrotate (0.0, 0.0, -90.0);
-			GMT_y_axis (0.0, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], CNULL, CNULL, LINEAR, 0, FALSE, frame_info.side[0]-1);
+			memcpy ((void *)&frame_info.axis[1], (void *)&frame_info.axis[0], sizeof (struct PLOT_AXIS));
+			frame_info.axis[1].label[0] = 0;
+			d_swap (project_info.x_scale, project_info.y_scale);
+			d_swap (project_info.x0, project_info.y0);
+			for (i = 0; i < 5; i++) frame_info.axis[1].item[i].parent = 1;
+			if (flip_anot)
+				GMT_xy_axis (-width, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[1], TRUE, frame_info.side[0]-1);
+			else
+				GMT_xy_axis (0.0, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[1], FALSE, frame_info.side[0]-1);
 			ps_rotatetrans (0.0, 0.0, 90.0);
 		}
 		else {
 			if (!skip_lines) {	/* First draw gridlines */
-				GMT_setpen (&gmtdefs.grid_pen);
+				GMT_setpen (&gmtdefs.grid_pen[0]);
 				x1 = xleft;
 				for (i = 0; i < GMT_n_colors; i++) {
 					xx = (reverse) ? xright - x1 : x1;
@@ -572,50 +658,61 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 				}
 				xx = (reverse) ? xleft : xright;
 				ps_plot (xx, 0.0, 3);
-				ps_plotr (0.0, width, 2);
+				ps_plotr (0.0, width, -2);
 			}
 			
-			/* Then anotate and draw tickmarks */
+			/* Then annotate and draw tickmarks */
 			
 			GMT_setpen (&gmtdefs.tick_pen);
 			x1 = xleft;
+			if (center) x1 += 0.5 * z_width[0];
 			for (i = 0; i < GMT_n_colors; i++) {
 				xx = (reverse) ? xright - x1 : x1;
-				ps_plot (xx, 0.0, 3);
-				if (all || (GMT_lut[i].anot & 1)) {
-					ps_plotr (0.0, -len, 2);
-					sprintf (text, format, GMT_lut[i].z_low);
-					ps_text (xx, anot_off, gmtdefs.anot_font_size, text, -90.0, 7, 0);
+				if (!center) ps_plot (xx, y_base, 3);
+				if (all || (GMT_lut[i].annot & 1)) {
+					if (!center) ps_plotr (0.0, -(len * dir), -2);
+					if (center && interval_annot)
+						sprintf (text, format, GMT_lut[i].z_low, GMT_lut[i].z_high);
+					else
+						sprintf (text, format, GMT_lut[i].z_low);
+					ps_text (xx, y_base + (dir * annot_off), gmtdefs.annot_font_size[0], text, -90.0, justify, 0);
 				}
 				else
-					ps_plotr (0.0, -len2, 2);
+					if (!center) ps_plotr (0.0, -(dir * len2), -2);
 				x1 += z_width[i];
 			}
-			xx = (reverse) ? xleft : xright;
-			ps_plot (xx, 0.0, 3);
-			if (all || (GMT_lut[GMT_n_colors-1].anot & 2)) {
-				ps_plotr (0.0, -len, 2);
-				sprintf (text, format, GMT_lut[GMT_n_colors-1].z_high);
-				ps_text (xx, anot_off, gmtdefs.anot_font_size, text, -90.0, 7, 0);
+			if (!center) {
+				xx = (reverse) ? xleft : xright;
+				ps_plot (xx, y_base, 3);
+				if (all || (GMT_lut[GMT_n_colors-1].annot & 2)) {
+					ps_plotr (0.0, -(len * dir), -2);
+					sprintf (text, format, GMT_lut[GMT_n_colors-1].z_high);
+					ps_text (xx, y_base + (dir * annot_off), gmtdefs.annot_font_size[0], text, -90.0, justify, 0);
+				}
+				else
+					ps_plotr (0.0, -(len2 * dir), -2);
 			}
-			else
-				ps_plotr (0.0, -len2, 2);
 		}
 
-		if (frame_info.label[0][0]) {
+		if (frame_info.axis[0].label[0]) {
 			ps_setfont (gmtdefs.label_font);
-			size = 0.9 * gmtdefs.label_font_size * GMT_u2u[GMT_PT][GMT_INCH];
-			x0 = 0.5 * (length + ((int)strlen (frame_info.label[0]) -1) * size);
-			text[1] = 0;
-			for (i = 0; i < (int)strlen (frame_info.label[0]); i++) {
-				x1 = x0 - i * size;
-				text[0] = frame_info.label[0][i];
-				ps_text (x1, label_off, gmtdefs.label_font_size, text, -90.0, 6, 0);
+			if (strchr (frame_info.axis[0].label, '@')) {	/* Must set text horizontally */
+				ps_text (xleft+0.5*length, y_base + (dir * label_off), gmtdefs.label_font_size, frame_info.axis[0].label, 0.0, justify, 0);
+			}
+			else {
+				size = 0.9 * gmtdefs.label_font_size * GMT_u2u[GMT_PT][GMT_INCH];
+				x0 = 0.5 * (length + ((int)strlen (frame_info.axis[0].label) -1) * size);
+				text[1] = 0;
+				for (i = 0; i < (int)strlen (frame_info.axis[0].label); i++) {
+					x1 = x0 - i * size;
+					text[0] = frame_info.axis[0].label[i];
+					ps_text (x1, y_base + (dir * label_off), gmtdefs.label_font_size, text, -90.0, 6, 0);
+				}
 			}
 		}
-		if (frame_info.label[1][0]) {	/* Add unit label */
-			ps_setfont (gmtdefs.anot_font);
-			ps_text (xright + gmtdefs.anot_offset + e_length, 0.5 * width, gmtdefs.anot_font_size, frame_info.label[1], -90.0, 2, 0);
+		if (unit[0]) {	/* Add unit label */
+			ps_setfont (gmtdefs.annot_font[0]);
+			ps_text (xright + gmtdefs.annot_offset[0] + e_length, 0.5 * width, gmtdefs.annot_font_size[0], unit, -90.0, 2, 0);
 		}
 		ps_rotatetrans (-width, 0.0, -90.0);
 
@@ -639,7 +736,7 @@ void fix_format (char *unit, char *format)
 	int i, j;
 	char text[80], new_format[BUFSIZ];
 	
-	/* Check if anotation units should be added */
+	/* Check if annotation units should be added */
 	
 	if (unit && unit[0]) {	/* Must append the unit string */
 		if (!strchr (unit, '%'))	/* No percent signs */
@@ -651,9 +748,9 @@ void fix_format (char *unit, char *format)
 			}
 			text[j] = 0;
 		}
-		if (text[0] == '-')	/* No space between anotation and unit */
+		if (text[0] == '-')	/* No space between annotation and unit */
 			sprintf (new_format, "%s%s", format, &text[1]);
-		else		/* 1 space between anotation and unit */
+		else		/* 1 space between annotation and unit */
 			sprintf (new_format, "%s %s", format, text);
 		strcpy (format, new_format);
 	}
