@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pstext.c,v 1.2.4.10 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pstext.c,v 1.35 2004/06/09 06:23:28 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -27,7 +27,7 @@
  * Version:	3.1 based on old 3.0
  * Version:	3.3: PW: Introducing paragraph mode -M
  * Version:	3.3.4: PW: Input x,y can be in dd:mm[:ss] format
- * Version:	3.4.3
+ * Version:	4
  *
  */
 
@@ -39,10 +39,11 @@ struct GMT_TEXT {
 	double paragraph_width;
 	int text_justify;
 	int paragraph_font;
-	int font_size;
+	double font_size;
 	double paragraph_angle;
 	int block_justify;
 	int boxflag;
+	int space_flag;
 	double x_space, y_space;	/* Extra spacing between box and text */
 	struct GMT_PEN boxpen;
 	struct GMT_PEN vecpen;
@@ -51,21 +52,20 @@ struct GMT_TEXT {
 	struct GMT_FILL boxfill;
 };
 
-int GMT_just_decode(char *key);
 void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT *info);
 
 main(int argc, char **argv)
 {
 	int i, j, k, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
-	int ix, iy, use_font[N_FONTS], n_processed = 0, n_read = 0, n_paragraphs = 0;
+	int ix, iy, *use_font, n_processed = 0, n_read = 0, n_paragraphs = 0;
 	size_t n_alloc;
 	
 	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x, plot_y;
-	double new_z_level = 0.0, z_level = 0.0;
+	double new_z_level = 0.0, z_level = 0.0, tmp, dx, dy;
 	
 	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, set_z = FALSE, master_record = FALSE, offset_outward = FALSE;
 	BOOLEAN solid = TRUE, do_paragraphs = FALSE, list = FALSE, done = FALSE, clip = TRUE, old_GMT_world_map, add;
-	BOOLEAN skip_text_records = FALSE;
+	BOOLEAN convert_az = FALSE, skip_text_records = FALSE;
 	
 	char text[BUFSIZ], line[BUFSIZ], this_font[80], just_key[5], txt_a[32], txt_b[32], txt_x[32], txt_y[32];
 	char *p, **word, pjust;
@@ -84,8 +84,8 @@ main(int argc, char **argv)
 
 	argc = GMT_begin (argc, argv);
 
-	T.x_space = T.y_space = 0.05;
-	if (gmtdefs.measure_unit == GMT_CM) T.x_space = T.y_space = 0.15 / 2.54;
+	T.x_space = T.y_space = 15.0;	/* 15% of font size is default clearance */
+	T.space_flag = 1;		/* Meaning we gave percentage */
 
 	/* Check and interpret the command line arguments */
 	
@@ -115,11 +115,15 @@ main(int argc, char **argv)
 					break;
 				
 				/* Supplemental parameters */
-			
+
+				case 'A':	/* Getting azimuths rather than directions, must convert via map projection */
+					convert_az = TRUE;
+					break;
 				case 'C':
-					sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
+					k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
 					T.x_space = GMT_convert_units (txt_a, GMT_INCH);
-					T.y_space = GMT_convert_units (txt_b, GMT_INCH);
+					T.y_space = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : T.x_space;
+					T.space_flag = (strchr (argv[i], '%')) ? 1 : 0;
 					break;
 				case 'D':
 					k = 2;
@@ -177,7 +181,7 @@ main(int argc, char **argv)
 						}
 						argv[i][j] = 0;
 					}
-					if (argv[i][2] && isdigit ((int)argv[i][2])) {	/* Also gave fill */
+					if (argv[i][2]) {	/* Also gave fill */
 						if (GMT_getfill (&argv[i][2], &T.boxfill)) {
 							GMT_fill_syntax ('W');
 							error++;
@@ -211,7 +215,7 @@ main(int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pstext <txtfile> -J<params> -R<west>/<east>/<south>/<north>\n");
-		fprintf (stderr, "\t[-B<tickinfo>] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n");
+		fprintf (stderr, "\t[-A] [-B<tickinfo>] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n");
 		fprintf (stderr, "\t[-H[<nrec>]] [-K] [-L] [-M[<flag>]] [-N] [-O] [-P] [-S<pen>] [-U] [-V] [-W[<fill>][o|O|c[<pen>]]]\n");
 		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z<zlevel>] [-c<ncopies>] [-:]\n\n");
 		fprintf (stderr, "\tReads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
@@ -236,7 +240,7 @@ main(int argc, char **argv)
 			fprintf (stderr, "\n\tFont #	Font Name\n");
 			fprintf (stderr, "\t------------------------------------\n");
 			for (i = 0; i < N_FONTS; i++)
-				fprintf (stderr, "\t%3d\t%s\n", i, GMT_font_name[i]);
+				fprintf (stderr, "\t%3d\t%s\n", i, GMT_font[i].name);
 		}
 			
 		if (GMT_quick) exit (EXIT_FAILURE);
@@ -244,9 +248,10 @@ main(int argc, char **argv)
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-A Angles given as azimuths; convert to directions using current projection\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "\t-C sets the clearance between characters and surrounding box [0.05i/0.05i]\n");
-		fprintf (stderr, "\t   Only used if -W has been set\n");
+		fprintf (stderr, "\t-C sets the clearance between characters and surrounding box.  Only used\n");
+		fprintf (stderr, "\t   if -W has been set.  Append units {c,i,m,p} or %% of fontsize [15%%]\n");
 		fprintf (stderr, "\t-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
 		fprintf (stderr, "\t   Use -Dj to move text origin away from point (direction determined by text's justification)\n");
 		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point (-M only)\n");
@@ -257,7 +262,7 @@ main(int argc, char **argv)
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
 		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
-		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
+		fprintf (stderr, "\t   parjust is one of (GMTMANSECTION)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		fprintf (stderr, "\t-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -278,7 +283,7 @@ main(int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	if (T.x_space < 0.0 || T.y_space < 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  clearances cannot be negative!\n", GMT_program);
@@ -315,8 +320,9 @@ main(int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
+	use_font = (int *) GMT_memory (VNULL, (size_t)N_FONTS, sizeof (int), GMT_program);
 	add = !(T.x_offset == 0.0 && T.y_offset == 0.0);
 	if (add && offset_outward) T.boxflag |= 64;
 	if (n_files > 0)
@@ -329,7 +335,7 @@ main(int argc, char **argv)
 	
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 		
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
@@ -344,18 +350,19 @@ main(int argc, char **argv)
 	if (clip) GMT_map_clip_on (GMT_no_rgb, 3);
 	
 	last_font = 0;
-	ix = (gmtdefs.xy_toggle);	iy = 1 - ix;
+	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
 	
 	/* if (draw_box && project_info.three_D) draw_box = FALSE; */	/* Not implemented yet */
 	
 	/* Mark used fonts */
 	
-	if (gmtdefs.want_euro_font) {
-		memset ((void *)use_font, 0, (size_t)(N_FONTS * sizeof (int)));
+	if (strcmp (gmtdefs.encoding.name, "Standard") != 0) {
+		memset (use_font, 0, N_FONTS * sizeof (int));
 		if (gmtdefs.unix_time) use_font[0] = use_font[1] = TRUE;
-		use_font[gmtdefs.anot_font] = TRUE;
+		use_font[gmtdefs.annot_font[0]] = TRUE;
+		use_font[gmtdefs.annot_font[1]] = TRUE;
 		if (frame_info.header[0]) use_font[gmtdefs.header_font] = TRUE;
-		if (frame_info.label[0][0] || frame_info.label[1][0] || frame_info.label[2][0]) use_font[gmtdefs.label_font] = TRUE;
+		if (frame_info.axis[0].label[0] || frame_info.axis[1].label[0] || frame_info.axis[2].label[0]) use_font[gmtdefs.label_font] = TRUE;
 	}
 
 	if (do_paragraphs) {
@@ -373,12 +380,12 @@ main(int argc, char **argv)
 			done = TRUE;
 		}
 		else if ((fp = fopen (argv[fno], "r")) == NULL) {
-			fprintf (stderr, "psxy: Cannot open file %s\n", argv[fno]);
+			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
 			continue;
 		}
 
-		if (!nofile && gmtdefs.verbose) fprintf (stderr, "pstext: Working on file %s\n", argv[fno]);
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
+		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
 
 		while (fgets (line, BUFSIZ, fp)) {
 
@@ -394,8 +401,8 @@ main(int argc, char **argv)
 						n_paragraphs++;
 					}
 
-					nscan = sscanf (&line[2], "%s %s %d %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
-					if (!(GMT_scanf (txt_x, &xy[ix]) && GMT_scanf (txt_y, &xy[iy]))) {
+					nscan = sscanf (&line[2], "%s %s %lf %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
+					if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
 						fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
@@ -411,8 +418,12 @@ main(int argc, char **argv)
 						if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 						skip_text_records = FALSE;	/* Since we got here we do not want to skip */
 					}
+					if (convert_az) {
+						GMT_azim_to_angle (xy[0], xy[1], 0.1, T.paragraph_angle, &tmp);
+						T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
+					}
 					T.text_justify = (int)pjust;
-					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key);
+					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
 					if (T.block_justify == -99) {
 						fprintf (stderr, "%s: Record %d had bad justification info (set to LB)\n", GMT_program, n_read);
 						T.block_justify = 1;
@@ -420,14 +431,14 @@ main(int argc, char **argv)
 					if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
 						T.paragraph_font = atoi (this_font);
 					else {	/* Gave a font name */
-						T.paragraph_font = GMT_key_lookup (this_font, GMT_font_name, N_FONTS);
+						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, N_FONTS);
 						if (T.paragraph_font == N_FONTS) {
-							fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font_name[0]);
+							fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 							T.paragraph_font = 0;
 						}
 					}
-					if (!use_font[T.paragraph_font] && gmtdefs.want_euro_font) {	/* Must reencode this font */
-						ps_def_euro_font (T.paragraph_font);
+					if (!use_font[T.paragraph_font] && strcmp (gmtdefs.encoding.name, "Standard") != 0) {	/* Must reencode this font */
+						ps_encode_font (T.paragraph_font);
 						use_font[T.paragraph_font] = TRUE;
 					}
 					T.line_spacing = GMT_convert_units (txt_a, GMT_INCH);
@@ -469,9 +480,8 @@ main(int argc, char **argv)
 				n_read++;
 			}
 			else {	/* Old-style pstext input */
-				if (skip_text_records) continue;	/* Skip all records for this paragraph */
-				nscan = sscanf (line, "%s %s %d %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
-				if (!(GMT_scanf (txt_x, &xy[ix]) && GMT_scanf (txt_y, &xy[iy]))) {
+				nscan = sscanf (line, "%s %s %lf %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
+				if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
 					fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 					exit (EXIT_FAILURE);
 				}
@@ -486,29 +496,18 @@ main(int argc, char **argv)
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 			
-				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key);
+				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
 				if (T.block_justify == -99) {
 					fprintf (stderr, "%s: Record %d had bad justification info (skipped)\n", GMT_program, n_read);
 					continue;
 				}
+				if (convert_az) {
+					GMT_azim_to_angle (xy[0], xy[1], 0.1, T.paragraph_angle, &tmp);
+					T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
+				}
 				if (add) {
 					if (offset_outward) {	/* Smart offset according to justification (from Dave Huang) */
-						switch (T.block_justify & 3) {
-							case 1:
-								plot_x += T.x_offset;
-								break;
-							case 3:
-								plot_x -= T.x_offset;
-								break;
-						}
-						switch (T.block_justify >> 2) {
-							case 0:
-								plot_y += T.y_offset;
-								break;
-							case 2:
-								plot_y -= T.y_offset;
-								break;
-						}
+						GMT_smart_justify (T.block_justify, T.paragraph_angle, T.x_offset, T.y_offset, &plot_x, &plot_y);
 					} else {	/* Default hard offset */
 						plot_x += T.x_offset;
 						plot_y += T.y_offset;
@@ -517,9 +516,9 @@ main(int argc, char **argv)
 				if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
 					T.paragraph_font = atoi (this_font);
 				else {	/* Gave a font name */
-					T.paragraph_font = GMT_key_lookup (this_font, GMT_font_name, N_FONTS);
+					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, N_FONTS);
 					if (T.paragraph_font == N_FONTS) {
-						fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font_name[0]);
+						fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 						T.paragraph_font = 0;
 					}
 				}
@@ -529,22 +528,30 @@ main(int argc, char **argv)
 					ps_setfont (T.paragraph_font);
 					last_font = T.paragraph_font;
 				}
-				if (!use_font[T.paragraph_font] && gmtdefs.want_euro_font) {	/* Must reencode this font */
-					ps_def_euro_font (T.paragraph_font);
+				if (!use_font[T.paragraph_font] && strcmp (gmtdefs.encoding.name, "Standard") != 0) {	/* Must reencode this font */
+					ps_encode_font (T.paragraph_font);
 					use_font[T.paragraph_font] = TRUE;
 				}
-				if (T.boxflag) {
+				if (T.boxflag & 31) {
 					GMT_setpen (&T.boxpen);
-					GMT_textbox3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, (T.boxflag & 1), T.x_space, T.y_space, T.boxfill.rgb);
+					if (T.space_flag) {	/* Meant % of fontsize */
+						dx = 0.01 * T.x_space * T.font_size / 72.0;
+						dy = 0.01 * T.y_space * T.font_size / 72.0;
+					}
+					else {
+						dx = T.x_space;
+						dy = T.y_space;
+					}
+					GMT_textbox3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, (T.boxflag & 5), dx, dy, T.boxfill.rgb);
 					GMT_setpen (&T.txtpen);
 				}
 				if (outline) {
 					GMT_setpen (&T.txtpen);
-					GMT_text3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, TRUE);
+					GMT_text3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, TRUE);
 				}
 				if (solid) {
 					ps_setpaint (T.txtfill.rgb);
-					GMT_text3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, FALSE);
+					GMT_text3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, FALSE);
 				}
 			}
 		} /* Go read next line */
@@ -563,7 +570,7 @@ main(int argc, char **argv)
 
 	GMT_world_map = old_GMT_world_map;
 
-	if (frame_info.plot) GMT_map_basemap ();
+	GMT_map_basemap ();
 		
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
@@ -576,70 +583,31 @@ main(int argc, char **argv)
 			fprintf (stderr, "pstext: Plotted %d text-strings\n", n_paragraphs);
 	}
 	
+	GMT_free ((void *)use_font);
+	
 	GMT_end (argc, argv);
 }
 
-int GMT_just_decode (char *key)
-{
-	int i = -1, j = -1, k;
-
-	/* Converts justification info like LL (lower left) to justification indices */
-
-	for (k = 0; k < (int)strlen (key); k++) {
-		switch (key[k]) {
-			case 'b':	/* Bottom baseline */
-			case 'B':
-				j = 0;
-				break;
-			case 'm':	/* Middle baseline */
-			case 'M':
-				j = 4;
-				break;
-			case 't':	/* Top baseline */
-			case 'T':
-				j = 8;
-				break;
-			case 'l':	/* Left Justified */
-			case 'L':
-				i = 1;
-				break;
-			case 'c':	/* Center Justified */
-			case 'C':
-				i = 2;
-				break;
-			case 'r':	/* Right Justified */
-			case 'R':
-				i = 3;
-				break;
-			default:
-				return (-99);
-		}
-	}
-
-	if (i < 0) {
-		fprintf (stderr, "%s: Horizontal text justification not set, defaults to L(eft)\n", GMT_program);
-		i = 1;
-	}
-	if (j < 0) {
-		fprintf (stderr, "%s: Vertical text justification not set, defaults to B(ottom)\n", GMT_program);
-		j = 1;
-	}
-		
-	return (j + i);
-}
-
 void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT *T) {
 	char *boxpen_texture = CNULL, *vecpen_texture = CNULL;
 	int boxpen_width, boxpen_offset, boxpen_rgb[3];
 	int vecpen_width, vecpen_offset, vecpen_rgb[3];
 	int i;
+	double dx, dy;
 
 	boxpen_texture = GMT_convertpen (&T->boxpen, &boxpen_width, &boxpen_offset, boxpen_rgb);
 	vecpen_texture = GMT_convertpen (&T->vecpen, &vecpen_width, &vecpen_offset, vecpen_rgb);
 
+	if (T->space_flag) {	/* Meant % of fontsize */
+		dx = 0.01 * T->x_space * T->font_size / 72.0;
+		dy = 0.01 * T->y_space * T->font_size / 72.0;
+	}
+	else {
+		dx = T->x_space;
+		dy = T->y_space;
+	}
 	ps_words (x, y, text, n_words, T->line_spacing, T->paragraph_width, T->text_justify, T->paragraph_font, T->font_size,
-		T->paragraph_angle, T->txtfill.rgb, T->block_justify, T->boxflag, T->x_offset, T->y_offset,
-		T->x_space, T->y_space,
+		T->paragraph_angle, T->txtfill.rgb, T->block_justify, T->boxflag, T->x_offset, T->y_offset, dx, dy,
 		boxpen_width, boxpen_texture, boxpen_offset, boxpen_rgb,
 		vecpen_width, vecpen_texture, vecpen_offset, vecpen_rgb,
 		T->boxfill.rgb);
@@ -647,5 +615,7 @@ void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT
 	if (boxpen_texture) GMT_free ((void *)boxpen_texture);
 	if (vecpen_texture) GMT_free ((void *)vecpen_texture);
 
-	for (i = 0; i < n_words; i++) GMT_free ((void *)text[i]);
+	for (i = 0; i < n_words; i++) {
+		GMT_free ((void *)text[i]);
+	}
 }
