@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pswiggle.c,v 1.4.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: pswiggle.c,v 1.24 2004/08/17 01:35:45 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -21,13 +21,13 @@
  * exceeds the value of dist_gap, a data gap is assumed.  The user may
  * select a preferred direction where s/he wants the positive anomalies
  * to be pointing towards.  Positive normal vectors to the trackline
- * will then always be withing +- 90 degrees of this direction.
+ * will then always be within +- 90 degrees of this direction.
  * Separate colors may be specified for positive anomaly, outline of
  * anomaly, and trackline.  Plotting of the outline and track is optional.
  *
  * Author:	Paul Wessel
  * Date:	20-JUN-2000
- * Version:	3.4.3
+ * Version:	4
  *
  */
  
@@ -43,7 +43,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 
 main (int argc, char **argv)
 {
-	int i, j, n, k, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields;
+	int i, j, n, k, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields, wantx, wanty;
 	
 	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, track = FALSE, paint_wiggle, draw_scale = FALSE;
 	BOOLEAN do_fill = FALSE, negative = FALSE, fix = FALSE, done, gave_xy = FALSE;
@@ -83,7 +83,9 @@ main (int argc, char **argv)
 				case 'x':
 				case 'Y':
 				case 'y':
+				case 'b':
 				case 'c':
+				case 'f':
 				case ':':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
@@ -91,9 +93,6 @@ main (int argc, char **argv)
 				
 				/* Supplemental parameters */
 			
-  				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'A':
 					pref_az = atof (&argv[i][2]);
 					break;
@@ -141,8 +140,10 @@ main (int argc, char **argv)
 					j = 0;
 					if (argv[i][2] == 'x') gave_xy = TRUE, j = 1;
 					k = sscanf (&argv[i][2+j], "%[^/]/%[^/]/%lf", txt_a, txt_b, &length);
-					x0 = GMT_ddmmss_to_degree (txt_a);
-					y0 = GMT_ddmmss_to_degree (txt_b);
+					wantx = (gave_xy) ? GMT_IS_FLOAT : GMT_IS_LON;
+					wanty = (gave_xy) ? GMT_IS_FLOAT : GMT_IS_LAT;
+					error += GMT_verify_expectations (wantx, GMT_scanf_arg (txt_a, wantx, &x0), txt_a);
+					error += GMT_verify_expectations (wanty, GMT_scanf_arg (txt_b, wanty, &y0), txt_b);
 					units = strrchr (argv[i], '/');	units++;
 					if (k != 3) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  Correct syntax\n", GMT_program);
@@ -173,9 +174,9 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"pswiggle %s - Plot xyz-series along tracks\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pswiggle <xyz-files> -J<params> -R<west/east/south/north> -Z<scale>\n");
-		fprintf (stderr, "\t[-A<azimuth>] [-B<tickinfo>] [-C<center>] [-D<gap>] [-E<az/el>] [-G<fill>] [-H[<nrec>]]\n");
-		fprintf (stderr, "\t[-I<az>] [-K] [-M[<flag>]] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
-		fprintf (stderr, "\t[-U] [-V] [-W<outlinepen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]\n");
+		fprintf (stderr, "\t[-A<azimuth>] [-B<tickinfo>] [-C<center>] [-D<gap>] [-E<az/el>] [-G<fill>] [-H[<nrec>]] [-I<az>]\n");
+		fprintf (stderr, "\t[-K] [-M[<flag>]] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>] -U] [-V]\n");
+		fprintf (stderr, "\t[[-W<outlinepen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>] [-f[i|o]<colinfo>]\n");
 
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -212,6 +213,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 3 input columns\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -232,23 +234,23 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Z option:  scale must be nonzero\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 3;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 3) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 3;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 3) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 3 columns\n", GMT_program);
 		error++;
 	}
 	      
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	GMT_map_setup (west, east, south, north);
@@ -257,7 +259,7 @@ main (int argc, char **argv)
 	
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
@@ -313,7 +315,7 @@ main (int argc, char **argv)
 			done = TRUE;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Reading data from standard input\n", GMT_program);
 #ifdef SET_IO_MODE
-			GMT_setmode (0);
+			GMT_setmode (GMT_IN);
 #endif
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -327,12 +329,12 @@ main (int argc, char **argv)
 			ps_comment (line);
 		}
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 		if (GMT_io.multi_segments) {
 			GMT_fgets (line, BUFSIZ, fp);
 			if (gmtdefs.verbose) ps_comment (line);
 		}
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 3;
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
 
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -424,7 +426,7 @@ main (int argc, char **argv)
 	}
 			
 	GMT_map_clip_off();
-	if (frame_info.plot) GMT_map_basemap ();
+	GMT_map_basemap ();
 
 	if (draw_scale) GMT_draw_z_scale (x0, y0, length, zscale, gave_xy, units);
 	
@@ -490,7 +492,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 				GMT_y_plot[n] = y[i];
 			}
 		}
-		if (project_info.three_D) GMT_2D_to_3D (GMT_x_plot, GMT_y_plot, n);
+		if (project_info.three_D) GMT_2D_to_3D (GMT_x_plot, GMT_y_plot, project_info.z_level, n);
 	}
 	
 	
@@ -508,7 +510,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 	}
 	
 	if (track) {	/* Finally draw track line */
-		if (project_info.three_D) GMT_2D_to_3D (x, y, np);
+		if (project_info.three_D) GMT_2D_to_3D (x, y, project_info.z_level, np);
 		sprintf (comment, "Track line");
 		ps_comment (comment);
 		GMT_setpen (pen_t);
@@ -518,29 +520,26 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 
 void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units)
 {
-	double dy, a, b, off;
+	double dy, off, xx[4], yy[4];
 	char txt[128];
 	
 	GMT_setpen (&gmtdefs.tick_pen);
 
 	if (!gave_xy) {
-		GMT_geo_to_xy (x0, y0, &a, &b);
-		x0 = a;	y0 = b;
+		GMT_geo_to_xy (x0, y0, &xx[0], &yy[0]);
+		x0 = xx[0];	y0 = yy[0];
 	}
 		
 	if (units)
-		sprintf (txt, "%lg %s", length, units);
+		sprintf (txt, "%g %s", length, units);
 	else
-		sprintf (txt, "%lg", length);
+		sprintf (txt, "%g", length);
 	dy = 0.5 * length * zscale;	
-	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 - dy, 0.0, &a, &b);
-	ps_plot (a, b, 3);
-	GMT_xyz_to_xy (x0, y0 - dy, 0.0, &a, &b);
-	ps_plot (a, b, 2);
-	GMT_xyz_to_xy (x0, y0 + dy, 0.0, &a, &b);
-	ps_plot (a, b, 2);
-	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 + dy, 0.0, &a, &b);
-	ps_plot (a, b, 2);
-	off = ((gmtdefs.map_scale_height > 0.0) ? gmtdefs.tick_length : 0.0) + gmtdefs.anot_offset;
-	GMT_text3d (x0 + off, y0, project_info.z_level, gmtdefs.anot_font_size, gmtdefs.anot_font, txt, 0.0, 5, 0);
+	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 - dy, 0.0, &xx[0], &yy[0]);
+	GMT_xyz_to_xy (x0, y0 - dy, 0.0, &xx[1], &yy[1]);
+	GMT_xyz_to_xy (x0, y0 + dy, 0.0, &xx[2], &yy[2]);
+	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 + dy, 0.0, &xx[3], &yy[3]);
+	ps_line (xx, yy, 4, 3, FALSE, TRUE);
+	off = ((gmtdefs.map_scale_height > 0.0) ? gmtdefs.tick_length : 0.0) + gmtdefs.annot_offset[0];
+	GMT_text3D (x0 + off, y0, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt, 0.0, 5, 0);
 }
