@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegy.c,v 1.3 2001/04/11 19:58:09 pwessel Exp $
+ *	$Id: pssegy.c,v 1.5 2003/04/11 22:57:16 pwessel Exp $
  *
  *    Copyright (c) 1999-2001 by T. Henstock
  *    See README file for copying and redistribution conditions.
@@ -289,13 +289,14 @@ main (int argc, char **argv)
 		if(gmtdefs.verbose) fprintf(stderr, "%s: read in %d trace locations\n", GMT_program, n_tracelist);
 	}
 
-       GMT_put_history (argc, argv);   /* Update .gmtcommands */
+       GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
 /* set up map projection and PS plotting */
 	GMT_map_setup (w, e, s, n);
         ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
                 gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-                gmtdefs.dpi, GMT_INCH , gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
+                gmtdefs.dpi, GMT_INCH , gmtdefs.paper_width, gmtdefs.page_rgb,
+		     gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	
 /* define area for plotting and size of array for bitmap */
