@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: splitxyz.c,v 1.2.4.7 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: splitxyz.c,v 1.15 2004/04/25 09:10:46 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -21,8 +21,8 @@
  * and split it into profile segments.
  * 
  * Author:	W. H. F. Smith
- * Date:	24 July, 1991-2001
- * Version:	3.4.3
+ * Date:	24 July, 1991-2000
+ * Version:	4
  */
 
 #include "gmt.h"
@@ -47,8 +47,8 @@ main(int argc, char **argv)
 	BOOLEAN xy_only = FALSE;
 
         double  desired_azim = 90.0, tolerance_azim = 360.0;
-        double  min_dist = 100.0, xy_filter = 0.0, z_filter = 0.0;
-	double	d_gap = 10.0, course_change = 0.0, d_to_km;
+        double  min_dist = 0.0, xy_filter = 0.0, z_filter = 0.0;
+	double	d_gap = 10.0, course_change = 0.0;
 	double	dy, dx, last_d, last_c, last_s, csum, ssum, this_c, this_s, dotprod;
 	double	mean_azim, fwork[F_RES], *in, out[3];
 	
@@ -71,47 +71,48 @@ main(int argc, char **argv)
 				case 'H':
 				case 'V':
 				case ':':
+				case 'b':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
 				
 				/* Supplemental parameters */
 				
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
-                               case 'A':
+				case 'A':
                                         if ( (sscanf(&argv[i][2], "%lf/%lf", &desired_azim, &tolerance_azim)) != 2) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -A option: Can't dechiper values\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -A option: Can't decipher values\n", GMT_program);
 						error = TRUE;
 					}
                                         break;
-                                case 'C':
+				case 'C':
                                         if ( (sscanf(&argv[i][2], "%lf", &course_change)) != 1) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Can't dechiper value\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Can't decipher value\n", GMT_program);
 						error = TRUE;
 					}
                                         break;
                                 case 'D':
                                         if ( (sscanf(&argv[i][2], "%lf", &min_dist)) != 1) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Can't dechiper value\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Can't decipher value\n", GMT_program);
 						error = TRUE;
 					}
                                         break;
 				case 'F':
                                         if ( (sscanf(&argv[i][2], "%lf/%lf", &xy_filter, &z_filter)) != 2) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Can't dechiper values\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Can't decipher values\n", GMT_program);
 						error = TRUE;
 					}
                                         break;
                                 case 'G':
                                         if ( (sscanf(&argv[i][2], "%lf", &d_gap)) != 1) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: Can't dechiper value\n", GMT_program);
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: Can't decipher value\n", GMT_program);
 						error = TRUE;
 					}
                                         break;
 				case 'M':
-                                        map_units = GMT_geographic_in = GMT_geographic_out = TRUE;
+                                        map_units = TRUE;
+					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
                                         break;
 				case 'N':
                                         strcpy(basename, &argv[i][2]);
@@ -138,19 +139,18 @@ main(int argc, char **argv)
 
 	if (argc == 1 || GMT_quick) {	/* Display usage */
  		fprintf (stderr, "splitxyz %s - Split xyz[dh] files into segments\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  splitxyz [<xyz[dh]file>] -C<course_change> [-A<azimuth>/<tolerance>]\n");
-		fprintf(stderr,"\t[-D<minimum_distance>] [-F<xy_filter>/<z_filter>] [-G<gap>] [-H[<nrec>]] [-M]\n");
-		fprintf(stderr,"\t[-N<namestem>] [-S] [-V] [-Z] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
+		fprintf(stderr,"usage:  splitxyz [<xyz[dh]file>] -C<course_change> -D<minimum_distance>\n");
+		fprintf(stderr,"\t[-A<azimuth>/<tolerance>] [-F<xy_filter>/<z_filter>] [-G<gap>] [-H[<nrec>]] [-M]\n");
+		fprintf(stderr,"\t[-N<namestem>] [-S] [-V] [-Z] [-:] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
  		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
 		fprintf(stderr,"\tGive xyz[dh]file name or read stdin.\n");
 		fprintf(stderr,"\t-C  Profile ends when change of heading exceeds <course_change>.\n");
+		fprintf(stderr,"\t-D Only write profile if length is at least <minimum_distance>.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr,"\t-A Only write profile if mean direction is w/in +/- <tolerance>\n");
 		fprintf(stderr,"\t     of <azimuth>. [Default = All].\n");
-		fprintf(stderr,"\t-D Only write profile if length is at least <minimum_distance>.\n");
-		fprintf(stderr,"\t     [Default = 100 dist units].\n");
 		fprintf(stderr,"\t-F Filter the data.  Give full widths of cosine arch filters for xy and z.\n");
 		fprintf(stderr,"\t   Defaults are both widths = 0, giving no filtering.\n");
 		fprintf(stderr,"\t   Use negative width to highpass.\n");
@@ -168,10 +168,16 @@ main(int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf(stderr,"\t     Default input columns is set given -S and -Z options.\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
         }
 
+	if (min_dist <= 0.0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Minimum segment distance must be positive\n", GMT_program);
+		error++;
+	}
 	if (course_change <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Course change tolerance must be positive\n", GMT_program);
 		error++;
@@ -196,48 +202,47 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Cannot specify z-filter while using -Z option\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[1] && !basename[0]) {
+	if (GMT_io.binary[GMT_OUT] && !basename[0]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary output requires a namestem in -N\n", GMT_program);
 		error++;
 	}
 	n_required_fields = (dh_supplied) ? 5 : ((xy_only) ? 2 : 3);
-	if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = n_required_fields;
-	if (GMT_io.binary[0])
-		n_expected_fields = GMT_io.ncol[0];
+	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_required_fields;
+	if (GMT_io.binary[GMT_IN])
+		n_expected_fields = GMT_io.ncol[GMT_IN];
 	else
 		n_expected_fields = n_required_fields;
-	if (GMT_io.binary[0] && n_expected_fields < n_required_fields) {
+	if (GMT_io.binary[GMT_IN] && n_expected_fields < n_required_fields) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d columns\n", GMT_program, n_required_fields);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 	
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
-	GMT_setmode (1);
+	GMT_setmode (GMT_OUT);
 #endif
 
 	tolerance_azim *= D2R;
-	if (desired_azim > 180.0) desired_azim -= 180.0;	/* Put in Easterly strikes  */
+	/* if (desired_azim > 180.0) desired_azim -= 180.0; */	/* Put in Easterly strikes  */
 	desired_azim = D2R * (90.0 - desired_azim);	/* Work in cartesian angle and radians  */
 	course_change *= D2R;
-	d_to_km = 0.001 * 2.0 * M_PI * gmtdefs.ellipse[N_ELLIPSOIDS-1].eq_radius / 360.0;
 
 	if (fp == NULL) {
 		fp = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
         n_alloc = GMT_CHUNK;
@@ -246,7 +251,7 @@ main(int argc, char **argv)
 
         data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (!GMT_io.binary[0] && gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (!GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
@@ -268,8 +273,8 @@ main(int argc, char **argv)
 				dy = (data[ndata].y - data[ndata-1].y);
 				dx = (data[ndata].x - data[ndata-1].x);
 				if (map_units) {
-					dy *= d_to_km;
-					dx *= (d_to_km * cos (0.5 * (data[ndata].y + data[ndata-1].y) * D2R) );
+					dy *= DEG2KM;
+					dx *= (DEG2KM * cos (0.5 * (data[ndata].y + data[ndata-1].y) * D2R) );
 				}
 				if (dy == 0.0 && dx == 0.0) {
 					data[ndata].d = data[ndata-1].d;
@@ -373,9 +378,11 @@ main(int argc, char **argv)
 			
 				/* List is long enough.  Check strike. Compute mean_azim in range [-pi/2, pi/2]:  */
 				
-				mean_azim = d_asin(ssum/sqrt(csum*csum + ssum*ssum));
+				/* mean_azim = d_asin(ssum/sqrt(csum*csum + ssum*ssum)); */
+				mean_azim = d_atan2 (ssum, csum);
 				mean_azim = fabs(mean_azim - desired_azim);
-				if (mean_azim <= tolerance_azim || mean_azim >= (M_PI-tolerance_azim) ) {
+				/* if (mean_azim <= tolerance_azim || mean_azim >= (M_PI-tolerance_azim) ) { */
+				if (mean_azim <= tolerance_azim) {
 				
 					/* List has acceptable strike.  */
 					
