@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: trend2d.c,v 1.3.4.8 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: trend2d.c,v 1.14 2004/04/25 09:10:46 pwessel Exp $
  *
  *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -62,7 +62,7 @@
  * Date:	17 June 1991-2000.
  * Revised:	12-JUN-1998 for GMT 3.1 (PW)
  *		10-JUL-2000 for GMT 3.3.5 (PW) Added -L option
- * Version:	3.4.3
+ * Version:	4
  */
 
 #include "gmt.h"
@@ -121,6 +121,8 @@ main(int argc, char **argv)
 				case 'H':
 				case 'V':
 				case ':':
+				case 'b':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
@@ -128,9 +130,6 @@ main(int argc, char **argv)
 				/* Supplemental parameters */
 				
 
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
 				case 'F':
 					j = 2;
 				       while (argv[i][j]) {
@@ -171,7 +170,9 @@ main(int argc, char **argv)
 					confid = (argv[i][2]) ? atof(&argv[i][2]) : 0.51;
 					break;
 				case 'L':
-					GMT_geographic_in = GMT_geographic_out = TRUE;
+					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
+					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'N':
 					if (argv[i][strlen (argv[i]) - 1] == 'r') robust = TRUE;
@@ -197,7 +198,7 @@ main(int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf(stderr,"trend2d %s - Fit a [weighted] [robust] polynomial for z = f(x,y) to ascii xyz[w]\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:  trend2d -F<xymrw> -N<n_model>[r] [<xyz[w]file>] [-C<condition_#>] [-H[<nrec>]] [-I[<confidence>]]\n\n");
-		fprintf(stderr,"	[-L] [-V] [-W] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
+		fprintf(stderr,"	[-V] [-W] [-:] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 
@@ -211,7 +212,6 @@ main(int argc, char **argv)
 		fprintf(stderr,"\t-I Iteratively Increase # model parameters, to a max of <n_model> so long as the\n");
 		fprintf(stderr,"\t   reduction in variance is significant at the <confidence> level.\n");
 		fprintf(stderr,"\t   Give -I without a number to default to 0.51 confidence level.\n");
-		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		GMT_explain_option ('V');
 		fprintf(stderr,"\t-W Weighted input given, weights in 4th column.  [Default is unweighted].\n");
 		GMT_explain_option (':');
@@ -219,6 +219,8 @@ main(int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf(stderr,"\t   Default is 3 (or 4 if -W is set) columns.\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -243,28 +245,28 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option.  Must request 1-10 parameters\n", GMT_program);
 		error++;
 	}	
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
 	n_req = (weighted_input) ? 4 : 3;
-	if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = n_req;
-	if (GMT_io.binary[0] && GMT_io.ncol[0] < n_req) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
+	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
-	GMT_setmode (1);
+	GMT_setmode (GMT_OUT);
 #endif
 	project_info.w = 0;	project_info.e = 360.0;	/* For -L not to cause trouble in GMT_input */
 	np = n_model_max;	/* Row dimension for matrices gtg and v  */
@@ -287,7 +289,7 @@ main(int argc, char **argv)
 	transform_x(data, n_data, xmin, xmax, ymin, ymax);	/* Set domain to [-1, 1] or [-pi, pi]  */
 
 	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s:  Read %d data with X values from %.8lg to %.8lg\n", GMT_program, n_data, xmin, xmax);
+		fprintf(stderr,"%s:  Read %d data with X values from %.8g to %.8g\n", GMT_program, n_data, xmin, xmax);
 		fprintf(stderr,"N_model\tRank\tChi_Squared\tSignificance\n");
 	}
 
@@ -420,14 +422,14 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, doub
 	if (fp == NULL) {
 		fp = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
 	(*data) = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted_input;
+	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
