@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys.c,v 1.1.1.1.4.2 2002/02/27 17:41:10 pwessel Exp $
+ *	$Id: x2sys.c,v 1.17 2004/08/20 20:43:38 pwessel Exp $
  *
- *      Copyright (c) 1999-2002 by P. Wessel
+ *      Copyright (c) 1999-2001 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      Contact info: www.soest.hawaii.edu/wessel
+ *      Contact info: www.soest.hawaii.edu/pwessel
  *--------------------------------------------------------------------*/
 /* x2sys.c contains the source code for the X2SYS crossover library
  * libx2sys.a.  The code is copylefted under the GNU Public Library
@@ -29,9 +29,7 @@
  * x2sys_read_list	: Read an ascii list of track names
  * x2sys_distances	: Calculate cumulative distances along track
  * x2sys_dummytimes	: Make dummy times for tracks missing times
- * x2sys_init_track	: Prepares a track for crossover analysis
  * x2sys_n_data_cols	: Gives number of data columns in this data set
- * x2sys_crossover	: Calculates crossovers for two data sets
  * x2sys_fopen		: Opening files with error message and exit
  * x2sys_fclose		: Closes files and gives error messages
  * x2sys_skip_header	: Skips the header record(s) in the open file
@@ -41,18 +39,22 @@
  * x2sys_free_info	: Frees the information structure
  * x2sys_free_data	: Frees the data matrix
  *------------------------------------------------------------------
+ * Core crossover functions are part of GMT:
+ * GMT_init_track	: Prepares a track for crossover analysis
+ * GMT_crossover	: Calculates crossovers for two data sets
+ * GMT_x_alloc		: Allocate space for crossovers
+ * GMT_x_free		: Free crossover structure
+ * GMT_ysort		: Sorting routine used in x2sys_init_track [Hidden]
+ *------------------------------------------------------------------
  * These routines are local to x2sys and used by the above routines:
  *
  * x2sys_set_home	: Initializes X2SYS paths
  * x2sys_record_length	: Returns the record length of current file
- * x2sys_ysort		: Sorting routine used in x2sys_init_track
- * x2sys_x_alloc	: Allocate space for crossovers
- * x2sys_x_free		: Free crossover structure
  *
  *------------------------------------------------------------------
  * Author:	Paul Wessel
- * Date:	26-JUN-1999
- * Version:	1.0, based on the spirit of the old xsystem code
+ * Date:	18-MAY-2004
+ * Version:	1.1, based on the spirit of the old xsystem code
  *
  */
 
@@ -61,28 +63,61 @@
 /* Global variables used by X2SYS functions */
 
 char *X2SYS_HOME;
-double *x2sys_Y;
+char *X2SYS_program;
 
 char *x2sys_xover_format = "%9.5lf %9.5lf %10.1lf %10.1lf %9.2lf %9.2lf %9.2lf %8.1lf %8.1lf %8.1lf %5.1lf %5.1lf\n";
 char *x2sys_xover_header = "%s %d %s %d\n";
 char *x2sys_header = "> %s %d %s %d\n";
+struct MGD77_CONTROL M;
 
 void x2sys_set_home (void);
 int x2sys_record_length (struct X2SYS_INFO *s);
-void x2sys_x_alloc (struct X2SYS_XOVER *X, int nx_alloc);
-void x2sys_x_free (struct X2SYS_XOVER *X);
+int MGD77_items[MGD77_DATA_COLS] = { 0, MGD77_LONGITUDE, MGD77_LATITUDE, 10, 11, 14, 15, 16, 18, 19, 20, 21, 22};
+
+#define MAX_DATA_PATHS 32
+char *x2sys_datadir[MAX_DATA_PATHS];	/* Directories where track data may live */
+int n_x2sys_paths = 0;			/* Number of these directories */
+
+void x2sys_path (char *fname, char *path)
+{
+	sprintf (path, "%s%c%s", X2SYS_HOME, DIR_DELIM, fname);
+}
 
 FILE *x2sys_fopen (char *fname, char *mode)
 {
 	FILE *fp;
+	char file[BUFSIZ];
 
-	if ((fp = fopen (fname, mode)) == NULL) {
-		fprintf (stderr, "x2sys: Error from fopen on %s using mode %s\n", fname, mode);
-		exit (EXIT_FAILURE);
+	if (mode[0] == 'w') {	/* Writing: Do this in X2SYS_HOME */
+		x2sys_path (fname, file);
+		if ((fp = fopen (file, mode)) == NULL) {	/* Not in X2SYS_HOME directory either */
+			fprintf (stderr, "x2sys: Error from fopen on %s using mode %s\n", file, mode);
+			exit (EXIT_FAILURE);
+		}
+	}
+	else {			/* Reading: Try both current directory and X2SYS_HOME */
+		if ((fp = fopen (fname, mode)) == NULL) {	/* Not in current directory */
+			x2sys_path (fname, file);
+			if ((fp = fopen (file, mode)) == NULL) {	/* Not in X2SYS_HOME directory either */
+				fprintf (stderr, "x2sys: Error from fopen on %s using mode %s\n", fname, mode);
+				exit (EXIT_FAILURE);
+			}
+		}
 	}
 	return (fp);
 }
 
+int x2sys_access (char *fname,  int mode)
+{
+	int k;
+	char file[BUFSIZ];
+	x2sys_path (fname, file);
+	if ((k = access (file, mode))) {	/* Not in X2SYS_HOME directory */
+		k = access (fname, mode);	/* Try in current directory */
+	}
+	return (k);
+}
+
 void x2sys_fclose (char *fname, FILE *fp)
 {
 
@@ -97,7 +132,7 @@ void x2sys_skip_header (FILE *fp, struct X2SYS_INFO *s)
 	int i;
 	char line[BUFSIZ];
 
-	if (s->ascii_in) {	/* Ascii, skip records */
+	if (s->ascii_in) {	/* ASCII, skip records */
 		for (i = 0; i < s->skip; i++) fgets (line, BUFSIZ, fp);
 	}
 	else {			/* Binary, skip bytes */
@@ -112,7 +147,7 @@ void x2sys_skip_header (FILE *fp, struct X2SYS_INFO *s)
  * array called data[] with each data value in it.
  */
 
-int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s)
+int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_IO *G)
 {
 	int j, k, i, n_read = 0;
 	BOOLEAN error = FALSE;
@@ -130,19 +165,32 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s)
 		switch (s->info[j].intype) {
 
 			case 'A':	/* ASCII Card Record, must extract columns */
-				if (j == 0) fgets (line, BUFSIZ, fp);	/* Get new record */
+				if (j == 0) {
+					s->ms_next = FALSE;
+					fgets (line, BUFSIZ, fp);	/* Get new record */
+					while (line[0] == '#' || line[0] == s->ms_flag) {
+						fgets (line, BUFSIZ, fp);
+						if (s->multi_segment) s->ms_next = TRUE;
+					}
+				}
 				strncpy (buffer, &line[s->info[j].start_col], s->info[j].n_cols);
 				buffer[s->info[j].n_cols] = 0;
-				data[j] = atof (buffer);
+				GMT_scanf (buffer, G->in_col_type[j], &data[j]);
 				break;
 
 			case 'a':	/* ASCII Record, get all columns directly */
 				k = 0;
+				s->ms_next = FALSE;
 				fgets (line, BUFSIZ, fp);
-				p = strtok (line, " ,\t");
+				while (line[0] == '#' || line[0] == s->ms_flag) {
+					fgets (line, BUFSIZ, fp);
+					if (s->multi_segment) s->ms_next = TRUE;
+				}
+				p = strtok (line, " ,\t\n");
 				while (p) {
-					data[k++] = atof (p);
-					p = strtok (NULL, " ,\t");
+					GMT_scanf (p, G->in_col_type[k], &data[k]);
+					k++;;
+					p = strtok (NULL, " ,\t\n");
 				}
 				return ((k != s->n_fields) ? -1 : 0);
 				break;
@@ -195,12 +243,13 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s)
 		else if (s->info[i].do_scale)
 			data[i] = data[i] * s->info[i].scale + s->info[i].offset;
 		if (GMT_is_dnan (data[i])) s->info[i].has_nans = TRUE;
+		if (i == s->x_col && s->geographic) GMT_lon_range_adjust (s->geodetic, &data[i]);
 	}
 
 	return ((error || n_read != s->n_fields) ? -1 : 0);
 }
  
-int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p)
+int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
 {
 	/* Reads the entire contents of the file given and returns the
 	 * number of data records.  The data matrix is return in the
@@ -211,8 +260,19 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 	size_t n_alloc;
 	FILE *fp;
 	double **z, *rec;
+	char path[BUFSIZ];
 
-	if ((fp = fopen (fname, "rb")) == NULL) {
+ 	if (n_x2sys_paths) {
+  		if (x2sys_get_data_path (path, fname)) {
+   			fprintf (stderr, "x2sys_read_file : Cannot find track %s\n", fname);
+     			return (-1);
+  		}
+  		if ((fp = fopen (path, "rb")) == NULL) {
+   			fprintf (stderr, "x2sys_read_mgd77file : Cannot open file %s\n", path);
+     			return (-1);
+  		}
+	}
+	else if ((fp = fopen (fname, "rb")) == NULL) {
 		fprintf (stderr, "x2sys_read_file: Could not open %s\n", fname);
 		return (-1);
 	}
@@ -222,19 +282,26 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 	rec = (double *) GMT_memory (VNULL, s->n_fields, sizeof (double), "x2sys_read_file");
 	z = (double **) GMT_memory (VNULL, s->n_fields, sizeof (double *), "x2sys_read_file");
 	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory (VNULL, n_alloc, sizeof (double), "x2sys_read_file");
+	p->ms_rec = (int *) GMT_memory (VNULL, n_alloc, sizeof (int), "x2sys_read_file");
 	x2sys_skip_header (fp, s);
+	if (s->multi_segment) p->n_segments = -1;	/* So that first increment sets it to 0 */
 
 	j = 0;
-	while (!x2sys_read_record (fp, rec, s)) {
+	while (!x2sys_read_record (fp, rec, s, G)) {	/* Gets the next data record */
 		for (i = 0; i < s->n_fields; i++) z[i][j] = rec[i];
+		if (s->multi_segment && s->ms_next) p->n_segments++;
+		p->ms_rec[j] = p->n_segments;
 		j++;
 		if (j == (int)n_alloc) {	/* Get more */
 			n_alloc += GMT_CHUNK;
 			for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], n_alloc, sizeof (double), "x2sys_read_file");
+			p->ms_rec = (int *) GMT_memory ((void *)p->ms_rec, n_alloc, sizeof (int), "x2sys_read_file");
 		}
 	}
 
 	fclose (fp);
+	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)j, sizeof (double), "x2sys_read_file");
+	p->ms_rec = (int *) GMT_memory ((void *)p->ms_rec, (size_t)j, sizeof (int), "x2sys_read_file");
 
 	*data = z;
 
@@ -245,7 +312,7 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 	return (j);
 }
 
-struct X2SYS_INFO *x2sys_initialize (char *fname)
+struct X2SYS_INFO *x2sys_initialize (char *fname, struct GMT_IO *G)
 {
 	/* Reads the format definition file and sets all information variables */
 
@@ -261,14 +328,22 @@ struct X2SYS_INFO *x2sys_initialize (char *fname)
 	X->info = (struct X2SYS_DATA_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
 	X->ascii_in = TRUE;
 	X->x_col = X->y_col = X->t_col = -1;
-	sprintf (line, "%s%c%s.def", X2SYS_HOME, DIR_DELIM, fname);
+	X->ms_flag = '>';	/* Default multisegment header flag */
+	sprintf (line, "%s%c%s.def\0", X2SYS_HOME, DIR_DELIM, fname);
 
 	fp = x2sys_fopen (line, "r");
 
-	if (!strcmp (fname, "gmt"))
+	if (!strcmp (fname, "gmt")) {
 		X->read_file = (PFI) x2sys_read_gmtfile;
-	else if (!strcmp (fname, "mgd77"))
+		X->geographic = TRUE;
+		X->geodetic = 0;
+	}
+	else if (!strcmp (fname, "mgd77")) {
 		X->read_file = (PFI) x2sys_read_mgd77file;
+		X->geographic = TRUE;
+		X->geodetic = 1;
+		MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	}
 	else
 		X->read_file = (PFI) x2sys_read_file;
 
@@ -277,6 +352,11 @@ struct X2SYS_INFO *x2sys_initialize (char *fname)
 		if (line[0] == '#') {
 			if (!strncmp (line, "#SKIP ", 6)) X->skip = atoi (&line[6]);
 			if (!strncmp (line, "#BINARY ", 7)) X->ascii_in = FALSE;
+			if (!strncmp (line, "#GEO ", 3)) X->geographic = TRUE;
+			if (!strncmp (line, "#MULTISEG ", 9)) {
+				X->multi_segment = TRUE;
+				sscanf (line, "%*s %c", &X->ms_flag);
+			}
 			continue;
 		}
 
@@ -310,6 +390,7 @@ struct X2SYS_INFO *x2sys_initialize (char *fname)
 	for (i = 0; i < X->n_fields; i++) {	/* Default is same order and use all columns */
 		X->out_order[i] = i;
 		X->use_column[i] = 1;
+		G->in_col_type[i] = G->out_col_type[i] = (X->x_col == i) ? GMT_IS_LON : ((X->y_col == i) ? GMT_IS_LAT : GMT_IS_UNKNOWN);
 	}
 	X->n_data_cols = x2sys_n_data_cols (X);
 	X->rec_size = (8 + X->n_data_cols) * sizeof (double);
@@ -426,9 +507,9 @@ void x2sys_set_home (void)
 		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 1), 1, "x2sys_set_home");
 		strcpy (X2SYS_HOME, this);
 	}
-	else if ((this = getenv ("GMTHOME")) != CNULL) {	/* Use GMT path */
-		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 11), 1, "x2sys_set_home");
-		sprintf (X2SYS_HOME, "%s%cshare%cx2sys", this, DIR_DELIM, DIR_DELIM);
+	else if ((this = getenv ("GMTHOME")) != CNULL) {	/* Assume GMT path */
+		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 13), 1, "x2sys_set_home");
+		sprintf (X2SYS_HOME, "%s%cshare%cx2sys\0", this, DIR_DELIM, DIR_DELIM);
 	}
 	else {	/* Set default path */
 #ifdef _WIN32
@@ -455,387 +536,6 @@ void x2sys_free_data (double **data, int n)
 	GMT_free ((void *)data);
 }
 
-struct X2SYS_SEGMENT *x2sys_init_track (double x[], double y[], int n)
-{
-	/* x2sys_init_track accepts the x-y track of length n and returns an array of
-	 * line segments that have been sorted on the minimum y-coordinate
-	 */
- 
-	int a, b;
-	size_t nl = n - 1;
-	struct X2SYS_SEGMENT *L;
-	int x2sys_ysort (const void *p1, const void *p2);
-
-	if (nl <= 0) {
-		fprintf (stderr, "x2sys: ERROR in x2sys_init_track; nl = %d\n", (int)nl);
-		exit (EXIT_FAILURE);
-	}
-
-	L = (struct X2SYS_SEGMENT *) GMT_memory (VNULL, nl, sizeof (struct X2SYS_SEGMENT), "x2sys_init_track");
-
-	for (a = 0, b = 1; b < n; a++, b++) {
-		if (y[b] < y[a]) {
-			L[a].start = b;
-			L[a].stop = a;
-		}
-		else {
-			L[a].start = a;
-			L[a].stop = b;
-		}
-	}
-
-	/* Sort on minimum y-coordinate, if tie then on 2nd coordinate */
-
-	x2sys_Y = y;	/* Sort routine needs this pointer */
-
-	qsort ((void *)L, nl, sizeof (struct X2SYS_SEGMENT), x2sys_ysort);
-
-	x2sys_Y = (double *)NULL;
-
-	return (L);
-}
-
-
-int x2sys_ysort (const void *p1, const void *p2)
-{
-	/* The double pointer x2sys_Y must be set to point to the relevant y-array
-	 * before this call!!! */
-
-	struct X2SYS_SEGMENT *a, *b;
-
-	a = (struct X2SYS_SEGMENT *)p1;
-	b = (struct X2SYS_SEGMENT *)p2;
-
-	if (x2sys_Y[a->start] < x2sys_Y[b->start]) return -1;
-	if (x2sys_Y[a->start] > x2sys_Y[b->start]) return  1;
-
-	/* Here they have the same low y-value, now sort on other y value */
-
-	if (x2sys_Y[a->stop] < x2sys_Y[b->stop]) return -1;
-	if (x2sys_Y[a->stop] > x2sys_Y[b->stop]) return  1;
-
-	/* Identical */
-
-	return (0);
-}
-
-int x2sys_crossover (double xa[], double ya[], struct X2SYS_SEGMENT A[], int na, double xb[], double yb[], struct X2SYS_SEGMENT B[], int nb, BOOLEAN internal, struct X2SYS_XOVER *X)
-{
-	int this_a, this_b, n_seg_a, n_seg_b, nx, xa_start, xa_stop, xb_start, xb_stop, ta_start, ta_stop, tb_start, tb_stop;
-	int nx_alloc;
-	BOOLEAN new_a, new_b, new_a_time;
-	double del_xa, del_xb, del_ya, del_yb, i_del_xa, i_del_xb, i_del_ya, i_del_yb, slp_a, slp_b, xc, yc, tx_a, tx_b;
-
-	if (na < 2 || nb < 2) return (0);	/* Need at least 2 points to make a segment */
-
-	this_a = this_b = nx = 0;
-	new_a = new_b = TRUE;
-	nx_alloc = GMT_SMALL_CHUNK;
-
-	n_seg_a = na - 1;
-	n_seg_b = nb - 1;
-
-	x2sys_x_alloc (X, -nx_alloc);
-
-	while (this_a < n_seg_a && yb[B[this_b].start] > ya[A[this_a].stop]) this_a++;	/* Go to first possible A segment */
-
-	while (this_a < n_seg_a) {
-
-		/* First check for internal neighboring segments which cannot cross */
-
-		if (internal && (this_a == this_b || (A[this_a].stop == B[this_b].start || A[this_a].start == B[this_b].stop) || (A[this_a].start == B[this_b].start || A[this_a].stop == B[this_b].stop))) {	/* Neighboring segments cannot cross */
-			this_b++;
-			new_b = TRUE;
-		}
-		else if (yb[B[this_b].start] > ya[A[this_a].stop]) {	/* Reset this_b and go to next A */
-			this_b = n_seg_b;
-		}
-		else if (yb[B[this_b].stop] < ya[A[this_a].start]) {	/* Must advance B in y-direction */
-			this_b++;
-			new_b = TRUE;
-		}
-		else {	/* Current A and B segments overlap in y-range */
-
-			if (new_a) {	/* Must sort this A new segment in x */
-				if (xa[A[this_a].stop] < xa[A[this_a].start]) {
-					xa_start = A[this_a].stop;
-					xa_stop  = A[this_a].start;
-				}
-				else {
-					xa_start = A[this_a].start;
-					xa_stop  = A[this_a].stop;
-				}
-				new_a = FALSE;
-				new_a_time = TRUE;
-			}
-
-			if (new_b) {	/* Must sort this new B segment in x */
-				if (xb[B[this_b].stop] < xb[B[this_b].start]) {
-					xb_start = B[this_b].stop;
-					xb_stop  = B[this_b].start;
-				}
-				else {
-					xb_start = B[this_b].start;
-					xb_stop  = B[this_b].stop;
-				}
-				new_b = FALSE;
-			}
-
-			/* OK, first check for any overlap in x range */
-
-			if (!((xa[xa_stop] < xb[xb_start]) || (xa[xa_start] > xb[xb_stop]))) {
-
-				/* We have segment overlap in x.  Now check if the segments cross  */
-
-				del_xa = xa[xa_stop] - xa[xa_start];
-				del_xb = xb[xb_stop] - xb[xb_start];
-				del_ya = ya[xa_stop] - ya[xa_start];
-				del_yb = yb[xb_stop] - yb[xb_start];
-
-				if (del_xa == 0.0) {	/* Vertical A segment: Special case */
-
-					i_del_xb = 1.0 / del_xb;
-					yc = yb[xb_start] + (xa[xa_start] - xb[xb_start]) * del_yb * i_del_xb;
-					if (!(yc < ya[A[this_a].start] || yc > ya[A[this_a].stop])) {	/* Did cross within the segment extents */
-						/* Only accept xover if occuring before segment end (in time) */
-
-						if (xb_start < xb_stop) {
-							tb_start = xb_start;	/* B Node first in time */
-							tb_stop = xb_stop;	/* B Node last in time */
-						}
-						else {
-							tb_start = xb_stop;	/* B Node first in time */
-							tb_stop = xb_start;	/* B Node last in time */
-						}
-						if (new_a_time) {
-							if (xa_start < xa_stop) {
-								ta_start = xa_start;	/* A Node first in time */
-								ta_stop = xa_stop;	/* A Node last in time */
-							}
-							else {
-								ta_start = xa_stop;	/* A Node first in time */
-								ta_stop = xa_start;	/* A Node last in time */
-							}
-							new_a_time = FALSE;
-						}
-
-						tx_a = ta_start + fabs ((yc - ya[ta_start]) / del_ya);
-						tx_b = tb_start + fabs (xa[xa_start] - xb[tb_start]) * i_del_xb;
-						if (tx_a < ta_stop && tx_b < tb_stop) {
-							X->x[nx] = xa[xa_start];
-							X->y[nx] = yc;
-							X->xnode[0][nx] = tx_a;
-							X->xnode[1][nx] = tx_b;
-							nx++;
-						}
-					}
-				}
-				else if (del_xb == 0.0) {	/* Vertical B segment: Special case */
-					
-					i_del_xa = 1.0 / del_xa;
-					yc = ya[xa_start] + (xb[xb_start] - xa[xa_start]) * del_ya * i_del_xa;
-					if (!(yc < yb[B[this_b].start] || yc > yb[B[this_b].stop])) {	/* Did cross within the segment extents */
-						/* Only accept xover if occuring before segment end (in time) */
-
-						if (xb_start < xb_stop) {
-							tb_start = xb_start;	/* B Node first in time */
-							tb_stop = xb_stop;	/* B Node last in time */
-						}
-						else {
-							tb_start = xb_stop;	/* B Node first in time */
-							tb_stop = xb_start;	/* B Node last in time */
-						}
-						if (new_a_time) {
-							if (xa_start < xa_stop) {
-								ta_start = xa_start;	/* A Node first in time */
-								ta_stop = xa_stop;	/* A Node last in time */
-							}
-							else {
-								ta_start = xa_stop;	/* A Node first in time */
-								ta_stop = xa_start;	/* A Node last in time */
-							}
-							new_a_time = FALSE;
-						}
-
-						tx_a = ta_start + fabs (xb[xb_start] - xa[ta_start]) * i_del_xa;
-						tx_b = tb_start + fabs ((yc - yb[tb_start]) / del_yb);
-						if (tx_a < ta_stop && tx_b < tb_stop) {
-							X->x[nx] = xb[xb_start];
-							X->y[nx] = yc;
-							X->xnode[0][nx] = tx_a;
-							X->xnode[1][nx] = tx_b;
-							nx++;
-						}
-					}
-				}
-				else if (del_ya == 0.0) {	/* Horizontal A segment: Special case */
-					
-					i_del_yb = 1.0 / del_yb;
-					xc = xb[xb_start] + (ya[xa_start] - yb[xb_start]) * del_xb * i_del_yb;
-					if (!(xc < xa[xa_start] || xc > xa[xa_stop])) {	/* Did cross within the segment extents */
-
-						/* Only accept xover if occuring before segment end (in time) */
-
-						if (xb_start < xb_stop) {
-							tb_start = xb_start;	/* B Node first in time */
-							tb_stop = xb_stop;	/* B Node last in time */
-						}
-						else {
-							tb_start = xb_stop;	/* B Node first in time */
-							tb_stop = xb_start;	/* B Node last in time */
-						}
-						if (new_a_time) {
-							if (xa_start < xa_stop) {
-								ta_start = xa_start;	/* A Node first in time */
-								ta_stop = xa_stop;	/* A Node last in time */
-							}
-							else {
-								ta_start = xa_stop;	/* A Node first in time */
-								ta_stop = xa_start;	/* A Node last in time */
-							}
-							new_a_time = FALSE;
-						}
-
-						tx_a = ta_start + fabs (xc - xa[ta_start]) / del_xa;
-						tx_b = tb_start + fabs ((ya[xa_start] - yb[tb_start]) * i_del_yb);
-						if (tx_a < ta_stop && tx_b < tb_stop) {
-							X->y[nx] = ya[xa_start];
-							X->x[nx] = xc;
-							X->xnode[0][nx] = tx_a;
-							X->xnode[1][nx] = tx_b;
-							nx++;
-						}
-					}
-				}
-				else if (del_yb == 0.0) {	/* Horizontal B segment: Special case */
-					
-					i_del_ya = 1.0 / del_ya;
-					xc = xa[xa_start] + (yb[xb_start] - ya[xa_start]) * del_xa * i_del_ya;
-					if (!(xc < xb[xb_start] || xc > xb[xb_stop])) {	/* Did cross within the segment extents */
-
-						/* Only accept xover if occuring before segment end (in time) */
-
-						if (xb_start < xb_stop) {
-							tb_start = xb_start;	/* B Node first in time */
-							tb_stop = xb_stop;	/* B Node last in time */
-						}
-						else {
-							tb_start = xb_stop;	/* B Node first in time */
-							tb_stop = xb_start;	/* B Node last in time */
-						}
-						if (new_a_time) {
-							if (xa_start < xa_stop) {
-								ta_start = xa_start;	/* A Node first in time */
-								ta_stop = xa_stop;	/* A Node last in time */
-							}
-							else {
-								ta_start = xa_stop;	/* A Node first in time */
-								ta_stop = xa_start;	/* A Node last in time */
-							}
-							new_a_time = FALSE;
-						}
-
-						tx_a = ta_start + fabs ((yb[xb_start] - ya[ta_start]) * i_del_ya);
-						tx_b = tb_start + fabs (xc - xb[tb_start]) / del_xb;
-						if (tx_a < ta_stop && tx_b < tb_stop) {
-							X->y[nx] = yb[xb_start];
-							X->x[nx] = xc;
-							X->xnode[0][nx] = tx_a;
-							X->xnode[1][nx] = tx_b;
-							nx++;
-						}
-					}
-				}
-				else {	/* General case */
-
-					i_del_xa = 1.0 / del_xa;
-					i_del_xb = 1.0 / del_xb;
-					slp_a = del_ya * i_del_xa;
-					slp_b = del_yb * i_del_xb;
-					if (slp_a != slp_b) {	/* Segments are not parallel */
-						xc = (yb[xb_start] - ya[xa_start] + slp_a * xa[xa_start] - slp_b * xb[xb_start]) / (slp_a - slp_b);
-						if (!(xc < xa[xa_start] || xc > xa[xa_stop] || xc < xb[xb_start] || xc > xb[xb_stop])) {	/* Did cross within the segment extents */
-
-							/* Only accept xover if occuring before segment end (in time) */
-
-							if (xb_start < xb_stop) {
-								tb_start = xb_start;	/* B Node first in time */
-								tb_stop = xb_stop;	/* B Node last in time */
-							}
-							else {
-								tb_start = xb_stop;	/* B Node first in time */
-								tb_stop = xb_start;	/* B Node last in time */
-							}
-							if (new_a_time) {
-								if (xa_start < xa_stop) {
-									ta_start = xa_start;	/* A Node first in time */
-									ta_stop = xa_stop;	/* A Node last in time */
-								}
-								else {
-									ta_start = xa_stop;	/* A Node first in time */
-									ta_stop = xa_start;	/* A Node last in time */
-								}
-								new_a_time = FALSE;
-							}
-
-							tx_a = ta_start + fabs (xc - xa[ta_start]) * i_del_xa;
-							tx_b = tb_start + fabs (xc - xb[tb_start]) * i_del_xb;
-							if (tx_a < ta_stop && tx_b < tb_stop) {
-								X->x[nx] = xc;
-								X->y[nx] = ya[xa_start] + (xc - xa[xa_start]) * slp_a;
-								X->xnode[0][nx] = tx_a;
-								X->xnode[1][nx] = tx_b;
-								nx++;
-							}
-						}
-					}
-				}
-
-				if (nx == nx_alloc) {
-					nx_alloc += GMT_SMALL_CHUNK;
-					x2sys_x_alloc (X, nx_alloc);
-				}
-			} /* End x-overlap */
-
-			this_b++;
-			new_b = TRUE;
-
-		} /* End y-overlap */
-
-		if (this_b == n_seg_b) {
-			this_a++;
-			this_b = (internal) ? this_a : 0;
-			new_a = new_b = TRUE;
-		}
-
-	} /* End while loop */
-
-	return (nx);
-}
-
-void x2sys_x_alloc (struct X2SYS_XOVER *X, int nx_alloc) {
-	if (nx_alloc < 0) {	/* Initial allocation */
-		nx_alloc = -nx_alloc;
-		X->x = (double *) GMT_memory (VNULL, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->y = (double *) GMT_memory (VNULL, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->xnode[0] = (double *) GMT_memory (VNULL, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->xnode[1] = (double *) GMT_memory (VNULL, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-	}
-	else {	/* Increment */
-		X->x = (double *) GMT_memory ((void *)X->x, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->y = (double *) GMT_memory ((void *)X->y, (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->xnode[0] = (double *) GMT_memory ((void *)X->xnode[0], (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-		X->xnode[1] = (double *) GMT_memory ((void *)X->xnode[1], (size_t)nx_alloc, sizeof (double), "x2sys_x_alloc");
-	}
-}
-
-void x2sys_x_free (struct X2SYS_XOVER *X) {
-	GMT_free ((void *)X->x);
-	GMT_free ((void *)X->y);
-	GMT_free ((void *)X->xnode[0]);
-	GMT_free ((void *)X->xnode[1]);
-}
-
 double *x2sys_distances (double x[], double y[], int n, int dist_flag)
 {
 	int this, prev;
@@ -887,7 +587,7 @@ double *x2sys_dummytimes (int n)
 	return (t);
 }
 
-int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p)
+int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
 {
 	/* Reads the entire contents of the file given and returns the
 	 * number of data records.  The data matrix is return in the
@@ -958,63 +658,69 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 	}
 
 	fclose (fp);
+	
+	p->ms_rec = NULL;
+	p->n_segments = 0;
 
 	*data = z;
 
 	return (p->n_rows);
 }
 
-int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p)
+int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
 {
-	int n_read, i, j, len, n_alloc = GMT_CHUNK;
-	char line[BUFSIZ];
+	int i, j, n_alloc = GMT_CHUNK;
+	char path[BUFSIZ];
 	double **z;
 	FILE *fp;
-	struct GMTMGG_REC record;
-	struct GMTMGG_TIME *gmt = NULL;
+	struct MGD77_DATA_RECORD D;
+	struct MGD77_HEADER_RECORD H;
 	double NaN;
 	
 	GMT_make_dnan(NaN);
 
-  	if ((fp = fopen (fname, "r")) == NULL) {
+  	if (n_x2sys_paths) {
+  		if (x2sys_get_data_path (path, fname)) {
+   			fprintf (stderr, "x2sys_read_mgd77file : Cannot find leg %s\n", fname);
+     			return (-1);
+  		}
+  		if ((fp = fopen (path, "r")) == NULL) {
+   			fprintf (stderr, "x2sys_read_mgd77file : Cannot find file %s\n", path);
+     			return (-1);
+  		}
+	}
+	else if ((fp = fopen (fname, "r")) == NULL) {
    		fprintf (stderr, "x2sys_read_mgd77file : Cannot find file %s\n", fname);
      		return (-1);
   	}
 
-	z = (double **) GMT_memory (VNULL, (size_t)6, sizeof (double *), "x2sys_read_mgd77file");
-	for (i = 0; i < 6; i++) z[i] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
+	if (!MGD77_Read_Header_Record (fp, &H)) {
+		fprintf (stderr, "%s: Error reading header sequence for cruise %s\n", X2SYS_program, fname);
+		exit (EXIT_FAILURE);
+	}
+	
+	z = (double **) GMT_memory (VNULL, (size_t)MGD77_DATA_COLS, sizeof (double *), "x2sys_read_mgd77file");
+	for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
 
-	j = n_read = 0;
-	while (fgets (line, BUFSIZ, fp)) {
-		n_read++;
-		if (!(line[0] == '3' || line[0] == '5')) continue;	/* Only process data records */
-		if ((len = (int)strlen(line)) != 121) {
-			fprintf (stderr, "x2sys_read_mgd77file: Record # %d has incorrect length (%d), skipped\n", n_read, len);
-			continue;
-		}
-		if (!gmtmgg_decode_MGD77 (line, FALSE, &record, &gmt)) {
-			z[0][j] = record.time;
-			z[1][j] = record.lon * MDEG2DEG;
-			z[2][j] = record.lat * MDEG2DEG;
-			z[3][j] = (record.gmt[0] == GMTMGG_NODATA) ? NaN : 0.1 * record.gmt[0];
-			z[4][j] = (record.gmt[1] == GMTMGG_NODATA) ? NaN : record.gmt[1];
-			z[5][j] = (record.gmt[2] == GMTMGG_NODATA) ? NaN : record.gmt[2];
-			j++;
-		}
-		else
-			fprintf (stderr, "x2sys_read_mgd77file: Trouble decoding record # %d (skipped)\n", n_read);
+	j = 0;
+	while (MGD77_Read_Data_Record (fp, &D)) {		/* While able to read a data record */
+		z[0][j] = D.time;
+		for (i = 1; i < MGD77_DATA_COLS; i++) z[i][j] = D.number[MGD77_items[i]];
 
+		j++;
 		if (j == n_alloc) {
 			n_alloc += GMT_CHUNK;
-			for (i = 0; i < 6; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
+			for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
 		}
 	}
 	fclose (fp);
 
 	strncpy (p->name, fname, 32);
-	p->year = gmt->first_year;
 	p->n_rows = j;
-	for (i = 0; i < 6; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)p->n_rows, sizeof (double), "x2sys_read_mgd77file");
+	for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)p->n_rows, sizeof (double), "x2sys_read_mgd77file");
+
+	p->ms_rec = NULL;
+	p->n_segments = 0;
 
 	*data = z;
 
@@ -1060,13 +766,253 @@ int x2sys_read_list (char *file, char ***list)
 	return (n);
 }
 
-void x2sys_adjust_longitudes (double *x, BOOLEAN geodetic)
+void x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, struct GMT_IO *G)
 {
-	if (geodetic) {	/* Enfore 0-360 */
-		while ((*x) < 0.0) (*x) += 360.0;
-		while ((*x) >= 360.0) (*x) -= 360.0;
-	} else {	/* enforce -180/+180 */
-		while ((*x) < -180.0) (*x) += 360.0;
-		while ((*x) >= 180.0) (*x) -= 360.0;
+	char tag_file[BUFSIZ], line[BUFSIZ], *p, *sfile;
+	int geodetic, error = 0;
+	BOOLEAN geographic = FALSE;
+	FILE *fp;
+	
+	x2sys_set_home ();
+
+	memset ((void *)B, 0, sizeof (struct X2SYS_BIX));
+	B->bin_x = B->bin_y = 1.0;
+	B->x_min = 0.0;	B->x_max = 360.0;	B->y_min = -90.0;	B->y_max = +90.0;
+	
+	sprintf (tag_file, "%s.tag", TAG);
+	if ((fp = x2sys_fopen (tag_file, "r")) == NULL) {
+		fprintf (stderr,"%s: Could not open file %s\n", X2SYS_program, tag_file);
+		exit (EXIT_FAILURE);
 	}
+	while (fgets (line, BUFSIZ, fp) && line[0] == '#');	/* Skip comment records */
+	
+	GMT_set_processed_option (8, FALSE);	/* In case -R has been processed before */
+	p = strtok (line, " \t");
+	while (p) {	/* Process the -D -I -R -G arguments from the header */
+		if (p[0] == '-') {
+			switch (p[1]) {
+				/* Common parameters */
+				case 'R':
+					error += GMT_get_common_args (p, &B->x_min, &B->x_max, &B->y_min, &B->y_max);
+					break;
+				
+				/* Supplemental parameters */
+				
+				case 'D':
+					sfile = &p[2];
+					break;
+				case 'G':	/* Geographical coordinates, set discontinuity */
+					geographic = TRUE;
+					geodetic = 0;
+					if (p[2] == 'd') geodetic = 2;
+					break;
+				case 'I':
+					if (p[2]) GMT_getinc (&p[2], &B->bin_x, &B->bin_y);
+					break;
+			}
+		}
+		p = strtok (NULL, " \t");
+	}
+	x2sys_fclose (tag_file, fp);
+
+	*s = x2sys_initialize (sfile, G);	/* Initialize X2SYS and info structure */
+
+	if (geographic) {
+		(*s)->geographic = TRUE;
+		(*s)->geodetic = geodetic;	/* Override setting */
+	}
+	x2sys_path_init (TAG);		/* Prepare directory paths to data */
+}
+
+void x2sys_bix_init (struct X2SYS_BIX *B, BOOLEAN alloc)
+{
+	B->i_bin_x = 1.0 / B->bin_x;
+	B->i_bin_y = 1.0 / B->bin_y;
+	B->nx_bin = irint ((B->x_max - B->x_min) * B->i_bin_x);
+	B->ny_bin = irint ((B->y_max - B->y_min) * B->i_bin_y);
+	B->nm_bin = B->nx_bin * B->ny_bin;
+	if (alloc) B->binflag = (unsigned int *) GMT_memory (VNULL, (size_t)B->nm_bin, sizeof (unsigned int), X2SYS_program);
+}		
+
+struct X2SYS_BIX_TRACK_INFO *x2sys_bix_make_entry (char *name, int id_no, int flag)
+{
+	struct X2SYS_BIX_TRACK_INFO *I;
+	I = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory (VNULL, (size_t)1, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
+	I->trackname = (char *) GMT_memory (VNULL, (size_t)(strlen(name)+1), sizeof (char), X2SYS_program);
+	strcpy (I->trackname, name);
+	I->track_id = id_no;
+	I->flag = flag;
+	I->next_info = NULL;
+	return (I);
+}
+
+struct X2SYS_BIX_TRACK *x2sys_bix_make_track (int id, int flag)
+{
+	struct X2SYS_BIX_TRACK *T;
+	T = (struct X2SYS_BIX_TRACK *) GMT_memory (VNULL, (size_t)1, sizeof (struct X2SYS_BIX_TRACK), X2SYS_program);
+	T->track_id = id;
+	T->track_flag = flag;
+	T->next_track = NULL;
+	return (T);
+}
+
+int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode)
+{
+	/* mode = 0 gives linked list, mode = 1 gives fixed array */
+	int id, flag, last_id = -1;
+	size_t n_alloc = GMT_CHUNK;
+	char track_file[BUFSIZ], track_path[BUFSIZ], line[BUFSIZ], name[BUFSIZ];
+	FILE *ftrack;
+	struct X2SYS_BIX_TRACK_INFO *this_info;
+	
+	sprintf (track_file, "%s_tracks.d", TAG);
+	x2sys_path (track_file, track_path);
+
+	if ((ftrack = GMT_fopen (track_path, "r")) == NULL) {
+		fprintf (stderr, "%s: Could not find %s\n", X2SYS_program, track_file);
+		exit (EXIT_FAILURE);
+	}
+
+	
+	if (mode == 1)
+		B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
+	else
+		B->head = this_info = x2sys_bix_make_entry ("-", 0, 0);
+		
+	fgets (line, BUFSIZ, ftrack);	/* Skip header record */
+	while (fgets (line, BUFSIZ, ftrack)) {
+		sscanf (line, "%s %d %d", name, &id, &flag);
+		if (mode == 1) {
+			if (id >= (int)n_alloc) {
+				n_alloc += GMT_CHUNK;
+				B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory ((void *)B->head, n_alloc, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
+			}
+			B->head[id].track_id = id;
+			B->head[id].flag = flag;
+			B->head[id].trackname = (char *) GMT_memory (VNULL, (size_t)(strlen(name)+1), sizeof (char), X2SYS_program);
+			strcpy (B->head[id].trackname, name);
+		}
+		else {
+			this_info->next_info = x2sys_bix_make_entry (name, id, flag);
+			this_info = this_info->next_info;
+		}
+		if (id > last_id) last_id = id;
+	}
+	GMT_fclose (ftrack);
+	last_id++;
+	if (mode == 1) B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory ((void *)B->head, (size_t)last_id, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
+
+	return (last_id);
+}
+
+void x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B)
+{
+	char index_file[BUFSIZ], index_path[BUFSIZ];
+	FILE *fbin;
+	int index, no_of_tracks, i, id, flag;
+	
+	sprintf (index_file, "%s_index.b",  TAG);
+	x2sys_path (index_file, index_path);
+	
+	if ((fbin = GMT_fopen (index_path, "rb")) == NULL) {
+		fprintf (stderr,"%s: Could not open %s\n", X2SYS_program, index_path);
+		exit (EXIT_FAILURE);
+	}
+
+	B->base = (struct X2SYS_BIX_DATABASE *) GMT_memory (VNULL, (size_t)B->nm_bin, sizeof (struct X2SYS_BIX_DATABASE), X2SYS_program);
+
+	while ((fread ((void *)(&index), sizeof (int), (size_t)1, fbin)) == 1) {
+		fread ((void *)(&no_of_tracks), sizeof (int), (size_t)1, fbin);
+		B->base[index].first_track = B->base[index].last_track = x2sys_bix_make_track (0, 0);
+		for (i = 0; i < no_of_tracks; i++) {
+			fread ((void *)(&id), sizeof (int), (size_t)1, fbin);
+			fread ((void *)(&flag), sizeof (int), (size_t)1, fbin);
+			B->base[index].last_track->next_track = x2sys_bix_make_track (id, flag);
+			B->base[index].last_track = B->base[index].last_track->next_track;
+			B->base[index].n_tracks++;
+		}
+	}
+	GMT_fclose (fbin);
+}
+
+int x2sys_bix_get_ij (double x, double y, int *i, int *j, struct X2SYS_BIX *B)
+{
+	int index;
+	
+	*j = (y == B->y_max) ? B->ny_bin - 1 : (int)floor ((y - B->y_min) * B->i_bin_y);
+	*i = (x == B->x_max) ? B->nx_bin - 1 : (int)floor ((x - B->x_min)  * B->i_bin_x);
+	index = (*j) * B->nx_bin + (*i);
+	if (index >= B->nm_bin) {
+		fprintf (stderr, "x2sys_binlist: Index (%d) outside range implied by -R -I! [0-%d>\n", index, B->nm_bin);
+		exit (EXIT_FAILURE);
+	}
+	
+	return (index);
+}
+
+/* gmtmggpath_init reads the SHAREDIR/share/mgg/gmtfile_paths file and gets all
+ * the gmtfile directories.
+ */
+ 
+void x2sys_path_init (char *TAG)
+{
+	int i;
+	char file[BUFSIZ], line[BUFSIZ];
+	FILE *fp;
+	
+	x2sys_set_home ();
+
+	sprintf (file, "%s%c%s_paths.txt", X2SYS_HOME, DIR_DELIM, TAG);
+	
+	n_x2sys_paths = 0;
+
+	if ((fp = fopen (file, "r")) == NULL) {
+		fprintf (stderr, "%s: Warning: path file %s for %s files not found\n", X2SYS_program, file, TAG);
+		fprintf (stderr, "%s: (Will only look in current directory for such files)\n", X2SYS_program);
+		return;
+	}
+	
+	while (fgets (line, BUFSIZ, fp) && n_x2sys_paths < MAX_DATA_PATHS) {
+		if (line[0] == '#') continue;	/* Comments */
+		if (line[0] == ' ' || line[0] == '\0') continue;	/* Blank line */
+		x2sys_datadir[n_x2sys_paths] = GMT_memory (VNULL, (size_t)1, (size_t)(strlen (line)), "x2sys_path_init");
+		line[strlen (line)-1] = 0;
+#if _WIN32
+		for (i = 0; line[i]; i++) if (line[i] == '/') line[i] = DIR_DELIM;
+#else
+		for (i = 0; line[i]; i++) if (line[i] == '\\') line[i] = DIR_DELIM;
+#endif
+		strcpy (x2sys_datadir[n_x2sys_paths], line);
+		n_x2sys_paths++;
+		if (n_x2sys_paths == MAX_DATA_PATHS) fprintf (stderr, "%s: Reached maximum directory (%d) count in %s!\n", X2SYS_program, MAX_DATA_PATHS, file);
+	}
+	fclose (fp);
+}
+	
+/* x2sys_get_data_path takes a track name as argument and returns the full path
+ * to where this data file can be found.  x2sys_path_init must be called first.
+ */
+ 
+int x2sys_get_data_path (char *track_path, char *track)
+{
+	int id;
+	char geo_path[BUFSIZ];
+	
+	/* First look in current directory */
+	
+	if (!access(track, R_OK)) {
+		strcpy(track_path, track);
+		return (0);
+	}
+	
+	/* Then look elsewhere */
+	
+	for (id = 0; id < n_x2sys_paths; id++) {
+		sprintf (geo_path, "%s%c%s", x2sys_datadir[id], DIR_DELIM, track);
+		if (!access (geo_path, R_OK)) {
+			strcpy (track_path, geo_path);
+			return (0);
+		}
+	}
+	return(1);
 }
