@@ -1,4 +1,4 @@
-/*	$Id: x_update.c,v 1.2.4.1 2002/02/27 17:41:10 pwessel Exp $
+/*	$Id: x_update.c,v 1.2 2001/04/04 00:59:32 pwessel Exp $
  *
  * XUPDATE will read a xover.d-file that contains a series of crossovers. The first
  * record contains leg1 year1 leg2 year2, and the next n records has all the
@@ -198,7 +198,7 @@ main (int argc, char *argv[])
 
 		/* Copy this info to xx_base.b */
 
-		sprintf (header,"%s %s %10d", lega, legb, n_x);
+		sprintf (header,"%s %s %10d\0", lega, legb, n_x);
 		if (fwrite ((void *)header, REC_SIZE, (size_t)1, fxb) != (size_t)1) {
 			fprintf(stderr,"x_update : Write header error at rec %d\n", n_rec);
 			exit (EXIT_FAILURE);
@@ -261,7 +261,7 @@ main (int argc, char *argv[])
 	/* Write first rec for next time x_update is used */
 
 	fseek (fxb, 0L, SEEK_SET);
-	sprintf(header,"%10d xx_base.b header",n_rec);
+	sprintf(header,"%10d xx_base.b header\0",n_rec);
 	if (fwrite((void *)header,REC_SIZE, (size_t)1, fxb) != (size_t)1) {
 		fprintf(stderr,"x_update : Write header error for n_rec = %d\n",n_rec);
 		exit (EXIT_FAILURE);
